/***********************************************************************
 * AgentPlanLogger.hh
 * Copyright (c) 2003 Virtual Werder
 *
 * implements:
 *   class AgentPlanLogger
 *
 * requires:
 *
 *
 * Created on: 17.12.2003 by aw
 * Maintainer: aw, pagra
 * Group:
 * Co-authors: pagra
 *
 * Changelog:
 *
 * $Id: AgentPlanLogger.hh,v 1.2 2004/03/18 15:17:54 khuebner Exp $
 *
 **********************************************************************/

#ifndef AGENTPLANLOGGER_HH
#define AGENTPLANLOGGER_HH

#include "Geometry.hh"
#include "common.hh"
#include "ConfigFile.hh"
#include <fstream>
#include <sstream>
#include <vector>

/**********************************************************************
 * This class logs some agent-data in SoccerScope2 AgentPlan-format.
 * The outputfile is a textfile
 * position: the actual position (in sscope2: circle)
 * bodydirection (in sscope2: line)
 * headdirection (in sscope2: shorter line
 *********************************************************************/

// forward declarations
class WorldModel;
class Myself;

class AgentPlanLogger {
private:

  WorldModel *wm;
  Myself *ms;
  ofstream out;
  int uNum;
  void openFile(void);

public:
  static AgentPlanLogger *apl;

  AgentPlanLogger(WorldModel *wm);

  // LowLevel
  void writePoint( const Vector2D &point,
		   APLLevel level=APL_L1,
		   int r=255, int g=255, int b=255,
		   int uNum=-1 );
  void writeLine( const Vector2D &p1, const Vector2D &p2,
		  APLLevel level=APL_L1,
		  int r=255, int g=255, int b=255,
		  int uNum=-1 );
  void writePolyline( vector<Vector2D> points,
		      APLLevel level=APL_L1,
		      int r=255, int g=255, int b=255,
		      int uNum=-1 );
  void writeEllipse( const Vector2D &upperLeft,
		     float width =2, float height=2,
		     APLLevel level=APL_L1,
		     int r=255, int g=255, int b=255,
		     int uNum=-1 );
 
  void writeNumber( std::string number,
		    const Vector2D &upperLeft,
		    float width = .5, float height=2,
		    APLLevel level=APL_L1,
		    int r=255, int g=255, int b=255);

  void writeDigit(  int digit,
		    const Vector2D &upperLeft,
		    float width = .5, float height=2,
		    APLLevel level=APL_L1,
		    int r=255, int g=255, int b=255);

  // HighLevel
  void writeBall( APLLevel level=APL_L1, float conf=-1 );
  void writeInterceptionPoint( APLLevel level=APL_L1, float conf=-1 );
  void writeAllPlayerPositions( APLLevel level=APL_L1, float conf=-1 );
  



  void writeAgentPlan();
};

#endif // AGENTPLANLOGGER_HH
