/***********************************************************************
 * Communicator.cc
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   class Communicator
 *
 * requires:
 *
 * Created on: 25.03.2001 by wenns
 * Maintainer: wenns
 * Group:
 * Co-authors: cyco
 *
 * Changelog:
 *
 * $Id: Communicator.cc,v 1.2 2004/03/30 16:11:21 harlekin Exp $
 *
 **********************************************************************/

#include <stdio.h>
#include <string.h>

#include "Communicator.hh"
#include "debug.hh"

int                Communicator::ownSocket = -1;
struct sockaddr_in Communicator::hostAddress;
int                Communicator::lengthOfHostAddress;


Communicator::Communicator(const char *hostName, int port){
  openConnection(hostName, port);
}


Communicator::~Communicator(){
  closeConnection();
}


void Communicator::sendMessage(const char *msg){
  DEBUG3("sending msg: \"", msg,"\"");
  //strlen()+1 because of the null terminator
  if (sendto(ownSocket, msg, strlen(msg)+1, 0,
             (struct sockaddr *) &hostAddress, sizeof(hostAddress))
      != (signed int)(strlen(msg)+1) )
    perror("sending failed: ");
}


void Communicator::getNextMessage(char *msg) const {
  static int lengthOfMessage;

  DEBUG1_SYNCHRO("[SENSE] \tgetNextMessage(): going into the recvfrom... " );

  if( (lengthOfMessage = recvfrom(ownSocket, msg, MAX_MSG-1, 0,
                                  (struct sockaddr *) &hostAddress,
                                  (socklen_t *)&lengthOfHostAddress))
    > 0 ){
    msg[lengthOfMessage] = '\0';
    //DEBUG1(msg);
  }else{
    perror("reading failed: ");
    _exit(-1);
  }
}


void Communicator::openConnection(const char *host, int port){
  struct hostent* hostEnvironment;
  struct in_addr* hostInternetAddress;
  struct sockaddr_in clientAddress;
  int socketID;
  unsigned int bufferSize = BUFFER_SIZE;

  // find the host
  if ((hostEnvironment = (struct hostent*) gethostbyname(host)) == NULL) {
    if (((long) inet_addr(host)) == -1) {
      perror("Can't find the host: ");
      _exit(-1);
    }
  }
  else {
    hostInternetAddress = (struct in_addr*) *hostEnvironment->h_addr_list;
    host = inet_ntoa (*hostInternetAddress);
  }

  // get the socket
  if ((socketID = socket(AF_INET, SOCK_DGRAM, 0)) < 0) {
    perror("Initialization of socket failed: ");
    _exit(-1);
  }
  // set the socket in the needed mode
  if ((setsockopt (socketID, SOL_SOCKET, SO_RCVBUF,  (char*) &bufferSize,
		    sizeof (int))) < 0) {
    perror("Initialization of socket failed: ");
    _exit(-1);
  }

  // bind the socket
  bzero((char *) &clientAddress, sizeof(clientAddress));
  clientAddress.sin_family = AF_INET;
  clientAddress.sin_addr.s_addr = htonl(INADDR_ANY);
  clientAddress.sin_port = htons(0);
  if (bind(socketID, (struct sockaddr *) &clientAddress,
	    sizeof(clientAddress)) < 0) {
    perror("Binding of socket failed: ");
    _exit(-1);
  }

  ownSocket = socketID;

  bzero((char *) &hostAddress, sizeof(hostAddress));
  hostAddress.sin_family = AF_INET;
  hostAddress.sin_addr.s_addr = inet_addr(host);
  hostAddress.sin_port = htons(port);

  lengthOfHostAddress = sizeof(hostAddress);
}


void Communicator::closeConnection() {
    close(ownSocket);
}

// $Source: /home/robocup/virtualwerder/CVSRepository/VirtualWerder2004/Communicator.cc,v $
