/***********************************************************************
 * Communicator.hh
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   class Communicator
 *
 * requires:
 *   varios network system-headers
 *
 * Created on: 25.03.2001 by wenns
 * Maintainer: wenns
 * Group:
 * Co-authors: cyco
 *
 * Changelog:
 *
 * $Id*
 *
 **********************************************************************/

#ifndef COMMUNICATOR_HH
#define COMMUNICATOR_HH

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>

#include "common.hh"
#include "ServerCommands.hh"  //SS_MSG_MAX_SIZE


/**
 * The UDP/IP-level caretaker.
 * The Communicator cares for the communikation with the soccerserver on
 * UDP/IP level. It sends strings to the soccerserver and reads strings from
 * the server. The reading is blocking.
 */
class Communicator{
  public:
   /**
    * Sets up a UDP/IP "connection".
    * @param hostName The hame of the host running soccerserver.
    * @param port The port, on which the soccerserver listens.
    */
    Communicator(const char *hostName, int port);

    /**
     * Closes the "connection" to the soccerserver.
     */
    ~Communicator();

    /**
     * Sends the string msg to the soccer server
     * @param msg the message to send
     */
    void sendMessage(const char *msg);

    /**
     * Receives the next message from the soccer server. This function blocks,
     * if there is no message on the socket.
     * @param msg the buffer to store the message into.
     */
    void getNextMessage(char *msg) const;

  private:
    //Constants
    /** maximal size of an incoming soccer server message */
    static const int MAX_MSG       = SS_MSG_MAX_SIZE;
    /** the buffer size of the used UDP-socket */
    static const int BUFFER_SIZE   = 64000;

    /** the file descriptor of the used UDP-socket */
    static int                ownSocket;
    /** internet address of the soccer server */
    static struct sockaddr_in hostAddress;
    static int                lengthOfHostAddress;

    static void openConnection(const char *host, int port);
    static void closeConnection();
};

// $Source: /home/robocup/virtualwerder/CVSRepository/VirtualWerder2004/Communicator.hh,v $
#endif // COMMUNICATOR_HH
