/**
 * TODO:
 *  - Eigene und "feindliche" Spieler in der unteren Tabelle besser unterscheiden (Icon vor die UNUM? Farblich hinterlegen?)
 *  - Weitere Daten rechts darstellen. Momentan noch die Optionen fr Formationen
 *  - XML-Parser um weitere Informationen erweitern (PlayerTypes)
 *  - "Play"-Option: Automatischer Durchlauf durch die Cycles (siehe neue Verlinkung)
 *  - XML-Parser dermaen anpassen, da er auch mit Zwischencycles arbeiten kann (siehe ncshten Punkt (Weg von der indizierung per Cycle-Nummer))
 *  - Durchlaufen der Cycles erleichtern. Fr den Fall, da mal ein Cycle fehlt (Doppelt verlinkte Liste (WorldModel?)?)
 *  - Anzeige der Rotation des Spielers (und des Necks) kontrollieren. von -180 zu +180 (momentan) oder +180 zu -180 ???
 *  - Anzeige der Geschwindigkeit
 *
 **/

import javax.swing.*;
import javax.swing.table.*;
import javax.swing.border.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import javax.swing.table.DefaultTableCellRenderer;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.LinkedList;

public class FormationEditor extends JFrame implements Serializable{
    final double VERSION = 0.80;

    // Constants used for graphical output (accessed by Field,Player?,Vector2D,FieldMouseInputAdapter)
    static final double BALL_SIZE = 0.6;
    static final double KICKABLE_MARGIN =  0.7;
    static final double PLAYER_SIZE  =     0.3;
    static final double PLAYER_SPEED =     1.0;
    static final double GOAL_WIDTH   =   2.75;
    static final double GOAL_LENGTH  =  14.02;
    static final double CIRCLE_SIZE  =   9.15;
    static final double PITCH_WIDTH  =  68.0;
    static final double PITCH_LENGTH = 105.0;
    static final double PENALTY_AREA_LENGTH = 16.5;
    static final double PENALTY_AREA_WIDTH  = 40.32;
    static final double PENALTY_X = PITCH_LENGTH/2.0 - PENALTY_AREA_LENGTH;
    static final double PENALTY_Y = PITCH_WIDTH/2.0 - PENALTY_AREA_WIDTH;
    static final double PITCH_MARGIN = 42;

    static boolean usingJelle = false;

    // Comments which will be added to the output-text/conf-file
    // % is similar to 'new line'. 
    public String getInformation(){
	return "# Formation-File%"+
	    "# Created with FormationEditor v"+ VERSION +"%"+
	    "#   Author: Philippe Maurice Schober%"+
	    "#   Co-Authors: %"+
	    "#%"+ 
	    "#%"+ 
	    "#%"+ 
	    "# We have the following player types:%"+
	    "# -----------------------------------%"+
	    "# ...Behaviour is called%"+
	    "#  1 = Goalie%"+
	    "#  2 = WingDefender (left)%"+
	    "#  3 = CenterBack (left)%"+
	    "#  4 = CenterBack (right)%"+
	    "#  5 = WingDefender (right)%"+
	    "#  6 = WingMidfielder (left)%"+
	    "#  7 = CenterMidfielder%"+
	    "#  8 = WingMidfielder (right)%"+
	    "#  9 = WingAttacker (left)%"+
	    "# 10 = CenterAttacker%"+
	    "# 11 = WingAttacker (right)%"+
	    "###%"+
	    "# Layout of information:%"+
	    "#------%"+
	    "# Formation ID%"+
	    "# X-HomePos of 11 players%"+
	    "# Y-HomePos of 11 players%"+
	    "# X-InitPos of 11 players%"+
	    "# Y-InitPos of 11 players%"+
	    "# PlayerBehaviour (see above) of 11 players%"+
	    "# X-Attraction factors%"+
	    "# Y-Attraction factors%"+
	    "# Stay behind ball..%"+
	    "# Minimal X-Coordinate of 11 players%"+
	    "# Maximal X-Coordinate of 11 players%"+
	    "####%";	    
    }

    public String getInformationUvA(){
	return "# Formation-File%"+
	    "# Created with FormationEditor v"+ VERSION +"%"+
	    "#   Author: Philippe Maurice Schober%"+
	    "#   Co-Authors: %"+
	    "#%"+ 
	    "#%"+ 
	    "#%"+ 
	    "# We have the following player types:%"+
	    "# -----------------------------------%"+
	    "# %"+
	    "#  0 = PT_UNKNOWN%"+
	    "#  1 = PT_GOALKEEPER%"+
	    "#  2 = PT_DEFENDER_CENTRAL%"+
	    "#  3 = PT_DEFENDER_SWEEPER%"+
	    "#  4 = PT_DEFENDER_WING%"+
	    "#  5 = PT_MIDFIELDER_CENTER%"+
	    "#  6 = PT_MIDFIELDER_WING%"+
	    "#  7 = PT_ATTACKER_WING%"+
	    "#  8 = PT_ATTACKER_CENTRAL%"+
	    "###%"+
	    "# Layout of Information:%"+
	    "#------%"+
	    "# #Formation Name%"+
	    "# Formation ID%"+
	    "# X-HomePos of 11 players%"+
	    "# Y-HomePos of 11 players%"+
	    "# PlayerBehaviour (see above) of 11 players%"+
	    "# X-Attraction factors of player types%"+
	    "# Y-Attraction factors of player types%"+
	    "# Stay behind ball for player types%"+
	    "# Minimal X-Coordinate of player types%"+
	    "# Maximal X-Coordinate of player types%"+
	    "####%";	    
    }
    
    // PopUp-Dialog asking for Filename
    public File getLoadFileName(int method){
	JFileChooser chooser = new JFileChooser(".");
	if (method == 0)
	    chooser.setFileFilter(new FormFileFilter());	    
	if (method == 1)
	    chooser.setFileFilter(new ConfFileFilter());
	if (method == 2)
	    chooser.setFileFilter(new XMLFileFilter());
	int returnVal = chooser.showOpenDialog(this);
	if(returnVal == JFileChooser.APPROVE_OPTION) {
	    return chooser.getSelectedFile();
	}
	return null;
    }

    // PopUp-Dialog asking for Filename
    public File getSaveFileName(boolean conf){
	JFileChooser chooser = new JFileChooser(".");
	if (!conf)
	    chooser.setFileFilter(new FormFileFilter());	    
	else
	    chooser.setFileFilter(new ConfFileFilter());
	int returnVal = chooser.showSaveDialog(this);
	if(returnVal == JFileChooser.APPROVE_OPTION) {
	    File file = chooser.getSelectedFile();
	    if (!conf)
		if (!((FormFileFilter)chooser.getFileFilter()).accept(file.getName()))
		    file = new File(file.getPath() +".form");
	    if (conf)
		if (!((ConfFileFilter)chooser.getFileFilter()).accept(file.getName()))
		    file = new File(file.getPath() +".conf");
	    return file;
	}
	return null;
    }

    // Saves a whole set of Formations to a .conf-(Text)-File
    public void saveAs(){
	try{
	    File file = getSaveFileName(true);
	    if (file == null) return;
	    if (file.getName() == null) return;
	    PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(file)));
	    String info = getInformation();
	    String inf[] = info.split("%");
	    for (int j = 0; j < inf.length; j++){
		out.println(inf[j]);
	    }
	    for (int i = 0; i < formations.size(); i++){
		Formation tW = (Formation)formations.get(i);
		out.println("####"+ tW.name);
		out.println(tW.id);
		out.println(tW.getHomeXText());
		out.println(tW.getHomeYText());
		out.println(tW.getInitXText());
		out.println(tW.getInitYText());
		out.println(tW.getBehaveText());
		out.println(tW.getAttrXText());
		out.println(tW.getAttrYText());
		out.println(tW.getBeBaText());
		out.println(tW.getMinXText());
		out.println(tW.getMaxXText());

	    }
	    out.close();
	}catch(Exception e){ e.printStackTrace(); }
	    
    }

    public void loadWorldModel(){
	try{
	    String fileName = getLoadFileName(2).getName();
	    XMLParser parser = new XMLParser();
	    parser.start(fileName);	
	    firstWorldModel = parser.firstWorldModel;
	    activeWorldModel = firstWorldModel;
	    activeWorldModelUNum = parser.unum;
	    mode = "wm";
	    parser = null;
	    cycleEdit.setEnabled(true);
	    cycleEdit2.setEnabled(true);
	    cycleLeft.setEnabled(false);
	    cycleRight.setEnabled(true);
	    formlist.setEnabled(false);
	    formlist2.setEnabled(false);
	    print.setEnabled(false);
	    showLimits.setEnabled(false);
	    pong.setEnabled(false);
	    s_export.setEnabled(false);
	    l_import.setEnabled(false);
	    copy.setEnabled(false);
	    create.setEnabled(false);
	    delete.setEnabled(false);
	    ul_wm.setEnabled(true);
	    play.setEnabled(true);
	    mstats.setEnabled(true);

	    sHP.setEnabled(false);
	    tf_id.setEnabled(false);
	    tf_name.setEnabled(false);

	    cycleEdit.setText(""+ activeWorldModel.cycle);
	    cycleEdit2.setText(""+ activeWorldModel.stoppedCycle);
	    jsp2.setViewportView(wmTable);
	    jsp2.setRowHeaderView(rowHeader2);
	    bottom.repaint();
	    field.repaint();
	}catch(Exception e){ }
    }

    public void unloadWorldModel(){
	//worldModels = null;
	firstWorldModel = null;
	activeWorldModel = null;
	activeWorldModelUNum = 0;
	mode = "init";
	cycleEdit.setEnabled(false);
	cycleEdit2.setEnabled(false);
	cycleLeft.setEnabled(false);
	cycleRight.setEnabled(false);
	formlist.setEnabled(true);
	formlist2.setEnabled(true);
	print.setEnabled(true);
	showLimits.setEnabled(true);
	pong.setEnabled(true);
	pong.setEnabled(true);
	s_export.setEnabled(true);
	l_import.setEnabled(true);
	copy.setEnabled(true);
	create.setEnabled(true);
	delete.setEnabled(true);
	ul_wm.setEnabled(false);
	play.setEnabled(false);
	mstats.setEnabled(false);
	if (stats != null){
	    stats.dispose();
	    stats = null;
	}
	if (pl != null){
	    pl.destroy();
	    pl = null;
	}
	sHP.setEnabled(true);
	tf_id.setEnabled(true);
	tf_name.setEnabled(true);
	cycleEdit.setText("");
	cycleEdit2.setText("");
	jsp2.setViewportView(jt);
	jsp2.setRowHeaderView(rowHeader);
	bottom.repaint();
	field.repaint();
    }

    // Loads a whole set of Formations from a .conf-(Text)-File
    public void loadFrom(){
	usingJelle = false;
	try {
	    File file = getLoadFileName(1);
	    if (file == null) return;
	    if (file.getName() == null) return;
	    BufferedReader in = new BufferedReader(new FileReader(file));	    
	    formations = new LinkedList();
	    Formation nf = new Formation();
	    String s, sn = "";	
	    String xn[];
	    int pos = 0;
	    while((s = in.readLine()) != null){
		if (s.trim().length() == 0) continue;
		if (s.charAt(0) == '#') {
		    sn = s.trim();
		    continue;
		}
		switch(pos){
		case 0:
		    nf = new Formation();
		    nf.name = sn.replaceAll("#","").trim();
		    nf.id = Integer.parseInt(s);
		    break;
		case 1: // Home - X
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].homePosition.setX(Double.parseDouble(xn[j]));
		    }		    
		    break;
		case 2: // Home - Y
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].homePosition.setY(Double.parseDouble(xn[j]));
		    }		    
		    break;
		case 3: // Init - X
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].initPosition.setX(Double.parseDouble(xn[j]));
		    }		    
		    break;
		case 4: // Init - Y
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].initPosition.setY(Double.parseDouble(xn[j]));
		    }
		    break;		    
		case 5: // Behaviour
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].type = Integer.parseInt(xn[j]) - 1;
		    }
		    break;
		case 6: // Attr - X
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].attrX = Double.parseDouble(xn[j]);
		    }
		    break;
		case 7: // Attr - Y
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].attrY = Double.parseDouble(xn[j]);
		    }
		    break;		    
		case 8: // BehindBall
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].behindBall = (Integer.parseInt(xn[j]) == 1 ? true : false);
		    }
		    break;		    
		case 9: // minX
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].minX = Double.parseDouble(xn[j]);
		    }
		    break;
		case 10: // maxX
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].maxX = Double.parseDouble(xn[j]);
		    }
		    pos = -1;
		    formations.add(nf);
		    break;		    
		}
		pos++;
	    }
	    in.close();
	    aF = (Formation)formations.getFirst();
	    updateFormationMenu();
	    for (int i = 0; i < 11; i++){
		sB[i].removeAllItems();
		for(int k = 0; k < 11; k++)
		    sB[i].addItem(behaviour[k]);
		sB[i].setSelectedIndex(aF.player[i].type);
	    }
	    playerButtons[0].doClick();
	    playerButtons[0].setSelected(true);	
	    sHP.setEnabled(true);
	    pong.setEnabled(true);
	    bottom.repaint();
	    field.repaint();
	}
	catch (Exception e) { e.printStackTrace(); }
    }

    // Saves a whole set of Formations to a .conf-(Text)-File
    public void saveAsUvA(){
	try{
	    File file = getSaveFileName(true);
	    if (file == null) return;
	    if (file.getName() == null) return;
	    PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(file)));
	    String info = getInformationUvA();
	    String inf[] = info.split("%");
	    for (int j = 0; j < inf.length; j++){
		out.println(inf[j]);
	    }
	    for (int i = 0; i < formations.size(); i++){
		Formation tW = (Formation)formations.get(i);
		out.println("# "+ tW.name);
		out.println(tW.id);
		out.println(tW.getHomeXText());
		out.println(tW.getHomeYText());
		out.println(tW.getBehaveTextUvA());
		out.println(tW.getAttrXTextUvA());
		out.println(tW.getAttrYTextUvA());
		out.println(tW.getBeBaTextUvA());
		out.println(tW.getMinXTextUvA());
		out.println(tW.getMaxXTextUvA());
	    }
	    out.close();
	}catch(Exception e){ e.printStackTrace(); }
	    
    }

    // Loads a whole set of Formations from a .conf-(Text)-File
    public void loadFromUvA(){
	usingJelle = true;
	try {
	    File file = getLoadFileName(1);
	    if (file == null) return;
	    if (file.getName() == null) return;
	    BufferedReader in = new BufferedReader(new FileReader(file));	    
	    formations = new LinkedList();
	    Formation nf = new Formation();

	    Player pts[] = new Player[9];
	    for (int i = 0; i < 9; i++)
		pts[i] = new Player();

	    String s, sn = "";	
	    String xn[];
	    int pos = 0;
	    while((s = in.readLine()) != null){
		if (s.trim().length() == 0) continue;
		if (s.charAt(0) == '#') {
		    sn = s.trim();
		    continue;
		}
		switch(pos){
		case 0:
		    nf = new Formation();
		    nf.name = sn.replaceAll("#","").trim();
		    nf.id = Integer.parseInt(s);
		    break;
		case 1: // Home - X
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].homePosition.setX(Double.parseDouble(xn[j]));
		    }		    
		    break;
		case 2: // Home - Y
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].homePosition.setY(Double.parseDouble(xn[j]));
		    }		    
		    break;
		case 3: // Behaviour
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 11; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			nf.player[i].type = Integer.parseInt(xn[j]);
		    }
		    break;
		case 4: // Attr - X
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 9; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			if (xn[j].charAt(0) == '#') continue;
			pts[i].attrX = Double.parseDouble(xn[j]);
		    }
		    break;
		case 5: // Attr - Y
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 9; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			if (xn[j].charAt(0) == '#') continue;
			pts[i].attrY = Double.parseDouble(xn[j]);
		    }
		    break;		    
		case 6: // BehindBall
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 9; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			if (xn[j].charAt(0) == '#') continue;
			pts[i].behindBall = (Integer.parseInt(xn[j]) == 1 ? true : false);
		    }
		    break;		    
		case 7: // minX
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 9; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			if (xn[j].charAt(0) == '#') continue;
			pts[i].minX = Double.parseDouble(xn[j]);
		    }
		    break;
		case 8: // maxX
		    xn = s.trim().split("[\\s]");
		    for(int i = 0, j = 0; i < 9; i++, j++){
			if (xn[j].trim().length() == 0){ i--; continue; }
			if (xn[j].charAt(0) == '#') continue;
			pts[i].maxX = Double.parseDouble(xn[j]);
		    }
		    pos = -1;
		    for (int i = 0; i < 11; i++){
			nf.player[i].attrX = pts[nf.player[i].type].attrX;
			nf.player[i].attrY = pts[nf.player[i].type].attrY;
			nf.player[i].behindBall = pts[nf.player[i].type].behindBall;
			nf.player[i].minX = pts[nf.player[i].type].minX;
			nf.player[i].maxX = pts[nf.player[i].type].maxX;
		    }		    
		    formations.add(nf);
		    break;	    
		}
		pos++;
	    }
	    in.close();
	    aF = (Formation)formations.getFirst();
	    updateFormationMenu();
	    for (int i = 0; i < 11; i++){
		sB[i].removeAllItems();
		for(int k = 0; k < 9; k++)
		    sB[i].addItem(behaviourUvA[k]);
		sB[i].setSelectedIndex(aF.player[i].type);
	    }
	    mode = "home";
	    sHP.setText("Home");
	    sHP.setEnabled(false);
	    pong.setEnabled(false);
	    playerButtons[0].doClick();
	    playerButtons[0].setSelected(true);	
	    bottom.repaint();
	    field.repaint();
	}
	catch (Exception e) { e.printStackTrace(); }
    }



    // Saves the LinkedList which contains the player-informations as a serialized object. Whatever...
    // (shouldnt be used since the objects may change and old data will be useless)
    public void save(){
	try {
	    File file = getSaveFileName(false);
	    if (file == null) return;
	    FileOutputStream fout = new FileOutputStream(file);
	    ObjectOutputStream oos = new ObjectOutputStream(fout);
	    oos.writeObject(formations);
	    oos.close();
	}
	catch (Exception e) { e.printStackTrace(); }
    }
    // Loades a LinkedList which contains the player-informations from a serialized object. Whatever...
    // (shouldnt be used since the objects may change and old data will be useless)
    public void load(){
	try {
	    File file = getLoadFileName(0);
	    if (file == null) return;
	    FileInputStream fin = new FileInputStream(file);
	    ObjectInputStream ois = new ObjectInputStream(fin);
	    LinkedList o = (LinkedList)ois.readObject();
	    ois.close();
	    formations = o;	
	    aF = (Formation)formations.getFirst();
	    for (int i = 0; i < 11; i++){
		sB[i].setSelectedIndex(aF.player[i].type);
	    }
	    updateFormationMenu();
	    playerButtons[0].doClick();
	    playerButtons[0].setSelected(true);	
	    bottom.repaint();
	}
	catch (Exception e) { e.printStackTrace(); }
    }

    // Static methods for controlling a Vector. "Is the player on the field?"
    public static boolean isInvalidVector2D(Vector2D v){
	return (Math.abs(v.x) > PITCH_LENGTH/2 ||
		Math.abs(v.y) > PITCH_WIDTH /2);
    }
    public static boolean isInvalidVector2DForInit(Vector2D v){
	return (         v.x  > 0 ||
			 Math.abs(v.y) > PITCH_WIDTH /2);
    }

    //public LinkedList worldModels;
    public WorldModel firstWorldModel;
    public WorldModel activeWorldModel;
    public int activeWorldModelUNum;
    public JButton cycleLeft, cycleRight;
    public JTextField cycleEdit, cycleEdit2;

    public LinkedList formations = new LinkedList();
    public MenuItemActionListener mial = new MenuItemActionListener();
    public Formation aF, sF;
    public Vector2D ballPos = new Vector2D();
    public String toChange="pos";
    public Field field;
    public JPanel main;
    public JPanel right, top, bottom;
    public JToggleButton playerButtons[], minXButtons[], maxXButtons[];
    public JButton sHP;
    public JButton play;
    public JButton jb_rotateView;
    public boolean rotateView;
    public int activePlayer = 0;
    public double  gridSize = 0.5;
    public String mode = "init"; // "home", "view", "ball"
    public Dimension tBD = new Dimension(130, 20);
    public JTextField tf_id, tf_name; 
    public JMenuBar menu = new JMenuBar();
    public JMenu file, options, graphics, rendering, formlist, formlist2;
    public JMenuItem save,copy,delete,s_export,l_import,l_wm,ul_wm,load,quit, zoomin,zoomout,gridp,gridm,print,create,mstats;
    public JMenuItem j_import, j_export;
    public JCheckBoxMenuItem antialias, tgrid, antialiasText, showLimits, pong;
    public ButtonGroup r_bg, f_bg, f_bg2;
    public JRadioButtonMenuItem r_quality, r_speed;
    public String behaviour[] = new String[]{"Goalie", 
					     "Wing Defender (left)",
                                             "Center Back Defender (left)",
					     "Center Back Defender (right)",
                                             "Wing Defender (right)",
                                             "Wing Midfielder (left)",
                                             "Center Midfielder",
					     "Wing Midfielder (right)",
                                             "Wing Attacker (left)",
                                             "Center Attacker",
                                             "Wing Attacker (right)"};
    public String behaviourUvA[] = new String[]{"PT_UNKNOWN", 
						"PT_GOALKEEPER",
						"PT_DEFENDER_CENTRAL",
						"PT_DEFENDER_SWEEPER",
						"PT_DEFENDER_WING",
						"PT_MIDFIELDER_CENTER",
						"PT_MIDFIELDER_WING",
						"PT_ATTACKER_WING",
						"PT_ATTACKER_CENTRAL"};
    public JComboBox sB[] = new JComboBox[11];
    public Pong p;
    public Play pl;

    static double ZOOM = 6; //6;

    // Output-Table
    JTable jt, wmTable; 
    
    public Dimension fieldSize = new Dimension((int)(ZOOM*(PITCH_LENGTH) + PITCH_MARGIN*2),
					       (int)(ZOOM*(PITCH_WIDTH ) + PITCH_MARGIN*2));
    public Dimension scrollSize = new Dimension((int)(6*(PITCH_LENGTH) + PITCH_MARGIN*2+5),
						(int)(6*(PITCH_WIDTH ) + PITCH_MARGIN*2+5));
    public Dimension rightSize = new Dimension(450, (int)fieldSize.getHeight());
    public Dimension topSize = new Dimension((int)fieldSize.getWidth() + 400, 40);
    public JScrollPane jsp;
    public TableModel dataModel  = new PlayerInfoModel();
    public TableModel dataModel2 = new WorldModelTableModel();
    public static boolean al = false;
    public BorderLayout bl = new BorderLayout();
    public GridLayout gl = new GridLayout(20,2);
    public int pressedKeyCodeOnField = -1;

    public ListModel lm = new FormationTableRowHeaderListModel();
    public ListModel lm2 = new WorldModelTableRowHeaderListModel();
    public JList rowHeader = new JList(lm);  
    public JList rowHeader2 = new JList(lm2);
    public JScrollPane jsp2;

    public Statistics stats;

    // Updates the Formation-Menu
    // Will be called after loading or when a new formation has be created
    public void updateFormationMenu(){
	formlist.removeAll();
	formlist2.removeAll();
	f_bg = new ButtonGroup();
	f_bg2 = new ButtonGroup();
	
	JRadioButtonMenuItem temp = new JRadioButtonMenuItem("none");
	temp.addActionListener(new FormationButton2Listener(null));
	formlist2.add(temp);
	f_bg2.add(temp);
	if (sF == null){ temp.setSelected(true); }
	for (int i = 0; i < formations.size(); i++){
	    Formation tF = (Formation)formations.get(i);
	    JRadioButtonMenuItem jrmb = new JRadioButtonMenuItem(tF.id +" - "+ tF.name);
	    JRadioButtonMenuItem jrmb2 = new JRadioButtonMenuItem(tF.id +" - "+ tF.name);
	    f_bg.add(jrmb);
	    formlist.add(jrmb);
	    f_bg2.add(jrmb2);
	    formlist2.add(jrmb2);
	    jrmb.addActionListener(new FormationButtonListener(tF));
	    jrmb2.addActionListener(new FormationButton2Listener(tF));
	    if (sF != null && tF.id == sF.id){
		jrmb2.setSelected(true);
	    }
	    if (tF.id == aF.id){
		jrmb.setSelected(true);
		tf_id.setText(""+ aF.id);
		tf_name.setText(aF.name);
	    }
	}	
    }

    // Creates a new (empty) Formation
    public void createNewFormation(){
	Formation newF = new Formation();
	for(int i = 0; i < formations.size(); i++){
	    int fID = ((Formation)formations.get(i)).id;
	    if (fID == newF.id){
		newF.id--;
		i = 0;
	    }
	}
	formations.add(newF);
	aF = newF;
	updateFormationMenu();
	for (int i = 0; i < 11; i++){
	    sB[i].setSelectedIndex(aF.player[i].type);
	}
	bottom.repaint();
	field.repaint();
    }

    // Creates a new (copied) Formation
    public void copyFormation(){	
	Formation newF = (Formation)aF.clone();
	newF.id = -1;
	newF.name = "Copy of "+ newF.name;
	for(int i = 0; i < formations.size(); i++){
	    int fID = ((Formation)formations.get(i)).id;
	    if (fID == newF.id){
		newF.id--;
		i = 0;
	    }
	}
	formations.add(newF);
	aF = newF;
	updateFormationMenu();
	for (int i = 0; i < 11; i++){
	    sB[i].setSelectedIndex(aF.player[i].type);
	}
	bottom.repaint();
	field.repaint();
    }
    // Deletes a formation. At least one formation has to remain
    public void deleteFormation(){	
	if (formations.size() == 1) return;
	formations.remove(aF);
	aF = (Formation)formations.getFirst();
	updateFormationMenu();
	for (int i = 0; i < 11; i++){
	    sB[i].setSelectedIndex(aF.player[i].type);
	}
	bottom.repaint();
	field.repaint();
    }

    // Outputs a single Formation to std::out
    public void print(){
	System.out.println("### FormationEditor v"+ VERSION);
	// HomePosition: X
	for(int i = 0; i < 11; i++)
	    System.out.print("  "+ aF.player[i].homePosition.getX() +" ");
	System.out.println();
	// HomePosition: Y
	for(int i = 0; i < 11; i++)
	    System.out.print("  "+ aF.player[i].homePosition.getY() +" ");
	System.out.println();
	// InitPosition: X
	for(int i = 0; i < 11; i++)
	    System.out.print("  "+ aF.player[i].initPosition.getX() +" ");
	System.out.println();
	// InitPosition: Y
	for(int i = 0; i < 11; i++)
	    System.out.print("  "+ aF.player[i].initPosition.getY() +" ");
	System.out.println();
	// Behaviour
	for(int i = 0; i < 11; i++)
	    System.out.print("  "+ (aF.player[i].type+1) +" ");
	System.out.println();
	// Attraction X
	for(int i = 0; i < 11; i++)
	    System.out.print("  "+ aF.player[i].attrX  +" ");
	System.out.println();
	// Attraction Y
	for(int i = 0; i < 11; i++)
	    System.out.print("  "+ aF.player[i].attrY +" ");
	System.out.println();
	// Behind Ball
	for(int i = 0; i < 11; i++)
	    System.out.print("  "+ (aF.player[i].behindBall ? 1 : 0) +" ");
	System.out.println();
	// Min X
	for(int i = 0; i < 11; i++)
	    System.out.print("  "+ aF.player[i].minX +" ");
	System.out.println();
	// Max X
	for(int i = 0; i < 11; i++)
	    System.out.print("  "+ aF.player[i].maxX +" ");
	System.out.println();
 	System.out.println();
    }

    public void cycleDisplayUpdate(){
	cycleEdit.setText(""+ activeWorldModel.cycle);
	cycleEdit2.setText(""+ activeWorldModel.stoppedCycle);
	cycleLeft.setEnabled( activeWorldModel.prev != null);
	cycleRight.setEnabled(activeWorldModel.next != null);
	field.repaint();
	bottom.repaint();

	if (stats != null)
	    stats.repaint();
    }

    public void play(){
	pl = new Play();
	pl.start();
	play.setText("Stop");
    }

    public void stop(){
	if (pl != null){
	    pl.destroy();
	    pl = null;
	}
	play.setText("Play");
    }

    public boolean cycleNext(){
	if (activeWorldModel.next == null) return false;
	activeWorldModel = activeWorldModel.next;
	cycleDisplayUpdate();
	return true;
    }
    public void cyclePrev(){
	activeWorldModel = activeWorldModel.prev;
	cycleDisplayUpdate();
    }

    public void rotate(){
	rotateView = !rotateView;
    }

    public void cycle(int c, int sc){      	
	if (c  <    1) return;
	if (c  > 6000) return;
	if (sc <    0) return;
	
	WorldModel temp = firstWorldModel;
	while (temp != null){
	    if (temp.cycle == c && temp.stoppedCycle == sc){
		activeWorldModel = temp;
		cycleDisplayUpdate();
		return;
	    }
	    temp = temp.next;
	}
	System.out.println("Entered cycle not found.");
	cycleDisplayUpdate();
    }

    // initializes the whole bitch
    public FormationEditor(){
	rotateView = false;
	setJMenuBar(menu);
	file = new JMenu("File");
	file.setMnemonic(KeyEvent.VK_F);
	graphics = new JMenu("Antialiasing");
	menu.add(file);
	create = new JMenuItem("New", KeyEvent.VK_N);
	create.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, ActionEvent.CTRL_MASK));
	file.add(create);
	copy = new JMenuItem("Copy", KeyEvent.VK_C);
	copy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, ActionEvent.CTRL_MASK));
	file.add(copy);
	delete = new JMenuItem("Delete", KeyEvent.VK_D);
	delete.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
	file.add(delete);
	file.addSeparator();
	save = new JMenuItem("Save...", KeyEvent.VK_S);
	save.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
	file.add(save);
	load = new JMenuItem("Load...", KeyEvent.VK_L);
	load.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.CTRL_MASK));
	file.add(load);
	file.addSeparator();
	s_export = new JMenuItem("Export...", KeyEvent.VK_X);	
	l_import = new JMenuItem("Import...", KeyEvent.VK_I);
	j_import = new JMenuItem("Import UvA...", KeyEvent.VK_V);
	j_export = new JMenuItem("Export UvA...", KeyEvent.VK_A);
	l_wm = new JMenuItem("Load Worldmodel...", KeyEvent.VK_W);
	ul_wm = new JMenuItem("Unload Worldmodel", KeyEvent.VK_U);
	mstats = new JMenuItem("Show Statistics", KeyEvent.VK_S);
	mstats.setEnabled(false);
	file.add(s_export);
	file.add(l_import);
	file.addSeparator();
	file.add(j_import);
	file.add(j_export);
	file.addSeparator();
	file.add(l_wm);
	file.add(ul_wm);
	ul_wm.setEnabled(false);
	file.addSeparator();
	quit = new JMenuItem("Quit", KeyEvent.VK_Q);
	quit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, ActionEvent.ALT_MASK));
	file.add(quit);
	options = new JMenu("Options");
	options.setMnemonic(KeyEvent.VK_O);
	menu.add(options);
	zoomin = new JMenuItem("Zoom in", KeyEvent.VK_PLUS);
	zoomin.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PLUS, ActionEvent.ALT_MASK));
	options.add(zoomin);
	zoomout = new JMenuItem("Zoom out", KeyEvent.VK_MINUS);
	zoomout.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_MINUS, ActionEvent.ALT_MASK));
	options.add(zoomout);
	options.addSeparator();
	tgrid = new JCheckBoxMenuItem("Toggle Grid");
	tgrid.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, ActionEvent.ALT_MASK));
	options.add(tgrid);
	gridp = new JMenuItem("Grid +", null); //KeyEvent.VK_PLUS);
	gridp.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PLUS, ActionEvent.CTRL_MASK));
	options.add(gridp);
	gridm = new JMenuItem("Grid -", null); //KeyEvent.VK_MINUS);
	gridm.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_MINUS, ActionEvent.CTRL_MASK));
	options.add(gridm);
	antialias = new JCheckBoxMenuItem("Use Antialiasing");
	antialias.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, ActionEvent.CTRL_MASK));
	antialiasText = new JCheckBoxMenuItem("Use Text-Antialiasing");
	antialiasText.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T, ActionEvent.CTRL_MASK));	
	graphics.add(antialias);
	graphics.add(antialiasText);
	rendering = new JMenu("Rendering");
	r_bg = new ButtonGroup();
	r_quality = new JRadioButtonMenuItem("Quality");
	r_quality.setSelected(true);
	r_bg.add(r_quality);
	rendering.add(r_quality);
	r_speed = new JRadioButtonMenuItem("Speed");
	r_speed.setSelected(true);
	r_bg.add(r_speed);
	rendering.add(r_speed);
	options.addSeparator();
	showLimits = new JCheckBoxMenuItem("Show Positioning Limits");
	showLimits.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_L, ActionEvent.ALT_MASK));
	options.add(showLimits);
	options.add(mstats);
	options.addSeparator();
	options.add(graphics);
	options.add(rendering);	
	print = new JMenuItem("Print Formation", KeyEvent.VK_P);
	print.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P, ActionEvent.ALT_MASK));	
	pong = new JCheckBoxMenuItem("Play Pong");
	options.addSeparator();
	options.add(pong);
	options.addSeparator();
	options.add(print);
	
	formlist = new JMenu("Formation");	
	formlist.setMnemonic(KeyEvent.VK_R);
	menu.add(formlist);
	formlist2 = new JMenu("Enemy Formation");
	formlist2.setMnemonic(KeyEvent.VK_E);
	menu.add(formlist2);

	cycleLeft = new JButton("<<");
	cycleRight = new JButton(">>");
	cycleEdit  = new JTextField("");
	cycleEdit2 = new JTextField("");
	play = new JButton("Play");
	jb_rotateView = new JButton("Rotate");
	

	// Some ActionListeners. 
	cycleLeft.addActionListener(    mial);
	cycleRight.addActionListener(   mial);
	cycleEdit.addActionListener(    mial);
	cycleEdit2.addActionListener(   mial);
	play.addActionListener(         mial);
	pong.addActionListener(         mial);
	s_export.addActionListener(     mial);
	l_import.addActionListener(     mial);
	j_import.addActionListener(     mial);
	j_export.addActionListener(     mial);
	l_wm.addActionListener(         mial);
	ul_wm.addActionListener(        mial);
	create.addActionListener(       mial);
	delete.addActionListener(       mial);
	copy.addActionListener(         mial);
	r_speed.addActionListener(      mial);
	r_quality.addActionListener(    mial);
	antialias.addActionListener(    mial);
	antialiasText.addActionListener(mial);
	showLimits.addActionListener(   mial);
	gridp.addActionListener(        mial);
	gridm.addActionListener(        mial);
	print.addActionListener(        mial);
	zoomin.addActionListener(       mial);
	zoomout.addActionListener(      mial);
	load.addActionListener(         mial);
	save.addActionListener(         mial);
	quit.addActionListener(         mial);
	tgrid.addActionListener(        mial);
	play.addActionListener(         mial);
	jb_rotateView.addActionListener(mial);
	mstats.addActionListener(       mial);
	tgrid.setState(true);
	top = new JPanel();

	// Both should not be used
	save.setEnabled(false);
	load.setEnabled(false);
		
	
	delete.setToolTipText("Removes the currently selected formation. At least one formation has to remain so it can't be deleted");
	copy.setToolTipText("Duplicates the currently selected formation");
	r_speed.setToolTipText("Select which criteria should be used for rendering");
	r_quality.setToolTipText("Select which criteria should be used for rendering");
	antialias.setToolTipText("Disables/Enables antialiasing. This can increase the visual quality but decrease the performance");
	antialiasText.setToolTipText("Disables/Enables antialiasing for text. This can increase the visual quality but decrease the performance");
	showLimits.setToolTipText("Disables/Enables the visibility of movement limits given by the attraction values and the maximal and minimal X values");
	gridm.setToolTipText("Resizes the grid");
	gridp.setToolTipText("Resizes the grid");
	print.setToolTipText("Prints the current formation to System.out");
	zoomout.setToolTipText("Zooms out of the field");
	zoomin.setToolTipText("Zooms in on the field");
	l_import.setToolTipText("Loads formations from a textfile which was previously saved");
	j_import.setToolTipText("Loads formations from a textfile which was previously saved using the UvA format");
	s_export.setToolTipText("Saves the formations as a textfile which can be read by the agent");
	tgrid.setToolTipText("Toggles the visibility of the grid");
	load.setToolTipText("Loads formations from a serialized object");
	save.setToolTipText("Saves the formations as serialized object");
	jb_rotateView.setToolTipText("Rotate the field/view by 180 degree");

	JLabel l_id = new JLabel("ID:");
	tf_id = new JTextField(10);
	JLabel l_name = new JLabel("Name:");
	tf_name = new JTextField(10);
	top.add(l_id);
	top.add(tf_id);
	top.add(l_name);
	top.add(tf_name);
	top.add(new JLabel("Edit:"));

	tf_id.addActionListener(mial);
	tf_name.addActionListener(mial);

	setDefaultLookAndFeelDecorated(true);
	setResizable(false);
	setIconImage(new ImageIcon("ball.gif").getImage());
	bl.setHgap(2);
	bl.setVgap(2);
	main = new JPanel(bl);
	field = new Field();
	gl.setHgap(2);
	gl.setVgap(2);
	right = new JPanel(gl);
	playerButtons = new JToggleButton[11];
	maxXButtons   = new JToggleButton[11];
	minXButtons   = new JToggleButton[11];
	aF = new Formation();
	formations.add(aF);
	updateFormationMenu();
	aF.name = "new";
	tf_name.setText(aF.name);
	tf_id.setText(""+ aF.id);

	sHP = new JButton("InitPosition");
	sHP.setPreferredSize(tBD);
	sHP.addActionListener(mial);
	top.add(sHP);
	top.add(new JLabel("Cycle: "));
	cycleEdit.setPreferredSize(new Dimension(70, (int)tBD.getHeight()));
	cycleEdit2.setPreferredSize(new Dimension(70, (int)tBD.getHeight()));
	Dimension sBD = new Dimension(50, (int)tBD.getHeight());
	//cycleLeft.setPreferredSize(sBD);
	//cycleRight.setPreferredSize(sBD);
	top.add(cycleLeft);
	top.add(cycleEdit);
	top.add(cycleEdit2);
	top.add(cycleRight);
	top.add(play);
	top.add(jb_rotateView);
	cycleLeft.setEnabled(false);
	cycleRight.setEnabled(false);
	cycleEdit.setEnabled(false);
	cycleEdit2.setEnabled(false);
	play.setEnabled(false);

	JPanel pSP[] = new JPanel[11];
	JPanel pSP2[] = new JPanel[11];

	for (int i = 0; i < 11; i++){
	    pSP[i] = new JPanel(new GridLayout(1,2));	   
	    pSP2[i] = new JPanel(new GridLayout(1,2));
	    minXButtons[i] = new JToggleButton("minX");
	    minXButtons[i].setName("minX"+ (i+1));
	    maxXButtons[i] = new JToggleButton("maxX");
	    maxXButtons[i].setName("maxX"+ (i+1));
	    minXButtons[i].setPreferredSize(new Dimension(5, 20));
	    minXButtons[i].setMargin(new Insets(0,0,0,0));
	    maxXButtons[i].setPreferredSize(new Dimension(5, 20));
	    maxXButtons[i].setMargin(new Insets(0,0,0,0));
	    playerButtons[i] = new JToggleButton("Player "+ (i+1));
	    playerButtons[i].setName("Player "+ (i+1));
	    playerButtons[i].setPreferredSize(new Dimension(90, 20));
	    playerButtons[i].setMargin(new Insets(0,0,0,0));
	    playerButtons[i].addActionListener(mial);
	    minXButtons[i].addActionListener(mial);
	    maxXButtons[i].addActionListener(mial);
	    playerButtons[i].setIcon(new ImageIcon("status-error.png"));
	    playerButtons[i].setSelectedIcon(new ImageIcon("status-selected.png"));
	    playerButtons[i].setHorizontalAlignment(SwingConstants.LEFT);
	    pSP[i].add(playerButtons[i]);
	    pSP2[i].add(minXButtons[i]);
	    pSP2[i].add(maxXButtons[i]);
	    pSP[i].add(pSP2[i]);

	    right.add(pSP[i]);
	    aF.player[i] = new Player(i+1, PITCH_WIDTH);
	    sB[i] = new JComboBox(behaviour);
	    sB[i].setSelectedIndex(i);
	    sB[i].addActionListener(mial);
	    right.add(sB[i]);
	}
	playerButtons[0].setSelected(true);
	
	field.setMaximumSize(fieldSize);
	field.setPreferredSize(fieldSize);
	// This could be another home made class?
	field.addMouseListener(new FieldMouseInputAdapter());
	field.setFocusable(true);

	field.addKeyListener(new FieldKeyListener());

	jsp = new JScrollPane(field);
	jsp.setPreferredSize(scrollSize);
	jsp.setMaximumSize(scrollSize);

	right.setMaximumSize(rightSize);
	right.setPreferredSize(rightSize);
	top.setMaximumSize(topSize);
	top.setPreferredSize(topSize);

	// what about this bitch?
	jt = new FormationTable(dataModel);
	// candidate for getting fucked away
	rowHeader.setCellRenderer(new RowHeaderRenderer(jt));
	jt.setColumnSelectionAllowed(false);
	jt.setDragEnabled(false);
	jt.setRowSelectionAllowed(false);

	wmTable = new JTable(dataModel2);
	rowHeader2.setCellRenderer(new RowHeaderRenderer(wmTable));
	wmTable.setColumnSelectionAllowed(false);
	wmTable.setDragEnabled(false);
	wmTable.setRowSelectionAllowed(false);

	//jt.setCellRenderer(new DefaultCellRenderer(new  JCheckBox()));
	
	bottom = new JPanel();
	jsp2 = new JScrollPane(jt);
	jsp2.setRowHeaderView(rowHeader);
	jsp2.setPreferredSize(new Dimension((int)topSize.getWidth(), jt.getRowHeight()*(jt.getRowCount()+1)+2));
	bottom.add(jsp2);

	main.add(jsp,    BorderLayout.CENTER);
	main.add(right,  BorderLayout.EAST);
	main.add(top,    BorderLayout.NORTH);
	main.add(bottom, BorderLayout.SOUTH);
	getContentPane().add(main);
	setTitle("FormationEditor");
	pack();
	setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
    }

    public static FormationEditor instance;

    public static void main(String args[]){
	instance = new FormationEditor();
	instance.setVisible(true);
    }
    /*
    public void update(Graphics g){
	super.update(g);
	if (stats != null)
	    stats.update(g);
    }
    */

};
