/***********************************************************************
 * Logger.cc
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   class Logger
 *
 * requires:
 *
 *
 * Created on: 17.12.2003 by wenns and mangels
 * Maintainer: mangels
 * Group:
 * Co-authors:
 *
 * Changelog:
 *
 * $Id: Logger.cc,v 1.2 2004/03/31 10:01:46 harlekin Exp $
 *
 **********************************************************************/

#include "Logger.hh"

Logger::Logger(){}


Logger::Logger(WorldModel *worldModel){
  wm   = worldModel;
  ball = &(wm->ball);
  ms   = &(wm->myself);
  myunum = ms->getUniformNumber();
  teammates = &(wm->myTeam.getPlayers());
  opponents = &(wm->theirTeam.getPlayers());
  if(myunum == UNUM_WHO_SHOULD_LOG)
    openForWrite();
}


void Logger::writeWorldModel(){
  //DEBUG1_WENNS("Logger::worldModel(): called");
  if(myunum != UNUM_WHO_SHOULD_LOG)
    return;
    
  Playmode mode = wm->getPlaymode();
  //TODO: adjust the mode in order to be compatible to the interpretation
  //of the monitor
  dinfo2.body.show.pmode = (char)mode+1; //FIXME
  dinfo2.body.show.time = htons(wm->getTime()->getTime());

  //fill in ball position and ball velocity
  dinfo2.body.show.ball.x =
    htonl((long)rint( ball->getPosition().getX() * SHOWINFO_SCALE2 )) ;
  dinfo2.body.show.ball.y =
    htonl((long)rint( ball->getPosition().getY() * SHOWINFO_SCALE2 )) ;
  dinfo2.body.show.ball.deltax =
    htonl((long)rint( ball->getVelocity().getX() * SHOWINFO_SCALE2 )) ;
  dinfo2.body.show.ball.deltay =
    htonl((long)rint( ball->getVelocity().getY() * SHOWINFO_SCALE2 )) ;

  int playerCount = teammates->size();
  int myUnum = ms->getUniformNumber();
  
  //fill in the teammates data
  for(int i = 0; i < MAX_PLAYER; i++){
    if(i < playerCount)
      (i == myUnum-1) ? writePlayer((*teammates)[i], i, myUnum)
	              : writePlayer((*teammates)[i], i);
    else
      writeDummi(i);
  }
  //fill in the opponents data
  playerCount = opponents->size();
  for(int i = MAX_PLAYER; i < MAX_PLAYER*2; i++){
    if(i-MAX_PLAYER < playerCount)
      writePlayer((*opponents)[i-MAX_PLAYER], i);
    else
      writeDummi(i);
  }

  //write the teamnames and scores in an array, the team that plays left first
  if(wm->getFieldSide() == FS_Left){
    strcpy(dinfo2.body.show.team[0].name, wm->myTeam.getName());
    dinfo2.body.show.team[0].score = htons(wm->getOurScore());
    strcpy(dinfo2.body.show.team[1].name, wm->theirTeam.getName());
    dinfo2.body.show.team[1].score = htons(wm->getTheirScore());
  }
  else{
    strcpy(dinfo2.body.show.team[0].name, wm->theirTeam.getName());
    dinfo2.body.show.team[0].score = htons(wm->getTheirScore());
    strcpy(dinfo2.body.show.team[1].name, wm->myTeam.getName());
    dinfo2.body.show.team[1].score = htons(wm->getOurScore());
  }  

  static bool wroteFinalCycle = false;
  if ( gameLogOpen () ){
    if (mode != PM_BeforeKickOff && mode != PM_TimeOver)
      writeLog(&dinfo2) ;
    else if (mode == PM_TimeOver && !wroteFinalCycle){
      writeLog(&dinfo2) ;
      wroteFinalCycle = true;
    }
  }

  gameLog->flush();
}


void Logger::writePlayer(const Player &p, const int i, const int myUnum){
  //in the log file format a bitmask is used to represent the varios
  //states of the player, such as "disabled", "goalie", "kicking", "tackling",
  //"colliding with player" and some others
  
  //our bit setting strategie: use only the bits DISABLE, STAND and GOALIE
  //ignore all other because they are either (probably) for internal use,
  //of no interest for the representation, or the usage of them is unclear
  //(DISCARD). For some of the bits there is a lack of information for all
  //players except the agent itself
  
  //the bitmask that contains information about the players current status
  int alive;
  //set all to DISABLE as default
  alive = DISABLE; 
  //if the player was ever seen, set to STAND
  if(!p.getTimeSeen().isUndefined())
    alive = STAND;
  //if he is a goalie, set the goalie bit
  if(p.isGoalKeeper())
    alive |= GOALIE;
    
  dinfo2.body.show.pos[i].mode = htons(alive);
  //TODO: look up the ss values for the player types
  dinfo2.body.show.pos[i].type = htons(p.getType());
  
  //write the position, velocity, and the angles of the player
  //the angles are awaited in radian, so convert them
  dinfo2.body.show.pos[i].x
    = htonl((long)rint( p.getPosition().getX() * SHOWINFO_SCALE2 ));
  dinfo2.body.show.pos[i].y
    = htonl((long)rint( p.getPosition().getY() * SHOWINFO_SCALE2 ));
  dinfo2.body.show.pos[i].deltax
    = htonl((long)rint( p.getVelocity().getX() * SHOWINFO_SCALE2 ));
  dinfo2.body.show.pos[i].deltay
    = htonl((long)rint( p.getVelocity().getY() * SHOWINFO_SCALE2 )) ;
  dinfo2.body.show.pos[i].body_angle
    = htonl((long)rint( Deg2Rad(p.getBodyDir()) * SHOWINFO_SCALE2 )) ;
  float relHeadAngle =
    Deg2Rad( (normalizeTo180Deg(p.getHeadDir() - p.getBodyDir())) );
  dinfo2.body.show.pos[i].head_angle
    = htonl((long)rint( relHeadAngle * SHOWINFO_SCALE2 )) ;

  float viewWidth;
  bool viewQuality;
  if( i == myUnum-1 ){ 
    //only own viewmode is known by the agent
    ViewQuality vq = ms->bodyInfo.getViewQuality();
    viewQuality = (vq == VQ_HIGH) ? true : false;
    viewWidth = Deg2Rad(ms->getViewAngle());
  }
  else{
    viewQuality = true;          //dummy: high
    viewWidth   = Deg2Rad(1);    //dummy: 1 (better for the visualisation than
                                 //undefined)
  }
  //set the view mode of the player
  dinfo2.body.show.pos[i].view_width
    = htonl((long)rint( viewWidth * SHOWINFO_SCALE2 ));
  dinfo2.body.show.pos[i].view_quality
    = htons((short)rint( viewQuality )) ;

  //set the values representing the stamina model
  dinfo2.body.show.pos[i].stamina
    = htonl((long)rint( p.getStamina() * SHOWINFO_SCALE2 )) ;
  dinfo2.body.show.pos[i].effort
    = htonl((long)rint( p.getEffort() * SHOWINFO_SCALE2 )) ;
  dinfo2.body.show.pos[i].recovery
    = htonl((long)rint( p.getRecovery() * SHOWINFO_SCALE2 )) ;
}


void Logger::writeDummi(int i){
  float dummiViewWidth = UNDEFINED_FLOAT;
  bool dummiViewQuality = true;
  
  //the bitmask that contains information about the players current status
  //the "dummies" shouldnt be shown and thus are all disabled
  int alive = DISABLE; 
    
  dinfo2.body.show.pos[i].mode = htons(alive);
  dinfo2.body.show.pos[i].type = htons( (short)UNDEFINED_INT );

  //write the position, velocity, and the angles of the player
  dinfo2.body.show.pos[i].x
    = htonl((long)rint( UNDEFINED_FLOAT * SHOWINFO_SCALE2 ));
  dinfo2.body.show.pos[i].y
    = htonl((long)rint( UNDEFINED_FLOAT * SHOWINFO_SCALE2 ));
  dinfo2.body.show.pos[i].deltax
    = htonl((long)rint( UNDEFINED_FLOAT * SHOWINFO_SCALE2 ));
  dinfo2.body.show.pos[i].deltay
    = htonl((long)rint( UNDEFINED_FLOAT * SHOWINFO_SCALE2 )) ;
  dinfo2.body.show.pos[i].body_angle
    = htonl((long)rint( UNDEFINED_FLOAT * SHOWINFO_SCALE2 )) ;
  dinfo2.body.show.pos[i].head_angle
    = htonl((long)rint( UNDEFINED_FLOAT * SHOWINFO_SCALE2 )) ;
  
  //set the view mode of the player
  dinfo2.body.show.pos[i].view_width
    = htonl((long)rint( dummiViewWidth * SHOWINFO_SCALE2 ));
  dinfo2.body.show.pos[i].view_quality
    = htons((short)rint( dummiViewQuality )) ;

  //set the values representing the stamina model
  dinfo2.body.show.pos[i].stamina
    = htonl((long)rint( UNDEFINED_FLOAT * SHOWINFO_SCALE2 )) ;
  dinfo2.body.show.pos[i].effort
    = htonl((long)rint( UNDEFINED_FLOAT * SHOWINFO_SCALE2 )) ;
  dinfo2.body.show.pos[i].recovery
    = htonl((long)rint( UNDEFINED_FLOAT * SHOWINFO_SCALE2 )) ;
}


void Logger::openForWrite(void){
  char *filename = "agentWorldModel.rcg";
  const int GAMELOG_VERSION = 3;

  gameLog = new ofstream();
  gameLog->open ( filename, ( ofstream::binary
                              | ofstream::out
                              | ofstream::trunc) );

  if ( !gameLogOpen () ){
    cerr << __FILE__ << ": " << __LINE__
      << ": can't open record_file " << filename << endl ;
    return;
  }

  // write version information
  char buf[5] ;
  buf[0] = 'U' ;
  buf[1] = 'L' ;
  buf[2] = 'G' ;
  buf[3] =  GAMELOG_VERSION;

  gameLogWrite ( buf, 4 );

  /*
  short mode = htons(PARAM_MODE);
  server_params_t stmp = convertServerParamsToStruct ();
  gameLogWrite((char *)(&mode), sizeof(mode) ) ;
  gameLogWrite((char *)(&stmp), sizeof(stmp) ) ;

  mode = htons(PPARAM_MODE);
  player_params_t ptmp = convertPlayerParamsToStruct ();
  gameLogWrite((char *)(&mode), sizeof(mode) ) ;
  gameLogWrite((char *)(&ptmp), sizeof(ptmp) ) ;

  mode = htons(PT_MODE);
  for ( int i = 0; i < Configuration::playerTypes; i++ ){
    player_type_t pttmp = convertPlayerTypeToStruct(i);
    gameLogWrite((char *)(&mode), sizeof(mode) ) ;
    gameLogWrite((char *)(&pttmp), sizeof(pttmp) ) ;
  }
  */
  gameLog->flush();
}

ostream& Logger::gameLogWrite ( const char* str, size_t n ){
  return gameLog->write ( str, n );
}

server_params_t Logger::convertServerParamsToStruct(){
   server_params_t tmp;

   tmp.gwidth = htonl( (long)(SHOWINFO_SCALE2 * Configuration::goalWidth) );
   tmp.inertia_moment = htonl( (long)(SHOWINFO_SCALE2 * Configuration::inertiaMoment) );
   tmp.psize = htonl( (long)(SHOWINFO_SCALE2 * Configuration::playerSize) );
   tmp.pdecay = htonl( (long)(SHOWINFO_SCALE2 * Configuration::playerDecay) );
   tmp.prand = htonl( (long)(SHOWINFO_SCALE2 * Configuration::playerRand) );
  tmp.pweight = htonl( (long)(SHOWINFO_SCALE2 * Configuration::playerWeight) );
  tmp.pspeed_max = htonl( (long)(SHOWINFO_SCALE2 * Configuration::playerSpeedMax) );
  tmp.paccel_max = htonl( (long)(SHOWINFO_SCALE2 * Configuration::playerAccelMax) );
  tmp.stamina_max = htonl( (long)(SHOWINFO_SCALE2 * Configuration::staminaMax) );
  tmp.stamina_inc = htonl( (long)(SHOWINFO_SCALE2 * Configuration::staminaIncMax) );
  tmp.recover_init = htonl( (long)(SHOWINFO_SCALE2 * 1.0) );//missing in our Configuration.hh
  tmp.recover_dthr = htonl( (long)(SHOWINFO_SCALE2 * Configuration::recoverDecThr) );
  tmp.recover_min = htonl( (long)(SHOWINFO_SCALE2 * Configuration::recoverMin) );
  tmp.recover_dec = htonl( (long)(SHOWINFO_SCALE2 * Configuration::recoverDec) );
  tmp.effort_init = htonl( (long)(SHOWINFO_SCALE2 * Configuration::effortInit) );
  tmp.effort_dthr = htonl( (long)(SHOWINFO_SCALE2 * Configuration::effortDecThr) );
  tmp.effort_min = htonl( (long)(SHOWINFO_SCALE2 * Configuration::effortMin) );
  tmp.effort_dec = htonl( (long)(SHOWINFO_SCALE2 * Configuration::effortDec) );
  tmp.effort_ithr = htonl( (long)(SHOWINFO_SCALE2 * Configuration::effortIncThr) );
  tmp.effort_inc = htonl( (long)(SHOWINFO_SCALE2 * Configuration::effortInc) );
  tmp.kick_rand = htonl( (long)(SHOWINFO_SCALE2 * Configuration::kickRand) );
  tmp.team_actuator_noise = htons( (short) Configuration::teamActuatorNoise );
  tmp.prand_factor_l = htonl( (long)(SHOWINFO_SCALE2 * Configuration::prandFactorL ) );
  tmp.prand_factor_r = htonl( (long)(SHOWINFO_SCALE2 * Configuration::prandFactorR ) );
  tmp.kick_rand_factor_l = htonl( (long)(SHOWINFO_SCALE2 * Configuration::kickRandFactorL ) );
  tmp.kick_rand_factor_r = htonl( (long)(SHOWINFO_SCALE2 * Configuration::kickRandFactorR ) );
  tmp.bsize = htonl( (long)(SHOWINFO_SCALE2 * Configuration::ballSize) );
  tmp.bdecay = htonl( (long)(SHOWINFO_SCALE2 * Configuration::ballDecay) );
  tmp.brand = htonl( (long)(SHOWINFO_SCALE2 * Configuration::ballRand) );
  tmp.bweight = htonl( (long)(SHOWINFO_SCALE2 * Configuration::ballWeight) );
  tmp.bspeed_max = htonl( (long)(SHOWINFO_SCALE2 * Configuration::ballSpeedMax) );
  tmp.baccel_max = htonl( (long)(SHOWINFO_SCALE2 * Configuration::ballAccelMax ) );
  tmp.dprate = htonl( (long)(SHOWINFO_SCALE2 * Configuration::dashPowerRate) );
  tmp.kprate = htonl( (long)(SHOWINFO_SCALE2 * Configuration::kickPowerRate) );
  tmp.kmargin = htonl( (long)(SHOWINFO_SCALE2 * Configuration::kickableMargin) );
  tmp.ctlradius = htonl( (long)(SHOWINFO_SCALE2 * Configuration::controlRadius) );
  tmp.ctlradius_width = htonl( (long)(SHOWINFO_SCALE2 * Configuration::controlRadius) );//missing in our Configuration.hh
  tmp.maxp = htonl( (long)(SHOWINFO_SCALE2 * Configuration::maxpower) );
  tmp.minp = htonl( (long)(SHOWINFO_SCALE2 * Configuration::minpower) );
  tmp.maxm = htonl( (long)(SHOWINFO_SCALE2 * Configuration::maxmoment) );
  tmp.minm = htonl( (long)(SHOWINFO_SCALE2 * Configuration::minmoment) );
  tmp.maxnm = htonl( (long)(SHOWINFO_SCALE2 * Configuration::maxneckmoment) );
  tmp.minnm = htonl( (long)(SHOWINFO_SCALE2 * Configuration::minneckmoment) );
  tmp.maxn = htonl( (long)(SHOWINFO_SCALE2 * Configuration::maxneckang) );
  tmp.minn = htonl( (long)(SHOWINFO_SCALE2 * Configuration::minneckang) );
  tmp.visangle = htonl( (long)(SHOWINFO_SCALE2 * Configuration::visibleAngle) );
  tmp.visdist = htonl( (long)(SHOWINFO_SCALE2 * Configuration::visibleDistance) );
  tmp.windir = htonl( (long)(SHOWINFO_SCALE2 * Configuration::windDir) );
  tmp.winforce = htonl( (long)(SHOWINFO_SCALE2 * Configuration::windForce) );
  tmp.winang = htonl( (long)(SHOWINFO_SCALE2 * Configuration::windAng) );
  tmp.winrand = htonl( (long)(SHOWINFO_SCALE2 * Configuration::windRand) );
  tmp.kickable_area = htonl( (long)(SHOWINFO_SCALE2 * Configuration::kickableMargin+Configuration::ballSize+Configuration::playerSize) );
  tmp.catch_area_l = htonl( (long)(SHOWINFO_SCALE2 * Configuration::catchableAreaL) );
  tmp.catch_area_w = htonl( (long)(SHOWINFO_SCALE2 * Configuration::catchableAreaW) );
  tmp.catch_prob = htonl( (long)(SHOWINFO_SCALE2 * Configuration::catchProbability) );//what the heck...?
  tmp.goalie_max_moves = htons( (short) Configuration::goalieMaxMoves );
  tmp.ckmargin = htonl( (long)(SHOWINFO_SCALE2 * Configuration::ckickMargin) );
  tmp.offside_area = htonl( (long)(SHOWINFO_SCALE2 * Configuration::offsideActiveAreaSize) );
  tmp.win_no = htons( (short) 0);//no wind = 0
  tmp.win_random = htons( (short) 0) ;//wind random = 0
  tmp.say_cnt_max = htons( (short) Configuration::sayCoachCntMax) ;
  tmp.SayCoachMsgSize = htons( (short) Configuration::sayCoachMsgSize) ;
  tmp.clang_win_size = htons( (short) Configuration::clangWinSize );
  tmp.clang_define_win = htons( (short) Configuration::clangDefineWin );
  tmp.clang_meta_win = htons( (short) Configuration::clangMetaWin );
  tmp.clang_advice_win = htons( (short) Configuration::clangAdviceWin );
  tmp.clang_info_win = htons( (short) Configuration::clangInfoWin );
  tmp.clang_mess_delay = htons( (short) Configuration::clangMessDelay );
  tmp.clang_mess_per_cycle = htons( (short) Configuration::clangMessPerCycle );
  tmp.half_time = htons( (short) Configuration::halfTime) ;
  tmp.sim_st = htons( (short) Configuration::simulatorStep) ;
  tmp.send_st = htons( (short) Configuration::sendStep) ;
  tmp.recv_st = htons( (short) Configuration::recvStep) ;
  tmp.sb_step = htons( (short) Configuration::senseBodyStep) ;
  tmp.lcm_st = htons( (short) 0);//!!!???!!!lcm_st!!!!???!!!
  tmp.M_say_msg_size = htons( (short) Configuration::sayMsgSize) ;
  tmp.M_hear_max = htons( (short) Configuration::hearMax) ;
  tmp.M_hear_inc = htons( (short) Configuration::hearInc) ;
  tmp.M_hear_decay = htons( (short) Configuration::hearDecay) ;
  tmp.cban_cycle = htons( (short) Configuration::catchBanCycle) ;
  tmp.slow_down_factor = htons( (short) Configuration::slowDownFactor) ;
  tmp.useoffside = htons( (short) Configuration::useOffside) ;
  tmp.kickoffoffside = htons( (short) Configuration::forbidKickOffOffside) ;
  tmp.offside_kick_margin = htonl( (long)(SHOWINFO_SCALE2 * Configuration::offsideKickMargin) );
  tmp.audio_dist = htonl( (long)(SHOWINFO_SCALE2 * Configuration::audioCutDist) );
  tmp.dist_qstep = htonl( (long)(SHOWINFO_SCALE2 * Configuration::quantizeStep) );
  tmp.land_qstep = htonl( (long)(SHOWINFO_SCALE2 * Configuration::quantizeStepL) );
  // #ifdef NEW_QSTEP
  tmp.dir_qstep = htonl( (long)(SHOWINFO_SCALE2 * -1) );//!!!???!!!
  tmp.dist_qstep_l = htonl( (long)(SHOWINFO_SCALE2 * -1) );//!!!???!!!
  tmp.dist_qstep_r = htonl( (long)(SHOWINFO_SCALE2 * -1) );//!!!???!!!
  tmp.land_qstep_l = htonl( (long)(SHOWINFO_SCALE2 * -1) );//!!!???!!!
  tmp.land_qstep_r = htonl( (long)(SHOWINFO_SCALE2 * -1) );//!!!???!!!
  tmp.dir_qstep_l = htonl( (long)(SHOWINFO_SCALE2 * -1) );//!!!???!!!
  tmp.dir_qstep_r = htonl( (long)(SHOWINFO_SCALE2 * -1) );//!!!???!!!
  // #else
  // tmp.dir_qstep = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  // tmp.dist_qstep_l = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  // tmp.dist_qstep_r = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  // tmp.land_qstep_l = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  // tmp.land_qstep_r = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  // tmp.dir_qstep_l = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  // tmp.dir_qstep_r = htonl( (long)(SHOWINFO_SCALE2 * -1) );
  // #endif
  tmp.CoachMode = htons( (short) Configuration::coach) ;
  tmp.CwRMode = htons( (short) Configuration::oldCoachHear) ;
  tmp.old_hear = htons( (short) Configuration::oldCoachHear) ;
  tmp.sv_st = htons( (short) 0) ;//!!!???!!!sv_st online coach look interval step!!!???!!!

  tmp.synch_mode   = htons( (short)Configuration::synchMode);//pfr:SYNCH
  tmp.synch_offset = htons( (short)Configuration::synchOffset);//pfr:SYNCH
  tmp.synch_micro_sleep = htons( (short)Configuration::synchMicroSleep);//pfr:SYNCH

  tmp.start_goal_l = htons( (short)Configuration::startGoalL );
  tmp.start_goal_r = htons( (short)Configuration::startGoalR );

  tmp.fullstate_l = htons( (short)Configuration::fullstateL );
  tmp.fullstate_r = htons( (short)Configuration::fullstateR );

  tmp.drop_time = htons( (short) Configuration::dropBallTime) ;

  tmp.slowness_on_top_for_left_team = htonl( (long)(SHOWINFO_SCALE2 * Configuration::slownessOnTopForLeftTeam) );
  tmp.slowness_on_top_for_right_team = htonl( (long)(SHOWINFO_SCALE2 * Configuration::slownessOnTopForLeftTeam) );

  tmp.point_to_ban =  htons( (short)Configuration::pointToBan );
  tmp.point_to_duration =  htons( (short)Configuration::pointToDuration );
  return tmp;
}

player_params_t Logger::convertPlayerParamsToStruct(){
  player_params_t tmp;

  tmp.player_types = htons( (short) Configuration::playerTypes );
  tmp.subs_max = htons( (short) Configuration::subsMax );
  tmp.pt_max = htons( (short) Configuration::ptMax );

  tmp.player_speed_max_delta_min =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::playerSpeedMaxDeltaMin) ));
  tmp.player_speed_max_delta_max =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::playerSpeedMaxDeltaMax) ));
  tmp.stamina_inc_max_delta_factor =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::staminaIncMaxDeltaFactor) ));

  tmp.player_decay_delta_min =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::playerDecayDeltaMin) ));
  tmp.player_decay_delta_max =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::playerDecayDeltaMax) ));
  tmp.inertia_moment_delta_factor =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::inertiaMomentDeltaFactor) ));

  tmp.dash_power_rate_delta_min =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::dashPowerRateDeltaMin) ));
  tmp.dash_power_rate_delta_max =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::dashPowerRateDeltaMax) ));
  tmp.player_size_delta_factor =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::playerSizeDeltaFactor) ));

  tmp.kickable_margin_delta_min =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::kickableMarginDeltaMin) ));
  tmp.kickable_margin_delta_max =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::kickableMarginDeltaMax) ));
  tmp.kick_rand_delta_factor =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::kickRandDeltaFactor) ));

  tmp.extra_stamina_delta_min =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::extraStaminaDeltaMin) ));
  tmp.extra_stamina_delta_max =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::extraStaminaDeltaMax) ));
  tmp.effort_max_delta_factor =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::effortMaxDeltaFactor) ));
  tmp.effort_min_delta_factor =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::effortMinDeltaFactor) ));
  tmp.random_seed = htonl( (long) Configuration::randomSeed );

  tmp.new_dash_power_rate_delta_min =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::newDashPowerRateDeltaMin) ));
  tmp.new_dash_power_rate_delta_max =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::newDashPowerRateDeltaMax) ));
  tmp.new_stamina_inc_max_delta_factor =
    htonl( (long)rint((SHOWINFO_SCALE2 * Configuration::newStaminaIncMaxDeltaFactor) ));

  return tmp;

}
player_type_t Logger::convertPlayerTypeToStruct(int i){
  player_type_t tmp;

  tmp.id = htons ( Configuration::playerTypesV[i].getTypeId() );
  tmp.player_speed_max =
    htonl ( (long)rint((Configuration::playerTypesV[i].getPlayerSpeedMax() * SHOWINFO_SCALE2) ));
  tmp.stamina_inc_max =
    htonl ( (long)rint((Configuration::playerTypesV[i].getStaminaIncMax() * SHOWINFO_SCALE2) ));
  tmp.player_decay =
    htonl ( (long)rint((Configuration::playerTypesV[i].getPlayerDecay() * SHOWINFO_SCALE2) ));
  tmp.inertia_moment =
    htonl ( (long)rint((Configuration::playerTypesV[i].getInertiaMoment() * SHOWINFO_SCALE2) ));
  tmp.dash_power_rate =
    htonl ( (long)rint((Configuration::playerTypesV[i].getDashPowerRate() * SHOWINFO_SCALE2) ));
  tmp.player_size =
    htonl ( (long)rint((Configuration::playerTypesV[i].getPlayerSize() * SHOWINFO_SCALE2) ));
  tmp.kickable_margin =
    htonl ( (long)rint((Configuration::playerTypesV[i].getKickableMargin() * SHOWINFO_SCALE2) ));
  tmp.kick_rand =
    htonl ( (long)rint((Configuration::playerTypesV[i].getKickRand() * SHOWINFO_SCALE2) ));
  tmp.extra_stamina =
    htonl ( (long)rint((Configuration::playerTypesV[i].getExtraStamina() * SHOWINFO_SCALE2) ));
  tmp.effort_max =
    htonl ( (long)rint((Configuration::playerTypesV[i].getEffortMax() * SHOWINFO_SCALE2) ));
  tmp.effort_min =
    htonl ( (long)rint((Configuration::playerTypesV[i].getEffortMin() * SHOWINFO_SCALE2) ));

  return tmp;
}


void Logger::writeLog(dispinfo_t2* msg)
{
  short mode;
  static char pm = (char)0;
  static team_t teams[2] = { { "", 0 }, { "", 0 } };

  // if playmode has changed write playmode
  if ( pm != msg->body.show.pmode ){
    pm = msg->body.show.pmode;

    mode = htons(PM_MODE);

    gameLogWrite((char *)(&mode), sizeof(mode)) ;
    gameLogWrite((char *)(&pm), sizeof(pm)) ;

  }

  // if teams or score has changed, write teams and score
  if ( strlen ( teams[0].name ) != strlen ( msg->body.show.team[0].name ) ||
       strcmp ( teams[0].name, msg->body.show.team[0].name ) ||
       teams[0].score != msg->body.show.team[0].score ||
       strlen ( teams[1].name ) != strlen ( msg->body.show.team[1].name ) ||
       strcmp ( teams[1].name, msg->body.show.team[1].name ) ||
       teams[1].score != msg->body.show.team[1].score
     )
  {
    strcpy ( teams[0].name, msg->body.show.team[0].name );
    teams[0].score = msg->body.show.team[0].score;
    strcpy ( teams[1].name, msg->body.show.team[1].name );
    teams[1].score = msg->body.show.team[1].score;

    mode = htons(TEAM_MODE);
    gameLogWrite((char *)(&mode), sizeof(mode)) ;
    gameLogWrite((char *)(teams), sizeof(team_t) * 2) ;
  }

  // write positional data
  short_showinfo_t2 pos_data;
  pos_data.ball = msg->body.show.ball;
  for ( int i = 0; i < MAX_PLAYER * 2; i++ )
    pos_data.pos[i] = msg->body.show.pos[i];
  pos_data.time = msg->body.show.time;

  mode = htons(SHOW_MODE);
  gameLogWrite((char *)(&mode), sizeof(mode)) ;
  gameLogWrite((char *)(&pos_data), sizeof(pos_data)) ;
}




