/*****************************************************************************
 * SenseObjects.cc
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   class Sensors
 *
 * requires:
 *
 * Created on: 29.09.2002 by wenns
 * Maintainer: harlekin
 * Group:
 * Co-authors:
 *
 * $Id: SenseObjects.cc,v 1.6 2004/05/16 11:29:05 aw Exp $
 *****************************************************************************/

#include "SenseObjects.hh"
#include "debug.hh"

namespace Sense {
  StatObject::StatObject()
  : relPos(UNDEFINED_FLOAT, UNDEFINED_FLOAT)
  {}
  StatObject::~StatObject() {}

  Line::Line()
  : id(L_Unknown)
  {}
  Line::~Line() {}

  Vector2D MobObject::calcVel( const float dist, const float dir,
                               const float distChg, const float dirChg ){
    
    float dirChgRad = Deg2Rad(dirChg);
    
    float x = distChg * cosDeg(dir) - dirChgRad * dist * sinDeg(dir);
    float y = distChg * sinDeg(dir) + dirChgRad * dist * cosDeg(dir);
    
    return Vector2D(x, y);
  }


  MobObject::MobObject()
  : close(false), relVel(UNDEFINED_FLOAT, UNDEFINED_FLOAT)
  {}
  MobObject::~MobObject() {}

  Flag::Flag()
  : id(MARKER_UNKNOWN)
  {}
  Flag::~Flag() {}

  Goal::Goal () {}
  Goal::~Goal() {}

  Ball::Ball() {}
  Ball::~Ball() {}

  Player::Player()
  : bodyDir(UNDEFINED_FLOAT), headDir(UNDEFINED_FLOAT),
    unum(UNDEFINED_INT), goalie(false), tackling(false),
    pointDir(UNDEFINED_FLOAT), type(UNDEFINED_INT), stamina(UNDEFINED_FLOAT),
    effort(UNDEFINED_FLOAT), recovery(UNDEFINED_FLOAT)
  {
    memset(teamname, '\0', TEAMNAME_LENGTH);
  }
  Player::~Player() {}

  Score::Score ()
  : time(0), ourScore(0), oppScore()
  {}
  Score::~Score () {}

  TackleInfo::TackleInfo()
  : expires(0), count(0)
  {}
  FocusInfo::FocusInfo()
  : targetSet(false), targetSide(FS_Unknown), targetUnum(UNDEFINED_INT),
    count(0)
  {}
  ArmInfo::ArmInfo()
  : movable(0), expires(0), target(UNDEFINED_FLOAT, UNDEFINED_FLOAT),
    count(0)
  {}

  SenseBody::SenseBody()
  : time(0), viewQuality(VQ_UNKNOWN), viewWidth(VW_UNKNOWN),
    stamina(UNDEFINED_INT), effort(UNDEFINED_FLOAT),
    speedAmount(UNDEFINED_FLOAT), speedDirection(UNDEFINED_FLOAT),
    headAngle(UNDEFINED_FLOAT), kickCount(0), dashCount(0), turnCount(0),
    sayCount(0), turnNeckCount(0), catchCount(0), moveCount(0),
    changeViewCount(0)
  {}
  SenseBody::~SenseBody(){}

  Fullstate::Fullstate()
  : time(0), viewQuality(VQ_UNKNOWN), viewWidth(VW_UNKNOWN),
    playmode(PM_BeforeKickOff), ourScore(0), oppScore(0),
    kickCount(0), dashCount(0), turnCount(0), sayCount(0), turnNeckCount(0),
    catchCount(0), moveCount(0), changeViewCount(0)
  {}
  Fullstate:: ~Fullstate() {}

  ServerParam::ServerParam () {}
  ServerParam::~ServerParam () {}

  void ServerParam::setAll(
                           const int catch_ban_cycle,
                           const int clang_advice_win,
                           const int clang_define_win,
                           const int clang_del_win,
                           const int clang_info_win,
                           const int clang_mess_delay,
                           const int clang_mess_per_cycle,
                           const int clang_meta_win,
                           const int clang_rule_win,
                           const int clang_win_size,
                           const int coach_port,
                           const int drop_ball_time,
                           const int freeform_send_period,
                           const int freeform_wait_period,
                           const int game_log_compression,
                           const int game_log_version,
                           const int goalie_max_moves,
                           const int half_time,
                           const int hear_decay,
                           const int hear_inc,
                           const int hear_max,
                           const int max_goal_kicks,
                           const int olcoach_port,
                           const int point_to_ban,
                           const int point_to_duration,
                           const int port,
                           const int recv_step,
                           const int say_coach_cnt_max,
                           const int say_coach_msg_size,
                           const int say_msg_size,
                           const int send_step,
                           const int send_vi_step,
                           const int sense_body_step,
                           const int simulator_step,
                           const int slow_down_factor,
                           const int start_goal_l,
                           const int start_goal_r,
                           const int synch_micro_sleep,
                           const int synch_offset,
                           const int tackle_cycles,
                           const int text_log_compression,
                           const char *game_log_dir,
                           const char *game_log_fixed_name,
                           const char *landmark_file,
                           const char *log_date_format,
                           const char *text_log_dir,
                           const char *text_log_fixed_name,
                           const bool coach,
                           const bool coach_w_referee,
                           const bool old_coach_hear,
                           const bool wind_none,
                           const float wind_random,
                           const bool back_passes,
                           const bool forbid_kick_off_offside,
                           const bool free_kick_faults,
                           const bool fullstate_l,
                           const bool fullstate_r,
                           const bool game_log_dated,
                           const bool game_log_fixed,
                           const bool game_logging,
                           const bool log_times,
                           const bool profile,
                           const bool proper_goal_kicks,
                           const bool record_messages,
                           const bool send_comms,
                           const bool synch_mode,
                           const bool team_actuator_noise,
                           const bool text_log_dated,
                           const bool text_log_fixed,
                           const bool text_logging,
                           const bool use_offside,
                           const bool verbose,
                           const float audio_cut_dist,
                           const float ball_accel_max,
                           const float ball_decay,
                           const float ball_rand,
                           const float ball_size,
                           const float ball_speed_max,
                           const float ball_weight,
                           const float catch_probability,
                           const float catchable_area_l,
                           const float catchable_area_w,
                           const float ckick_margin,
                           const float control_radius,
                           const float dash_power_rate,
                           const float effort_dec,
                           const float effort_dec_thr,
                           const float effort_inc,
                           const float effort_inc_thr,
                           const float effort_init,
                           const float effort_min,
                           const float goal_width,
                           const float inertia_moment,
                           const float kick_power_rate,
                           const float kick_rand,
                           const float kick_rand_factor_l,
                           const float kick_rand_factor_r,
                           const float kickable_margin,
                           const float maxmoment,
                           const float maxneckang,
                           const float maxneckmoment,
                           const float maxpower,
                           const float minmoment,
                           const float minneckang,
                           const float minneckmoment,
                           const float minpower,
                           const float offside_active_area_size,
                           const float offside_kick_margin,
                           const float player_accel_max,
                           const float player_decay,
                           const float player_rand,
                           const float player_size,
                           const float player_speed_max,
                           const float player_weight,
                           const float prand_factor_l,
                           const float prand_factor_r,
                           const float quantize_step,
                           const float quantize_step_l,
                           const float recover_dec,
                           const float recover_dec_thr,
                           const float recover_min,
                           const float slowness_on_top_for_left_team,
                           const float slowness_on_top_for_right_team,
                           const float stamina_inc_max,
                           const float stamina_max,
                           const float stopped_ball_vel,
                           const float tackle_back_dist,
                           const float tackle_dist,
                           const float tackle_exponent,
                           const float tackle_power_rate,
                           const float tackle_width,
                           const float visible_angle,
                           const float visible_distance,
                           const float wind_ang,
                           const float wind_dir,
                           const float wind_force,
                           const float wind_rand
                          ){
    catchBanCycle =             catch_ban_cycle;
    clangAdviceWin =            clang_advice_win;
    clangDefineWin =            clang_define_win;
    clangDelWin =               clang_del_win;
    clangInfoWin =              clang_info_win;
    clangMessDelay =            clang_mess_delay;
    clangMessPerCycle =         clang_mess_per_cycle;
    clangMetaWin =              clang_meta_win;
    clangRuleWin =              clang_rule_win;
    clangWinSize =              clang_win_size;
    coachPort =                 coach_port;
    dropBallTime =              drop_ball_time;
    freeformSendPeriod =        freeform_send_period;
    freeformWaitPeriod =        freeform_wait_period;
    gameLogCompression =        game_log_compression;
    gameLogVersion =            game_log_version;
    goalieMaxMoves =            goalie_max_moves;
    halfTime =                  half_time;
    hearDecay =                 hear_decay;
    hearInc =                   hear_inc;
    hearMax =                   hear_max;
    maxGoalKicks =              max_goal_kicks;
    olcoachPort =               olcoach_port;
    pointToBan =                point_to_ban;
    pointToDuration =           point_to_duration;
    this->port =                port;
    recvStep =                  recv_step;
    sayCoachCntMax =            say_coach_cnt_max;
    sayCoachMsgSize =           say_coach_msg_size;
    sayMsgSize =                say_msg_size;
    sendStep =                  send_step;
    sendViStep =                send_vi_step;
    senseBodyStep =             sense_body_step;
    simulatorStep =             simulator_step;
    slowDownFactor =            slow_down_factor;
    startGoalL =                start_goal_l;
    startGoalR =                start_goal_r;
    synchMicroSleep =           synch_micro_sleep;
    synchOffset =               synch_offset;
    tackleCycles =              tackle_cycles;
    textLogCompression =        text_log_compression;
    gameLogDir =                strdup(game_log_dir);
    gameLogFixedName =          strdup(game_log_fixed_name);
    landmarkFile =              strdup(landmark_file);
    logDateFormat =             strdup(log_date_format);
    textLogDir =                strdup(text_log_dir);
    textLogFixedName =          strdup(text_log_fixed_name);
    this->coach =               coach;
    coachWReferee =             coach_w_referee;
    oldCoachHear =              old_coach_hear;
    windNone =                  wind_none;
    windRandom =                wind_random;
    backPasses =                back_passes;
    forbidKickOffOffside =      forbid_kick_off_offside;
    freeKickFaults =            free_kick_faults;
    fullstateL =                fullstate_l;
    fullstateR =                fullstate_r;
    gameLogDated =              game_log_dated;
    gameLogFixed =              game_log_fixed;
    gameLogging =               game_logging;
    logTimes =                  log_times;
    this->profile =             profile;
    properGoalKicks =           proper_goal_kicks;
    recordMessages =            record_messages;
    sendComms =                 send_comms;
    synchMode =                 synch_mode;
    teamActuatorNoise =         team_actuator_noise;
    textLogDated =              text_log_dated;
    textLogFixed =              text_log_fixed;
    textLogging =               text_logging;
    useOffside =                use_offside;
    this->verbose =                   verbose;
    audioCutDist =              audio_cut_dist;
    ballAccelMax =              ball_accel_max;
    ballDecay =                 ball_decay;
    ballRand =                  ball_rand;
    ballSize =                  ball_size;
    ballSpeedMax =              ball_speed_max;
    ballWeight =                ball_weight;
    catchProbability =          catch_probability;
    catchableAreaL =            catchable_area_l;
    catchableAreaW =            catchable_area_w;
    ckickMargin =               ckick_margin;
    controlRadius =             control_radius;
    dashPowerRate =             dash_power_rate;
    effortDec =                 effort_dec;
    effortDecThr =              effort_dec_thr;
    effortInc =                 effort_inc;
    effortIncThr =              effort_inc_thr;
    effortInit =                effort_init;
    effortMin =                 effort_min;
    goalWidth =                 goal_width;
    inertiaMoment =             inertia_moment;
    kickPowerRate =             kick_power_rate;
    kickRand =                  kick_rand;
    kickRandFactorL =           kick_rand_factor_l;
    kickRandFactorR =           kick_rand_factor_r;
    kickableMargin =            kickable_margin;
    this->maxmoment =           maxmoment;
    this->maxneckang =          maxneckang;
    this->maxneckmoment =       maxneckmoment;
    this->maxpower =            maxpower;
    this->minmoment =           minmoment;
    this->minneckang =          minneckang;
    this->minneckmoment =       minneckmoment;
    this->minpower =            minpower;
    offsideActiveAreaSize =     offside_active_area_size;
    offsideKickMargin =         offside_kick_margin;
    playerAccelMax =            player_accel_max;
    playerDecay =               player_decay;
    playerRand =                player_rand;
    playerSize =                player_size;
    playerSpeedMax =            player_speed_max;
    playerWeight =              player_weight;
    prandFactorL =              prand_factor_l;
    prandFactorR =              prand_factor_r;
    quantizeStep =              quantize_step;
    quantizeStepL =             quantize_step_l;
    recoverDec =                recover_dec;
    recoverDecThr =             recover_dec_thr;
    recoverMin =                recover_min;
    slownessOnTopForLeftTeam =  slowness_on_top_for_left_team;
    slownessOnTopForRightTeam = slowness_on_top_for_right_team;
    staminaIncMax =             stamina_inc_max;
    staminaMax =                stamina_max;
    stoppedBallVel =            stopped_ball_vel;
    tackleBackDist =            tackle_back_dist;
    tackleDist =                tackle_dist;
    tackleExponent =            tackle_exponent;
    tacklePowerRate =           tackle_power_rate;
    tackleWidth =               tackle_width;
    visibleAngle =              visible_angle;
    visibleDistance =           visible_distance;
    windAng =                   wind_ang;
    windDir =                   wind_dir;
    windForce =                 wind_force;
    windRand =                  wind_rand;
  }

  Audio::Audio ()
  : time(0)
  {}
  Audio::~Audio () {}

  RefAudio::RefAudio ()
  : playmode(PM_BeforeKickOff), goals(0)
  {
    memset(unknownRefAudio, '\0', msgLength);
  }
  RefAudio::~RefAudio () {}

  SelfAudio::SelfAudio () {
    memset(msg, '\0', msgLength);
  }
  SelfAudio::~SelfAudio (){}

  PlayerAudio::PlayerAudio ()
  : dir(UNDEFINED_FLOAT), unum(UNDEFINED_INT), side(S_Unknown)
  {}
  PlayerAudio::~PlayerAudio (){}

  CoachAudio::CoachAudio (){
    memset(msg, '\0', msgLength);
  }
  CoachAudio::~CoachAudio (){}

  Error::Error ()
  : type(E_UnknownError)
  {
    memset(unknownErrorMsg, '\0', msgLength);
  }
  Error::~Error () {}

  Warning::Warning ()
  : type(W_UnknownWarning)
  {
    memset(unknownWarningMsg, '\0', msgLength);
  }
  Warning::~Warning () {}

  InitMsg::InitMsg ()
  : side(S_Unknown), unum(UNDEFINED_INT), playmode(PM_BeforeKickOff)
  {}
  InitMsg::~InitMsg () {}

  PlayerParam::PlayerParam (){}
  PlayerParam::~PlayerParam () {}

  void PlayerParam::setAll(
                           int player_types,
                           int pt_max,
                           float random_seed,
                           int subs_max,
                           float dash_power_rate_delta_max,
                           float dash_power_rate_delta_min,
                           float effort_max_delta_factor,
                           float effort_min_delta_factor,
                           float extra_stamina_delta_max,
                           float extra_stamina_delta_min,
                           float inertia_moment_delta_factor,
                           float kick_rand_delta_factor,
                           float kickable_margin_delta_max,
                           float kickable_margin_delta_min,
                           float new_dash_power_rate_delta_max,
                           float new_dash_power_rate_delta_min,
                           float new_stamina_inc_max_delta_factor,
                           float player_decay_delta_max,
                           float player_decay_delta_min,
                           float player_size_delta_factor,
                           float player_speed_max_delta_max,
                           float player_speed_max_delta_min,
                           float stamina_inc_max_delta_factor
                          )
  {
    playerTypes =                 player_types;
    ptMax =                       pt_max;
    randomSeed =                  random_seed;
    subsMax =                     subs_max;
    dashPowerRateDeltaMax =       dash_power_rate_delta_max;
    dashPowerRateDeltaMin =       dash_power_rate_delta_min;
    effortMaxDeltaFactor =        effort_max_delta_factor;
    effortMinDeltaFactor =        effort_min_delta_factor;
    extraStaminaDeltaMax =        extra_stamina_delta_max;
    extraStaminaDeltaMin =        extra_stamina_delta_min;
    inertiaMomentDeltaFactor =    inertia_moment_delta_factor;
    kickRandDeltaFactor =         kick_rand_delta_factor;
    kickableMarginDeltaMax =      kickable_margin_delta_max;
    kickableMarginDeltaMin =      kickable_margin_delta_min;
    newDashPowerRateDeltaMax =    new_dash_power_rate_delta_max;
    newDashPowerRateDeltaMin =    new_dash_power_rate_delta_min;
    newStaminaIncMaxDeltaFactor = new_stamina_inc_max_delta_factor;
    playerDecayDeltaMax =         player_decay_delta_max;
    playerDecayDeltaMin =         player_decay_delta_min;
    playerSizeDeltaFactor =       player_size_delta_factor;
    playerSpeedMaxDeltaMax =      player_speed_max_delta_max;
    playerSpeedMaxDeltaMin =      player_speed_max_delta_min;
    staminaIncMaxDeltaFactor =    stamina_inc_max_delta_factor;
  }


  PlayerType::PlayerType () {}
  PlayerType::~PlayerType () {}

  CoachFreeform::CoachFreeform () {
    formation = -1;
    offsideX = 0;
    for (int i = 0; i < 11; i++){
      playerTypes[i] = 0;
    }
  }
  CoachFreeform::~CoachFreeform () {}

  void PlayerType::setAll(
                          const int id,
                          const float player_speed_max,
                          const float stamina_inc_max,
                          const float player_decay,
                          const float inertia_moment,
                          const float dash_power_rate,
                          const float player_size,
                          const float kickable_margin,
                          const float kick_rand,
                          const float extra_stamina,
                          const float effort_max,
                          const float effort_min
                         )
  {
    this->id =       id;
    playerSpeedMax = player_speed_max;
    staminaIncMax =  stamina_inc_max;
    playerDecay =    player_decay;
    inertiaMoment =  inertia_moment;
    dashPowerRate =  dash_power_rate;
    playerSize =     player_size;
    kickableMargin = kickable_margin;
    kickRand =       kick_rand;
    extraStamina =   extra_stamina;
    effortMax =      effort_max;
    effortMin =      effort_min;
    if (playerDecay == 1.0)
      playerSpeedMaxAct = playerSpeedMax;
    else{
      playerSpeedMaxAct = min((float)((dashPowerRate * effortMax * 100.0)/
			      (1.0 - playerDecay)), playerSpeedMax);
    }
  }


  Substitution::Substitution ()
  : unum(UNDEFINED_INT), type(UNDEFINED_INT)
  {}
  Substitution::~Substitution () {}

}; //Sense


