/*****************************************************************************
 * SenseThread.cc
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   namespace SenseThread
 *
 * requires:
 *
 * Created on: 21.04.2002 by wenns
 * Maintainer: mosh
 * Group:
 * Co-authors: harlekin
 *
 *
 * $Id: SenseThread.cc,v 1.17 2004/05/07 11:52:38 mosh Exp $
 *****************************************************************************/

#include "SenseThread.hh"

#include "WorldModel.hh"
#include "Sensors.hh"
#include "debug.hh"
#include "ConfigFile.hh"


namespace SenseThread{
  WorldModel      *wm;
  Sensors         *sensors;
  struct itimerval itv;
  bool             fsUsed;  

  /**
   * adds an amount af millisecs to a timeval
   * @param summand1 The timeval, to which an amount of millisec should be
   * added
   * @param msec the amount of millisec, that should be added.
   * @return the sum as timeval
   */
  struct timeval timerAddMsec(const struct timeval *summand1, int msec);

  /**
   * this function sets the OS-timer for the SIGALRM signal depending on the
   * time of last sense body, time of last see and the current see-interval.
   */
  void setTimeSignal();

  /*
   * calculates the see time intervall according to a certain view quality
   * and a certain view mode
   * @param vq should be set to the current ViewQuality of the agent
   * @param vm should be set to the current ViewMode of the agent
   * @return the time interval of the see messages corresponding to the values of
   * the parameters
   */
  int calculateSeeInterval(ViewQuality vq, ViewWidth vw);
  void calculateHoles();
};


struct timeval SenseThread::timerAddMsec(const struct timeval *summand1,
                                         const int msec){
  static timeval sum, summand2;
  summand2.tv_sec = 0;
  summand2.tv_usec = MILLI_TO_MICRO(msec);
  timeradd(summand1, &summand2, &sum);

  return sum;
}

/* old Method
void *SenseThread::senseCallback(void *p){
  Param *param = (Param*)p;
  wm      = (WorldModel*)param->vp1;
  sensors = (Sensors*)param->vp2;
  fsUsed  = ConfigFile::useFullstate;

  itv.it_interval.tv_sec  = 0;
  itv.it_interval.tv_usec = 0;
  itv.it_value.tv_sec     = 0;
  itv.it_value.tv_usec    = 0;

  while ( true ){
    sensors->sense();
    // only used for synchronized mode
    if( sensors->thinkReceived() ) {
      // Every message for this cycle has arrived. Start thinking!
      wm->setThinkSegment();
      // tell the worldmodel, that a new message has arrived and that the
      // agent should do something now
      wm->signalNewInfo();
      DEBUG1_SYNCHRO("[SENSE] got think");
    }
    if( sensors->sbReceived() ){
      wm->setTimeLastSB();
      if ( !Configuration::synchMode ) {
	setTimeSignal();
      }
      wm->signalNewInfo();
    }
    if( sensors->seeReceived() && !fsUsed ) //wake up on see only if fullstate
                                            //should not be used
      wm->setTimeLastSee();
    if( sensors->fsReceived() && fsUsed ){
      DEBUG1_SYNCHRO("[SENSE] \tgot fs");
      wm->signalNewInfo();
    }
    if( sensors->initReceived() ){
      DEBUG1_SYNCHRO("[SENSE] \tgot init");
      wm->signalNewInfo();
    }
  }
  return NULL;
}
*/
static bool oldSet=false;
static Time oldCycle;
void *SenseThread::senseCallback(void *p){
  Param *param = (Param*)p;
  wm      = (WorldModel*)param->vp1;
  sensors = (Sensors*)param->vp2;
  fsUsed  = ConfigFile::useFullstate;
 //to keep them the whole cycle
      itv.it_interval.tv_sec  = 0;
      itv.it_interval.tv_usec = 0;
      itv.it_value.tv_sec     = 0;
      itv.it_value.tv_usec    = 0;
      
 
  while ( true ){
    sensors->sense();
    // only used for synchronized mode
    if( sensors->thinkReceived() ) {
      // Every message for this cycle has arrived. Start thinking!
      wm->setThinkSegment();
      // tell the worldmodel, that a new message has arrived and that the
      // agent should do something now
      wm->signalNewInfo();
    }
    if( sensors->sbReceived() ){
      wm->setTimeLastSB();
      if ( !Configuration::synchMode ) {
	setTimeSignal();
      }
      wm->signalNewInfo();
      
      calculateHoles();
    }
    
    if( sensors->seeReceived() && !fsUsed ){ //wake up on see only if fullstate
                                            //should not be used
      wm->setTimeLastSee();
      Time now = *(wm->getTime());

      ViewQuality vq = wm->myself.bodyInfo.getViewQuality();
      ViewWidth   vw = wm->myself.bodyInfo.getViewWidth();
      
      if(vq == VQ_HIGH && vw == VW_NORMAL){
	if((now.getCycles()-oldCycle.getCycles() )==1){
	  setTriStep(2);
	}
	if((now.getCycles()-oldCycle.getCycles() )>=2 || !oldSet){
	  setTriStep(1);
	  oldSet=true;
	}
      }
      
      if(vq == VQ_HIGH && vw == VW_NARROW){
	if((now.getCycles()-oldCycle.getCycles() )==0 || !oldSet){
	  setTriStep(1);
	}
	if((now.getCycles()-oldCycle.getCycles() )==1 || !oldSet){
	  setTriStep(3);

	}
      }
      
      //int diff= timerDiffMsec( wm->getTimeLastSb(), wm->getTimeLastSee()); // millis
      oldCycle = *(wm->getTime());
      // DEBUG2_MOSH("---->SEE DIFF",diff);
    }

    if( sensors->fsReceived() && fsUsed ){
      wm->signalNewInfo();
    }

    if( sensors->initReceived() ){
      wm->signalNewInfo();
    }

  }
  return NULL;
}

/*   old Method 
void SenseThread::setTimeSignal(){
  static float noSeeFraction     = ConfigFile::noSeeFraction;
  static float firstHalfFraction = ConfigFile::firstHalfFraction;
  static float lastHalfFraction  = ConfigFile::lastHalfFraction;

  static const float    CYCLE    = 100000;  //ms
  static float          fraction;
  static ViewQuality    vq;
  static ViewWidth      vw;
  static int            seeInterval, tillNextSee;
  static struct timeval timeLastSee, timeLastSb, timeNextSee;

  if( fsUsed )
    //if fullstate information should be used
    fraction = noSeeFraction;
  else{
    vq = wm->myself.bodyInfo.getViewQuality();
    vw = wm->myself.bodyInfo.getViewWidth();
    seeInterval = calculateSeeInterval(vq, vw);

    timeLastSee = *(wm->getTimeLastSee());
    timeLastSb  = *(wm->getTimeLastSb());

    timeNextSee =  timerAddMsec(&timeLastSee, seeInterval);
    tillNextSee =  timerDiffMsec(&timeNextSee, &timeLastSb);

    if(tillNextSee > 100)      //no see this cycle
      //as sure as possible
      fraction = noSeeFraction;
    else if (tillNextSee > 50) //see in last half
      //al late as possible
      fraction = lastHalfFraction;
    else                       //see in first half
      //maybe no sigalrm?
      fraction = firstHalfFraction;
  }


  itv.it_value.tv_usec = (long int)(CYCLE * fraction);
  setitimer( ITIMER_REAL, &itv, NULL );
  DEBUG1_SYNCHRO("[SENSE] \tsetTimeSignal(): timer set");
}
*/
static int triStep=1;
void SenseThread::setTriStep(int step){
  triStep=step;
}

static float lastHalfFraction = 0;
void SenseThread::calculateHoles(){
  static int nrHolesLastTime  = 0;
  static int nrHolesLastCycle = 0;
  static Time timeBeginInterval = Time(0,0);
  static Time timeLastHole      = Time(0,0);
  static int holes = 0;

  int counts = wm->myself.bodyInfo.getImportantCounts();
  int queued = wm->getQueuedCommandCounters().getImportant();

  int nrHoles  = queued - counts;
  int newHoles = nrHoles - nrHolesLastCycle;
  if (newHoles > 0){
    timeLastHole = *(wm->getTime());
    holes++;
    DEBUG1_HARLEKIN("SenseThread::calculateHoles(): New Hole");
  }
  nrHolesLastCycle = nrHoles;

  int    iTimeDiff = (*(wm->getTime()) - timeBeginInterval).getCycles();
  double dHolePerc = (double)(holes - nrHolesLastTime)/iTimeDiff * 100;
  if (/*!isLastMessageSee() &&*/ (iTimeDiff > 400) && dHolePerc > 2.0 && lastHalfFraction > 0.7){
    lastHalfFraction -= 0.05;
    timeBeginInterval = *(wm->getTime());
    DEBUG8_HARLEKIN("SenseThread::calculateHoles(): Cycles: ", iTimeDiff ," Holes: ", holes, " lowering to: ", lastHalfFraction, " blah: ", (((float)(holes-nrHolesLastTime))/(float)iTimeDiff));
    nrHolesLastTime = holes;
  }
  //DEBUG4_HARLEKIN("Holes: ", holes, " dHolePerc: ", dHolePerc);
}

void SenseThread::setTimeSignal(){
  if (lastHalfFraction == 0){
    lastHalfFraction = ConfigFile::lastHalfFraction;
  }
  static float noSeeFraction     = ConfigFile::noSeeFraction;
  static float firstHalfFraction = ConfigFile::firstHalfFraction;
  //static double lastHalfFraction  = ConfigFile::lastHalfFraction;

  // How long does a cycle take
  double  cycle_length = Configuration::simulatorStep * 1000 * 
                         Configuration::slowDownFactor;  //micros
  static float          fraction;
  //static ViewQuality    vq;
  //static ViewWidth      vw;
  static int            seeInterval, tillNextSee; // millis
  static struct timeval timeLastSee, timeLastSb, timeNextSee; // micros


  ViewQuality vq = wm->myself.bodyInfo.getViewQuality();
  ViewWidth   vw = wm->myself.bodyInfo.getViewWidth();

  if( fsUsed ){
    //if fullstate information should be used
    fraction = lastHalfFraction;
  }else{
    //vq = wm->myself.bodyInfo.getViewQuality();
    //vw = wm->myself.bodyInfo.getViewWidth();
    seeInterval = calculateSeeInterval(vq, vw); // milis
    
    timeLastSee = *(wm->getTimeLastSee()); // micros
    timeLastSb  = *(wm->getTimeLastSb());  // micros

    timeNextSee =  timerAddMsec(&timeLastSee, seeInterval); // micros
    tillNextSee =  timerDiffMsec(&timeNextSee, &timeLastSb); // millis
    
    //fraction = noSeeFraction;
    
    /*
    if(tillNextSee > Configuration::simulatorStep * Configuration::slowDownFactor)      //no see this cycle
      //as soon as possible
      fraction = noSeeFraction;
    else if (tillNextSee > Configuration::simulatorStep * Configuration::slowDownFactor / 2) //see in last half
      //al late as possible
      fraction = lastHalfFraction;
    else                       //see in first half
      //maybe no sigalrm?
      fraction = firstHalfFraction;
    */

    fraction = lastHalfFraction;
      
    if(triStep == 3)  {    //see in firstHalf 
      //DEBUG1_MOSH("seeFirst");
      fraction = firstHalfFraction;     
    }else if (triStep == 2){ //no see  
      fraction = noSeeFraction;
      //DEBUG1_MOSH("noSee");
    }else {                       //see in last half
      fraction = lastHalfFraction;
      //   DEBUG1_MOSH("lastHalf");    
    }    

    if(vq == VQ_HIGH && vw == VW_NARROW){
      if(triStep==3)
	fraction = firstHalfFraction;
      else
	fraction = lastHalfFraction;
    }

    triStep++;
    if(triStep >= 4 || triStep < 1)
      triStep = 1;
  
  }

  itv.it_value.tv_usec = (long int)(cycle_length * fraction); // micros
  //DEBUG2_HARLEKIN("timer : ",itv.it_value.tv_usec);
  setitimer( ITIMER_REAL, &itv, NULL );
  DEBUG2_SYNCHRO("[SENSE] \tsetTimeSignal(): timer set:", itv.it_value.tv_usec);
}


int SenseThread::calculateSeeInterval(ViewQuality vq, ViewWidth vw){
  float viewQualFactor  = 1.0;
  float viewWidthFactor = 1.0;

  switch( vw ){
  case VW_NARROW:  viewWidthFactor = 0.5; break;
  case VW_NORMAL:  viewWidthFactor = 1.0; break;
  case VW_WIDE:    viewWidthFactor = 2.0; break;
  case VW_UNKNOWN:
  default:
    DEBUG1("SenseThread::calculateSeeInterval(): ViewWidth unknown");
  }

  switch( vq ){
  case VQ_LOW:     viewQualFactor = 0.5; break;
  case VQ_HIGH:    viewQualFactor = 1.0; break;
  case VQ_UNKNOWN:
  default:
    DEBUG1("SenseThread::calculateSeeInterval(): ViewQuality unknown");
  }
  float step = (float)(Configuration::slowDownFactor * Configuration::sendStep);
  int interval = (int)(step * viewQualFactor * viewWidthFactor);
  //DEBUG2_HARLEKIN("Interval: ", interval);
  return interval; // Milliseconds
}











