/***********************************************************************
 * WorldModelLogger.cc
 * Copyright (c) 2003 Virtual Werder
 *
 * implements:
 *   class WorldModelLogger
 *
 * requires:
 *
 *
 * 3B
 * Created on: 20.10.2003 by harlekin
 * Maintainer: harlekin
 * Group:
 * Co-authors:
 *
 * Changelog:
 *
 * $Id: WorldModelLogger.cc,v 1.6 2004/04/03 07:58:14 harlekin Exp $
 *
 **********************************************************************/

#include "WorldModelLogger.hh"
#include "WorldModel.hh"

// static members
WorldModelLogger *WorldModelLogger::wml = NULL;

WorldModelLogger::WorldModelLogger(WorldModel *worldModel) {
  wm = worldModel;
  ms = &(worldModel->myself);
  uNum = ms->getUniformNumber(); 
  openFile();
}

void WorldModelLogger::openFile() {
  using namespace std;
  stringstream tempName;
  tempName << "./worldmodellogs/worldmodel-player-" << uNum << ".xml";
  const char* const filename = tempName.str().c_str();
  // open file
  //out.open(filename, ios_base::out | ios_base::ate); !works with g++-3.2 only!
  out.open(filename, ( ofstream::out | ofstream::trunc) );
}

void WorldModelLogger::writeCurrentCycle(){
  if ( (!out.is_open()) ) return;
  out << "<wm>" << endl
      << "<cycle>" << wm->getTime()->getTime() << "</cycle>" << endl;
  out << "<sc>" << wm->getTime()->getStoppedCount() << "</sc>" << endl;
  writeBall(&(wm->ball));
  // TODO: Decide which side
  writeTeam(wm->myTeam.getName(),    "left",  &(wm->myTeam));
  writeTeam(wm->theirTeam.getName(), "right", &(wm->theirTeam));
  out << "<ox>" << wm->getOffsideX() << "</ox>" << endl;
  out << "</wm>" << endl;
  out.flush();
}

void WorldModelLogger::writeBall(Ball *ball){
  out << "<ball>" << endl
      << "<v>" << endl
      << "<x>" << ball->getVelocity().getX() << "</x>" << endl
      << "<y>" << ball->getVelocity().getY() << "</y>" << endl
      << "<c>" << ball->getVelConfidence( wm->getTime() ) << "</c>" << endl
      << "</v>" << endl
      << "<pos>" << endl
      << "<x>" << ball->getPosition().getX() << "</x>" << endl
      << "<y>" << ball->getPosition().getY() << "</y>" << endl
      << "<c>" << ball->getConfidence( wm->getTime() ) << "</c>" << endl
      << "</pos>" << endl
      << "</ball>" << endl;
}

void WorldModelLogger::writePlayer( Player *player ){
  out << "<p>" << endl 
      << "<unum>" << player->getUniformNumber() << "</unum>" << endl
      << "<v>" << endl
      << "<x>" << player->getVelocity().getX() << "</x>" << endl
      << "<y>" << player->getVelocity().getY() << "</y>" << endl
      << "<c>" << player->getVelConfidence( wm->getTime() ) << "</c>" << endl
      << "</v>" << endl
      << "<pos>" << endl
      << "<x>" << player->getPosition().getX() << "</x>" << endl
      << "<y>" << player->getPosition().getY() << "</y>" << endl
      << "<c>" << player->getConfidence( wm->getTime() ) << "</c>" << endl
      << "</pos>" << endl
      << "<d>" << endl
      << "<he>" << player->getHeadDir() << "</he>" << endl
      << "<bo>" << player->getBodyDir() << "</bo>" << endl
      << "</d>" << endl
      << "<g>" << (player->isGoalKeeper() ? 1 : 0) << "</g>" << endl
      << "<pt>" << player->getType() << "</pt>" << endl
      << "</p>" << endl;
  
}

void WorldModelLogger::writeTeam(const char* teamname,const char* teamside, Team *team ) {
  if ( (!out.is_open()) ) return;

  out << "<t>" << endl
      << "<name>" << teamname << "</name>" << endl
      << "<side>" << teamside << "</side>" << endl;
  vector<Player>::iterator i;
  vector<Player> players = team->getPlayers();
  for(i = players.begin(); i != players.end(); ++i){
    writePlayer(&(*i));
  }
  out << "</t>" << endl;
}

void WorldModelLogger::writeLeadIn(){
  if ( (!out.is_open()) ) return;
  out << "<game>" << endl 
      << "<worldmodels>" << endl
      << "<activeplayer>" << uNum << "</activeplayer>" << endl;
  out.flush();
}
void WorldModelLogger::writeLeadOut(){
  if ( (!out.is_open()) ) return;
  out << "</worldmodels>" << endl 
      << "</game>" << endl;
  out.flush();
}
