#include <sstream>

#include "ConfFileData.hh"

using std::ostringstream;


deque<string> ConfFileData::headerLines;
deque<string> ConfFileData::codeLines;
deque<string> ConfFileData::keywordLines;
deque<string> ConfFileData::tokenLines;
deque<string> ConfFileData::ruleLines;
deque<string> ConfFileData::defaultLines;

map<string,string> ConfFileData::vars;


void
ConfFileData::addVariableLong  (const char *name, long        defaultvalue)
{
  map<string,string>::iterator i;

  if( (i=vars.find(name))==vars.end() )
  {
     addHeaderLineLong(name);
       addCodeLineLong(name,defaultvalue);
    addKeywordLineLong(name);
       addRuleLineLong(name);
    addDefaultLineLong(name,defaultvalue);

    vars[name]="LONG";
  }
  else
    cerr << "ERROR: \"" << name << "\" was previously defined (as LONG)!"
	 << endl;
}
void
ConfFileData::addVariableDouble(const char *name, double      defaultvalue)
{
  map<string,string>::iterator i;

  if( (i=vars.find(name))==vars.end() )
  {
     addHeaderLineDouble(name);
       addCodeLineDouble(name,defaultvalue);
    addKeywordLineDouble(name);
       addRuleLineDouble(name);
    addDefaultLineDouble(name,defaultvalue);

    vars[name]="DOUBLE";
  }
  else
    cerr << "ERROR: \"" << name << "\" was previously defined (as DOUBLE)!"
	 << endl;
}
void
ConfFileData::addVariableString(const char *name, const char *defaultvalue)
{
  map<string,string>::iterator i;

  if( (i=vars.find(name))==vars.end() )
  {
     addHeaderLineString(name);
       addCodeLineString(name,defaultvalue);
    addKeywordLineString(name);
       addRuleLineString(name);
    addDefaultLineString(name,defaultvalue);

    vars[name]="STRING";
  }
  else
    cerr << "ERROR: \"" << name << "\" was previously defined (as STRING)!"
	 << endl;
}
void
ConfFileData::addComment(const char *text)
{
     addHeaderLineComment(text);
       addCodeLineComment(text);
    addKeywordLineComment(text);
       addRuleLineComment(text);
    addDefaultLineComment(text);
}
void
ConfFileData::addEmpty()
{
     addHeaderLineEmpty();
       addCodeLineEmpty();
    addKeywordLineEmpty();
       addRuleLineEmpty();
    addDefaultLineEmpty();
}


void
ConfFileData::dumpHeaderLines(ostream &out,
			      const char *datestr, const char *argv0)
{
  out << "// ****************************************************\n"
      << "// **** created " << datestr
      << "// ****  by     " << argv0 << "\n"
      << "\n"
      << "#ifndef CONFIGFILE_HH\n"
      << "#define CONFIGFILE_HH\n"
      << "\n"
      << "#include <string>\n"
      << "\n"
      << "class ConfigFile\n"
      << "{\n"
      << "public:\n"
      << endl;

  for(deque<string>::iterator i=headerLines.begin();
      i<headerLines.end();
      ++i)
    out << *i << endl;

  out << "\n"
      << "\n"
      << "    ConfigFile();\n"
      << "\n"
      << "    static bool parseConfigFile(const char *new_filename,\n"
      << "                                bool verboseScanner=false,\n"
      << "                                bool verboseParser=false);\n"
      << "};\n"
      << "\n"
      << "#endif   //ConfigFile\n"
      << "\n"
      << "// ****************************************************" << endl;
}
void
ConfFileData::dumpCodeLines(ostream &out,
			    const char *datestr, const char *argv0)
{
  out << "// ****************************************************\n"
      << "// **** created " << datestr
      << "// ****  by     " << argv0 << "\n"
      << "#include <stdio.h>\n"
      << "#include \"ConfigFile.hh\"\n"
      << "extern void yyrestart( FILE *new_file );\n"
      << "extern int readConfFile_parse(void);\n"
      << "extern int yy_flex_debug, readConfFile_debug;\n"
      << "\n"
      << endl;

  for(deque<string>::iterator i=codeLines.begin();
      i<codeLines.end();
      ++i)
    out << *i << endl;

  out << "\n"
      << "\n"
      << "ConfigFile::ConfigFile()\n"
      << "{}\n"
      << "\n"
      << "bool ConfigFile::parseConfigFile(const char *new_filename,\n"
      << "                                 bool verboseScanner/*=false*/,\n"
      << "                                 bool verboseParser/*=false*/)\n"
      << "{\n"
      << "  FILE *new_file;\n"
      << "  bool rc=false;\n"
      << "  \n"
      << "  if((new_file=fopen(new_filename,\"r\")))\n"
      << "  {\n"
      << "    yyrestart(new_file);\n"
      << "    yy_flex_debug=verboseScanner;\n"
      << "    readConfFile_debug=verboseParser;\n"
      << "    rc=(readConfFile_parse()==0);\n"
      << "    fclose(new_file);\n"
      << "  }\n"
      << "  return rc;\n"
      << "}\n"
      << "// ****************************************************" << endl;
}
void
ConfFileData::dumpKeywordLines(ostream &out)
{
  for(deque<string>::iterator i=keywordLines.begin();
      i<keywordLines.end();
      ++i)
    out << *i << endl;
}
void
ConfFileData::dumpRuleLines(ostream &out)
{
  for(deque<string>::iterator i=tokenLines.begin();
      i<tokenLines.end();
      ++i)
    out << *i << endl;

  out << "\n%%\n" << endl;

  for(deque<string>::iterator i=ruleLines.begin();
      i<ruleLines.end();
      ++i)
    out << *i << endl;

  out << "\t\t;\n";  //wenns
}
void
ConfFileData::dumpDefaultLines(ostream &out,
			       const char *datestr, const char *argv0)
{
  out << "# **** created " << datestr
      << "# **** by      " << argv0 << endl;

  for(deque<string>::iterator i=defaultLines.begin();
      i<defaultLines.end();
      ++i)
    out << *i << endl;
}


//*****************************************************************************
void
ConfFileData::addHeaderLineLong(const char *name)
{
  ostringstream line;

  line << "    static long   " << name << ";";
  headerLines.push_back(line.str());
}
void
ConfFileData::addHeaderLineDouble(const char *name)
{
  ostringstream line;

  line << "    static double " << name << ";";
  headerLines.push_back(line.str());
}
void
ConfFileData::addHeaderLineString(const char *name)
{
  ostringstream line;

  line << "    static std::string " << name << ";";
  headerLines.push_back(line.str());
}
void
ConfFileData::addHeaderLineComment(const char *text)
{
  ostringstream line;

  line << "    //" << text;
  headerLines.push_back(line.str());
}
void
ConfFileData::addHeaderLineEmpty()
{
  ostringstream line;

  //line;
  headerLines.push_back(line.str());
}


void
ConfFileData::addCodeLineLong(const char *name, long         defaultvalue)
{
  ostringstream line;

  line << "long   ConfigFile::" << name << "\t= "
       << defaultvalue << ";";
  codeLines.push_back(line.str());
}
void
ConfFileData::addCodeLineDouble(const char *name, double     defaultvalue)
{
  ostringstream line;

  line << "double ConfigFile::" << name << "\t= "
       << defaultvalue << ";";
  codeLines.push_back(line.str());
}
void
ConfFileData::addCodeLineString(const char *name, const char *defaultvalue)
{
  ostringstream line;

  line << "std::string ConfigFile::" << name << "\t= "
       << defaultvalue << ";";
  codeLines.push_back(line.str());
}
void
ConfFileData::addCodeLineComment(const char *text)
{
  ostringstream line;

  line << "//" << text;
  codeLines.push_back(line.str());
}
void
ConfFileData::addCodeLineEmpty()
{
  ostringstream line;

  //line;
  codeLines.push_back(line.str());
}


void
ConfFileData::addKeywordLineLong(const char *name)
{
  ostringstream line;

  line << name << "\t\t{ return KW_" << name << "; }";
  keywordLines.push_back(line.str());
}
void
ConfFileData::addKeywordLineDouble(const char *name)
{
  ostringstream line;

  line << name << "\t\t{ return KW_" << name << "; }";
  keywordLines.push_back(line.str());
}
void
ConfFileData::addKeywordLineString(const char *name)
{
  ostringstream line;

  line << name << "\t\t{ return KW_" << name << "; }";
  keywordLines.push_back(line.str());
}
void
ConfFileData::addKeywordLineComment(const char *text)
{
  ostringstream line;

  line << "  /*" << text << " */";
  keywordLines.push_back(line.str());
}
void
ConfFileData::addKeywordLineEmpty()
{
  ostringstream line;

  //line;
  keywordLines.push_back(line.str());
}


void
ConfFileData::addRuleLineLong(const char *name)
{
  ostringstream tline;
  ostringstream rline;

  tline << "%token\t<lval>\tKW_" << name;
  tokenLines.push_back(tline.str());

  if(ruleLines.empty())
    rline << "configline:\t  KW_" << name << " COLON LONG\n"
	  << "\t\t{ ConfigFile::" << name << "=$3; }"
	 ;
  else
    rline << "\t\t| KW_" << name << " COLON LONG\n"
	  << "\t\t{ ConfigFile::" << name << "=$3; }"
	 ;
  ruleLines.push_back(rline.str());
}
void
ConfFileData::addRuleLineDouble(const char *name)
{
  ostringstream tline;
  ostringstream rline;

  tline << "%token\t<lval>\tKW_" << name;
  tokenLines.push_back(tline.str());

  if(ruleLines.empty())
    rline << "configline:\t  KW_" << name << " COLON DOUBLE\n"
	  << "\t\t{ ConfigFile::" << name << "=$3; }\n"
	  << "\t\t| KW_" << name << " COLON LONG\n"
	  << "\t\t{ ConfigFile::" << name << "=$3; }"
         ; // double accepts DOUBLE _and_ LONG
  else
    rline << "\t\t| KW_" << name << " COLON DOUBLE\n"
	  << "\t\t{ ConfigFile::" << name << "=$3; }\n"
	  << "\t\t| KW_" << name << " COLON LONG\n"
	  << "\t\t{ ConfigFile::" << name << "=$3; }"
	 ; // double accepts DOUBLE _and_ LONG
  ruleLines.push_back(rline.str());
}
void
ConfFileData::addRuleLineString(const char *name)
{
  ostringstream tline;
  ostringstream rline;

  tline << "%token\t<lval>\tKW_" << name;
  tokenLines.push_back(tline.str());

  if(ruleLines.empty())
    rline << "configline:\t  KW_" << name << " COLON CHARS\n"
	  << "\t\t{ ConfigFile::" << name << "=$3; }"
	 ;
  else
    rline << "\t\t| KW_" << name << " COLON CHARS\n"
	  << "\t\t{ ConfigFile::" << name << "=$3; }"
	 ;
  ruleLines.push_back(rline.str());
}
void
ConfFileData::addRuleLineComment(const char *text)
{
  // add nothing to the rule section
}
void
ConfFileData::addRuleLineEmpty()
{
  // add nothing to the rule section
}


void
ConfFileData::addDefaultLineLong(const char *name, long defaultvalue)
{
  ostringstream line;

  line << name << ":\t\t" << defaultvalue
       << "\t# default: <" << defaultvalue << ">"
      ;
  defaultLines.push_back(line.str());
}
void
ConfFileData::addDefaultLineDouble(const char *name, double defaultvalue)
{
  ostringstream line;

  line << name << ":\t\t" << defaultvalue
       << "\t# default: <" << defaultvalue << ">"
      ;
  defaultLines.push_back(line.str());
}
void
ConfFileData::addDefaultLineString(const char *name, const char *defaultvalue)
{
  ostringstream line;

  line << name << ":\t\t" << defaultvalue
       << "\t# default: <" << defaultvalue << ">"
      ;
  defaultLines.push_back(line.str());
}
void
ConfFileData::addDefaultLineComment(const char *text)
{
  ostringstream line;

  line << "#" << text;
  defaultLines.push_back(line.str());
}
void
ConfFileData::addDefaultLineEmpty()
{
  ostringstream line;

  //line;
  defaultLines.push_back(line.str());
}
