/*****************************************************************************
 * ParserIntfc.hh
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   class ParserIntfc
 *
 * requires:
 *
 * Created on: 19.09.2002 by wenns
 * Maintainer: wenns
 * Group:
 * Co-authors:
 *
 *
 * $Id: ParserIntfc.hh,v 1.3 2004/04/03 20:18:29 harlekin Exp $
 *****************************************************************************/

#ifndef PARSERINTFC_HH
#define PARSERINTFC_HH

#include <iostream>
using namespace std;

enum PlayerParam{
  PP_unknown,
  PP_PlayerTypes,
  PP_PTMax,
  PP_RandomSeed,
  PP_SubsMax,
  PP_DashPowerRateDeltaMax,
  PP_DashPowerRateDeltaMin,
  PP_EffortMaxDeltaFactor,
  PP_EffortMinDeltaFactor,
  PP_ExtraStaminaDeltaMax,
  PP_ExtraStaminaDeltaMin,
  PP_InertiaMomentDeltaFactor,
  PP_KickRandDeltaFactor,
  PP_KickableMarginDeltaMax,
  PP_KickableMarginDeltaMin,
  PP_NewDashPowerRateDeltaMax,
  PP_NewDashPowerRateDeltaMin,
  PP_NewStaminaIncMaxDeltaFactor,
  PP_PlayerDecayDeltaMax,
  PP_PlayerDecayDeltaMin,
  PP_PlayerSizeDeltaFactor,
  PP_PlayerSpeedMaxDeltaMax,
  PP_PlayerSpeedMaxDeltaMin,
  PP_StaminaIncMaxDeltaFactor
};

enum ServerParam{
  SP_CatchBanCycle, // 0
  SP_CLangAdviceWin, // 1
  SP_CLangDefineWin,
  SP_CLangDelWin,
  SP_CLangInfoWin,
  SP_CLangMessDelay, // 5
  SP_CLangMessPerCycle,
  SP_CLangMetaWin,
  SP_CLangRuleWin,
  SP_CLangWinSize,
  SP_CoachPort, // 10
  SP_DropBallTime,
  SP_FreeformSendPeriod,
  SP_FreeformWaitPeriod,
  SP_GameLogCompression,
  SP_GameLogVersion, // 15
  SP_GoalieMaxMoves,
  SP_HalfTime,
  SP_HearDecay,
  SP_HearInc,
  SP_HearMax, // 20
  SP_MaxGoalKicks,
  SP_OLCoachPort,
  SP_PointToBan,
  SP_PointToDuration,
  SP_Port, // 25
  SP_RecvStep,
  SP_SayCoachCntMax,
  SP_SayCoachMsgSize,
  SP_SayMsgSize,
  SP_SendStep, // 30
  SP_SendVIStep,
  SP_SenseBodyStep,
  SP_SimulatorStep,
  SP_SlowDownFactor,
  SP_StartGoalL, // 35
  SP_StartGoalR,
  SP_SynchMicroSleep,
  SP_SynchOffset,
  SP_TackleCycles,
  SP_TextLogCompression, // 40
  /* QuotetSTRs */
  SP_GameLogDir,
  SP_GameLogFixedName,
  SP_LandmarkFile,
  SP_LogDateFormat,
  SP_TextLogDir, // 45
  SP_TextLogFixedName,
  /* BOOLs */
  SP_Coach,
  SP_CoachWReferee,
  SP_OldCoachHear,
  SP_WindNone, // 50
  SP_WindRandom, // 51
  /* ON/OFFs */
  SP_BackPasses,
  SP_ForbidKickOffOffside,
  SP_FreeKickFaults,
  SP_FullstateL, // 55
  SP_FullstateR,
  SP_GameLogDated,
  SP_GameLogFixed,
  SP_GameLogging,
  SP_LogTimes, // 60
  SP_Profile,
  SP_ProperGoalKicks,
  SP_RecordMessages,
  SP_SendComms,
  SP_SynchMode, // 65
  SP_TeamActuatorNoise, // 66
  SP_TextLogDated,
  SP_TextLogFixed,
  SP_TextLogging,
  SP_UseOffside,
  SP_Verbose,
  /* VALs (doubles, floats etc.) */
  SP_AudioCutDist,
  SP_BallAccelMax,
  SP_BallDecay,
  SP_BallRand,
  SP_BallSize,
  SP_BallSpeedMax,
  SP_BallWeight,
  SP_CatchProbability,
  SP_CatchableAreaL,
  SP_CatchableAreaW,
  SP_CKickMargin,
  SP_ControlRadius,
  SP_DashPowerRate,
  SP_EffortDec,
  SP_EffortDecThr,
  SP_EffortInc,
  SP_EffortIncThr,
  SP_EffortInit,
  SP_EffortMin,
  SP_GoalWidth,
  SP_InertiaMoment,
  SP_KickPowerRate,
  SP_KickRand,
  SP_KickRandFactorL,
  SP_KickRandFactorR,
  SP_KickableMargin,
  SP_MaxMoment,
  SP_MaxNeckAng,
  SP_MaxNeckMoment,
  SP_MaxPower,
  SP_MinMoment,
  SP_MinNeckAng,
  SP_MinNeckMoment,
  SP_MinPower,
  SP_OffsideActiveAreaSize,
  SP_OffsideKickMargin,
  SP_PlayerAccelMax,
  SP_PlayerDecay,
  SP_PlayerRand,
  SP_PlayerSize,
  SP_PlayerSpeedMax,
  SP_PlayerWeight,
  SP_PRandFactorL,
  SP_PRandFactorR,
  SP_QuantizeStep,
  SP_QuantizeStepL,
  SP_RecoverDec,
  SP_RecoverDecThr,
  SP_RecoverMin,
  SP_SlownessOnTopForLeftTeam,
  SP_SlownessOnTopForRightTeam,
  SP_StaminaIncMax,
  SP_StaminaMax,
  SP_StoppedBallVel,
  SP_TackleBackDist,
  SP_TackleDist,
  SP_TackleExponent,
  SP_TacklePowerRate,
  SP_TackleWidth,
  SP_VisibleAngle,
  SP_VisibleDistance,
  SP_WindAng,
  SP_WindDir,
  SP_WindForce,
  SP_WindRand
};

/** this will be linked to the generated parser */
extern int parse_msg( const char*, void* );

class ParserIntfc {
public:
  ParserIntfc();
  virtual ~ParserIntfc();

  /**
   * lets the parsing occur
   * @param msg the message to parse
   */
  void parse (const char *msg );

  /**
   * parses a message in the coach language format. Its not implemented yet
   * and is intended to be the hook to build in the std coach language
   * parser distributed with the soccer server.
   * @param msg the message to parse
   */
  bool parseCLang ( const char *msg ){
    cerr << "Received Formation ID: " << atoi(msg) << endl;
    cerr << "Coach language not supported yet," << endl;
    cerr << "\"" << msg << "\" cannot be parsed" << endl;
    return true;
  }

  //----------------------------- shared --------------------------
  /**
   * \name side events
   *
   * the following events are generated, when a side specification, and
   * namely "l", "r", "our" or "opp" is parsed
   */
  //@{
  virtual void buildLeftSide() = 0;
  virtual void buildRightSide() = 0;
  virtual void buildOurSide() = 0;
  virtual void buildOppSide() = 0;
  virtual void buildNeutralSide() = 0;
  //@}
  
  
  /**
   * \name playmode events
   *
   * the following events are generated, when a playmode specification
   * is parsed
   */
  //@{
  virtual void buildBeforeKickOffPMode() = 0;
  virtual void buildTimeOverPMode() = 0;
  virtual void buildPlayOnPMode() = 0;
  virtual void buildKickOffLPMode() = 0;
  virtual void buildKickOffRPMode() = 0;
  virtual void buildKickInLPMode() = 0;
  virtual void buildKickInRPMode() = 0;
  virtual void buildFreeKickLPMode() = 0;
  virtual void buildFreeKickRPMode() = 0;
  virtual void buildCornerKickLPMode() = 0;
  virtual void buildCornerKickRPMode() = 0;
  virtual void buildGoalKickLPMode() = 0;
  virtual void buildGoalKickRPMode() = 0;
  virtual void buildGoalLPMode() = 0;
  virtual void buildGoalRPMode() = 0;
  virtual void buildGoalLPMode(const int goals) = 0;
  virtual void buildGoalRPMode(const int goals) = 0;
  virtual void buildDropBallPMode() = 0;
  virtual void buildOffsideLPMode() = 0;
  virtual void buildOffsideRPMode() = 0;
  virtual void buildPenaltyKickLPMode() = 0;
  virtual void buildPenaltyKickRPMode() = 0;
  virtual void buildPausePMode() = 0;
  virtual void buildHumanJudgePMode() = 0;
  virtual void buildFoulLPMode() = 0;
  virtual void buildFoulRPMode() = 0;
  virtual void buildFoulChargeLPMode() = 0;
  virtual void buildFoulChargeRPMode() = 0;
  virtual void buildFoulPushLPMode() = 0;
  virtual void buildFoulPushRPMode() = 0;
  virtual void buildFoulMultipleAttackLPMode() = 0;
  virtual void buildFoulMultipleAttackRPMode() = 0;
  virtual void buildFoulBallOutLPMode() = 0;
  virtual void buildFoulBallOutRPMode() = 0;
  virtual void buildBackPassLPMode() = 0;
  virtual void buildBackPassRPMode() = 0;
  virtual void buildFreeKickFaultLPMode() = 0;
  virtual void buildFreeKickFaultRPMode() = 0;
  virtual void buildGoalieCatchBallLPMode() = 0;
  virtual void buildGoalieCatchBallRPMode() = 0;
  virtual void buildHalfTimePMode() = 0;
  virtual void buildTimeUpRPMode() = 0;
  virtual void buildIndirectFreeKickLPMode() = 0;
  virtual void buildIndirectFreeKickRPMode() = 0;
  virtual void buildPenaltySetupLPMode() = 0;
  virtual void buildPenaltySetupRPMode() = 0;
  virtual void buildPenaltyReadyLPMode() = 0;
  virtual void buildPenaltyReadyRPMode() = 0;
  virtual void buildPenaltyTakenLPMode() = 0;
  virtual void buildPenaltyTakenRPMode() = 0;
  virtual void buildPenaltyMissLPMode() = 0;
  virtual void buildPenaltyMissRPMode() = 0;
  virtual void buildPenaltyScoreLPMode() = 0;
  virtual void buildPenaltyScoreRPMode() = 0;
  virtual void buildPenaltySideL() = 0;
  virtual void buildPenaltySideR() = 0;
  //@}

  /**
   * \name view mode events
   *
   * the following events are generated, when a view mode specification
   * is parsed, such as "(view_mode high normal)"
   */
  //@{
  virtual void buildViewQualityHigh() = 0;
  virtual void buildViewQualityLow() = 0;

  virtual void buildViewWidthNarrow() = 0;
  virtual void buildViewWidthNormal() = 0;
  virtual void buildViewWidthWide() = 0;
  //@}

  /**
   * this event is generated when arm info is parsed. The parameters
   * correspond direct to the tokens in the arm message (which is a part of
   * fullstate and sensebody messages)
   */
  virtual void buildArmInfo( const int movable, const int expires,
			     const float targetDist, const float targetDir,
			     const int count ) = 0;


  //----------------------------- init ----------------------------------
  /**
   * this event is generated when parsing of the init server reply message
   * is complete.
   * @param unum the uniform number that is assigned to the player
   */
  virtual void buildInit( const int unum ) = 0;


  //----------------------------- server param ----------------------------
  /**
   * this event is generated when parsing of the server param  message
   * is complete.
   * Look at the manual, server sources, ... , for the meaning of the
   * parameters 
   */
  virtual void buildServerParam() = 0;
  virtual void buildServerParam(const ServerParam sp, const int    val) = 0;
  virtual void buildServerParam(const ServerParam sp, const double val) = 0;
  virtual void buildServerParam(const ServerParam sp, const bool   val) = 0;
  virtual void buildServerParam(const ServerParam sp, const char  *val) = 0;
  virtual void buildServerParam(
    const int catch_ban_cycle,
    const int clang_advice_win,
    const int clang_define_win,
    const int clang_del_win,
    const int clang_info_win,
    const int clang_mess_delay,
    const int clang_mess_per_cycle,
    const int clang_meta_win,
    const int clang_rule_win,
    const int clang_win_size,
    const int coach_port,
    const int drop_ball_time,
    const int freeform_send_period,
    const int freeform_wait_period,
    const int game_log_compression,
    const int game_log_version,
    const int goalie_max_moves,
    const int half_time,
    const int hear_decay,
    const int hear_inc,
    const int hear_max,
    const int max_goal_kicks,
    const int olcoach_port,
    const int point_to_ban,
    const int point_to_duration,
    const int port,
    const int recv_step,
    const int say_coach_cnt_max,
    const int say_coach_msg_size,
    const int say_msg_size,
    const int send_step,
    const int send_vi_step,
    const int sense_body_step,
    const int simulator_step,
    const int slow_down_factor,
    const int start_goal_l,
    const int start_goal_r,
    const int synch_micro_sleep,
    const int synch_offset,
    const int tackle_cycles,
    const int text_log_compression,
    const char *game_log_dir,
    const char *game_log_fixed_name,
    const char *landmark_file,
    const char *log_date_format,
    const char *text_log_dir,
    const char *text_log_fixed_name,
    const bool coach,
    const bool coach_w_referee,
    const bool old_coach_hear,
    const bool wind_none,
    const float wind_random,
    const bool back_passes,
    const bool forbid_kick_off_offside,
    const bool free_kick_faults,
    const bool fullstate_l,
    const bool fullstate_r,
    const bool game_log_dated,
    const bool game_log_fixed,
    const bool game_logging,
    const bool log_times,
    const bool profile,
    const bool proper_goal_kicks,
    const bool record_messages,
    const bool send_comms,
    const bool synch_mode,
    const bool team_actuator_noise,
    const bool text_log_dated,
    const bool text_log_fixed,
    const bool text_logging,
    const bool use_offside,
    const bool verbose,
    const float audio_cut_dist,
    const float ball_accel_max,
    const float ball_decay,
    const float ball_rand,
    const float ball_size,
    const float ball_speed_max,
    const float ball_weight,
    const float catch_probability,
    const float catchable_area_l,
    const float catchable_area_w,
    const float ckick_margin,
    const float control_radius,
    const float dash_power_rate,
    const float effort_dec,
    const float effort_dec_thr,
    const float effort_inc,
    const float effort_inc_thr,
    const float effort_init,
    const float effort_min,
    const float goal_width,
    const float inertia_moment,
    const float kick_power_rate,
    const float kick_rand,
    const float kick_rand_factor_l,
    const float kick_rand_factor_r,
    const float kickable_margin,
    const float maxmoment,
    const float maxneckang,
    const float maxneckmoment,
    const float maxpower,
    const float minmoment,
    const float minneckang,
    const float minneckmoment,
    const float minpower,
    const float offside_active_area_size,
    const float offside_kick_margin,
    const float player_accel_max,
    const float player_decay,
    const float player_rand,
    const float player_size,
    const float player_speed_max,
    const float player_weight,
    const float prand_factor_l,
    const float prand_factor_r,
    const float quantize_step,
    const float quantize_step_l,
    const float recover_dec,
    const float recover_dec_thr,
    const float recover_min,
    const float slowness_on_top_for_left_team,
    const float slowness_on_top_for_right_team,
    const float stamina_inc_max,
    const float stamina_max,
    const float stopped_ball_vel,
    const float tackle_back_dist,
    const float tackle_dist,
    const float tackle_exponent,
    const float tackle_power_rate,
    const float tackle_width,
    const float visible_angle,
    const float visible_distance,
    const float wind_ang,
    const float wind_dir,
    const float wind_force,
    const float wind_rand
    ) = 0;


  //----------------------------- player param ----------------------------
  /**
   * this event is generated when parsing of the player param message
   * is complete.
   * Look at the manual, server sources, ... , for the meaning of the
   * parameters 
   */
  virtual void buildPlayerParam() = 0;
  virtual void buildPlayerParam(const PlayerParam pp, const int   val) = 0;
  virtual void buildPlayerParam(const PlayerParam pp, const float val) = 0;
  //virtual void buildPlayerParam(const PlayerParam pp, const bool   val) = 0;
  //virtual void buildPlayerParam(const PlayerParam pp, const char  *val) = 0;
  virtual void buildPlayerParam(
    const int player_types,
    const int pt_max,
    const float random_seed,
    const int subs_max,
    const float dash_power_rate_delta_max,
    const float dash_power_rate_delta_min,
    const float effort_max_delta_factor,
    const float effort_min_delta_factor,
    const float extra_stamina_delta_max,
    const float extra_stamina_delta_min,
    const float inertia_moment_delta_factor,
    const float kick_rand_delta_factor,
    const float kickable_margin_delta_max,
    const float kickable_margin_delta_min,
    const float new_dash_power_rate_delta_max,
    const float new_dash_power_rate_delta_min,
    const float new_stamina_inc_max_delta_factor,
    const float player_decay_delta_max,
    const float player_decay_delta_min,
    const float player_size_delta_factor,
    const float player_speed_max_delta_max,
    const float player_speed_max_delta_min,
    const float stamina_inc_max_delta_factor
    ) = 0;


  //----------------------------- player type ---------------------------
  /**
   * this event is generated when parsing of the player type message
   * is complete.
   * Look at the manual, server sources, ... , for the meaning of the
   * parameters 
   */
  virtual void buildPlayerType(
    const int id,
    const float player_speed_max,
    const float stamina_inc_max,
    const float player_decay,
    const float inertia_moment,
    const float dash_power_rate,
    const float player_size,
    const float kickable_margin,
    const float kick_rand,
    const float extra_stamina,
    const float effort_max,
    const float effort_min
    ) = 0;


  //----------------------------- sense body ---------------------------
  /**
   * this event is generated when the parsing of a sense-body message is
   * complete.
   * The parameters correspond direct to the tokens in the message
   */
  virtual void buildSenseBody(
    const int time,
    const float stamina,
    const float effort,
    const float speedAmount,
    const float speedDir,
    const float headAngle,
    const int kickCount,
    const int dashCount,
    const int turnCount,
    const int sayCount,
    const int turnNeckCount,
    const int catchCount,
    const int moveCount,
    const int changeViewCount ) = 0;

  /**
   * \name focus info events
   *
   * the following events are generated, when a focus info specification
   * is parsed, such as "(focus (target none) (count 0))"
   */
  //@{
  virtual void buildFocusInfo( const int count ) = 0;
  virtual void buildFocusTargetNone() = 0;
  virtual void buildFocusTarget( const int unum ) = 0;

  virtual void buildTackleInfo( const int expires, int count ) = 0;
  //@}

  //----------------------------- visual -----------------------------
  /**
   * this event is generated, when the parsing of a see message is complete 
   * @param time the server cycle of the message
   */
  virtual void buildVisual( const int time ) = 0;

  /**
   * this event is generated, when the parsing of a see_global message is
   * complete 
   * @param time the server cycle of the message
   */
  virtual void buildGlobalVisual( const int time ) = 0;


  /**
   * \name line events
   * 
   * this events are generated, when the parsing of a line in a see message
   * is complete. dir is relative direction, dist - relative distance
   */
  //@{
  virtual void buildLine( const float dir ) = 0;
  virtual void buildLine( const float dist, const float dir ) = 0;
  //@}

  /**
   * \name goal and flag events
   * 
   * this events are generated, when the parsing of a flag or a goal in a
   * see message is complete. close it true if the flag is in the feeling
   * area of the sensing agent, dist is relative distance, dir - relative
   * direction, distChg - distance changing, dirChg - direction changing
   * (the latter can be sensed because of noise by not moving objects, too). 
   */
  //@{
  virtual void buildFlag( const bool close,
			  const float dist, const float dir,
			  const float distChg, const float dirChg ) = 0;
  virtual void buildFlag( const bool close,
			  const float dist, const float dir ) = 0;
  virtual void buildFlag( const bool close, const float dir ) = 0;
  virtual void buildGoal( const bool close,
			  const float dist, const float dir,
			  const float distChg, const float dirChg ) = 0;
  virtual void buildGoal( const bool close,
			  const float dist, const float dir ) = 0;
  virtual void buildGoal( const bool close, const float dir ) = 0;
  //@}
  
  /**
   * this event is generated, when a of a goal specification in a see_global
   * or fullstate is parsed
   * @param posx the global x coordinate of the goal
   * @param posy the global y coordinate of the goal
   */
  virtual void buildGlobalGoal( const float posx, const float posy ) = 0;

  /**
   * \name ball events
   * 
   * this events are generated, when a ball specification in a
   * see message is parsed. "close" it true if the flag is in the feeling
   * area of the sensing agent, "dist" is relative distance, "dir" - relative
   * direction, "distChg" - distance changing, "dirChg" - direction changing
   * From the latter the current velocity of the ball can be extracted, see
   * the manual for details
   */
  //@{
  virtual void buildBall( const bool close,
			  const float dist, const float dir,
			  const float distChg, const float dirChg ) = 0;
  virtual void buildBall( const bool close,
			  const float dist, const float dir ) = 0;
  virtual void buildBall( const bool close, const float dir ) = 0;
  //@}

  /**
   * this event is generated, when a ball specification in a see_global
   * message is parsed
   * @param posx the global x coordinate of the goal
   * @param posy the global y coordinate of the goal
   * @param velx the x offset of the global velocity 
   * @param vely the y offset of the global velocity
   */
  virtual void buildGlobalBall( const float posx, const float posy,
				const float velx, const float vely ) = 0;

  /**
   * \name player events
   * 
   * this events are generated, when a player specification in a
   * see message is parsed. "close" is true if the player is in the feeling
   * area of the sensing agent, "dist" is relative distance, "dir" - relative
   * direction, "distChg" - distance changing, "dirChg" - direction changing,
   * "bodyDir" - the relative body direction, "faceDir" - the relative face
   * direction.
   * The changing values can be used to calculate the relative velocity of
   * the sensed player, see the manual for details
   */
  //@{
  virtual void buildPlayer( const bool close,
			    const float dist, const float dir,
			    const float distChg, const float dirChg,
			    const float bodyDir, const float faceDir ) = 0;
  virtual void buildPlayer( const bool close,
			    const float dist, const float dir,
			    const float distChg, const float dirChg ) = 0;
  virtual void buildPlayer( const bool close,
			    const float dist, const float dir ) = 0;
  virtual void buildPlayer( const bool close, const float dir) = 0;
  //@}

  /**
   * \name player extra info events
   * 
   * this events are generated, when extra info for a player is parsed. This
   * info carries information about tackle and pointto actions of the sensed
   * player.
   * "tackling" it true in the tackling phase, i.e. the sensed player is either
   * tackling right now or he has tackled and the tackle ban cycles aren't
   * passed yet
   * "pointdir" is the relative direction the sensed player is pointing
   */
  //@{
  virtual void buildPlayerExtraInfo( const bool tackling ) = 0;
  virtual void buildPlayerExtraInfo( const float pointdir ) = 0;
  virtual void buildPlayerExtraInfo( const float pointdir, const bool tackling ) = 0;
  //@}

  /**
   * this event is generated, when a player specification in a see_global
   * message is parsed.
   * @param posx the global x coordinate of the sensed player
   * @param posy the global y coordinate of the sensed player
   * @param velx the x offset of the global velocity of the sensed player 
   * @param vely the y offset of the global velocity of the sensed player
   * @param bodyDir the absolute body direction of the sensed player
   * @param faceDir the absolute face direction of the sensed player
   * direction.
   */
  virtual void buildGlobalPlayer( const float posx, const float posy,
				  const float velx, const float vely,
				  const float bodyDir, const float faceDir) = 0;

  /**
   * \name player name events
   * 
   * this events are generated, when player identity is parsed. The player
   * identity consist of a uniform number, team name, and goalie flag;
   * the presence of all components is optional. The parameter of the
   * functions below correspond direct to the named identity components 
   */
  //@{
  virtual void buildPlayerName() = 0;
  virtual void buildPlayerName( const char *teamname ) = 0;
  virtual void buildPlayerName( const char *teamname,
				const int unum ) = 0;
  virtual void buildPlayerName( const char *teamname,
				const int unum,
				const bool goalie ) = 0;
  //@}

  /**
   * this event is generated, when a "flag offset" is parsed. Flag offset is
   * defined as the number often appended to the flag id, for example the "20"
   * in "(f t r 20)"
   * @param offset the actual flag offset
   */
  virtual void buildFlagOffset( const int offset ) = 0;

  /**
   * \name location events
   * 
   * this events are generated, when a location specification is parsed.
   */
  //@{
  virtual void buildTopLocation()     = 0;
  virtual void buildBottomLocation()  = 0;
  virtual void buildLeftLocation()    = 0;
  virtual void buildRightLocation()   = 0;
  virtual void buildCenterLocation()  = 0;
  virtual void buildPenaltyLocation() = 0;
  virtual void buildGoalLocation()    = 0;
  //@}

  //----------------------------- score ---------------------------
  /**
   * this event is generated when a score-message has been parsed.
   * @param time the number of soccer-server-cycle, in which the message has
   * been generated
   * @param our number of goals shout by our team
   * @param their number of goals shout by their team
   */
  virtual void buildScore( const int time, const int our, const int their) = 0;


  //----------------------------- fullstate ---------------------------
  /**
   * this event is generated when the parsing of a fullstate-message has been
   * completed
   * @param time the number of soccer-server-cycle, in which the message has
   * been generated
   */
  virtual void buildFullState( const int time ) = 0;

  /**
   * this event is generated when the counts included in the
   * fullstate-message are parsed. Any count stores the number of times a
   * certain soccer-server-command has been received from the agent. 
   */
  virtual void buildFsCount(
    const int kickCount,
    const int dashCount,
    const int turnCount,
    const int sayCount,
    const int turnNeckCount,
    const int catchCount,
    const int moveCount,
    const int changeViewCount
    ) = 0;

  /**
   * this event is generated when the scores included in the
   * fullstate-message are parsed.
   * @param our number of goals our team has scored
   * @param their number of goals their team has scored
   */
  virtual void buildFsScore( const int our, const int their) = 0;

  /**
   * this event is generated when the ball information included in the
   * fullstate-message is parsed. 
   * @param posx the global x coordinate of the goal
   * @param posy the global y coordinate of the goal
   * @param velx the x offset of the global velocity 
   * @param vely the y offset of the global velocity
   */
  virtual void buildFsBall( const float posx, const float posy,
			    const float velx, const float vely ) = 0;

  /**
   * this event is generated, when a player specification in a
   * fullstate-message is parsed.
   * @param posx the global x coordinate of the player
   * @param posy the global y coordinate of the player
   * @param velx the x offset of the global velocity of the player 
   * @param vely the y offset of the global velocity of the player
   * @param bodyDir the absolute body direction of the player
   * @param faceDir the absolute face direction of the player
   * direction.
   */
  virtual void buildFsPlayer(
    const float posx,
    const float posy,
    const float velx,
    const float vely,
    const float bodyDir,
    const float faceDir
    ) = 0;
  
  /**
   * this event is generated, when player identity in a fullstate-message
   * has been parsed. Actually a player identity in a fullstate-message
   * consist of a side and uniform number, but the side of the player is
   * submitted by an extra event (build...Side)
   * @param unum the uniform number of the player
   */
  virtual void buildFsPlayername( const int unum ) = 0;

  /**
   * \name player type events
   * 
   * this events are generated, when a player type specification, included in
   * the fullstate-message, is parsed. A player is either a goalie or of type
   * [0..6]
   */
  //@{
  virtual void buildFsPlayerType( const bool goalie ) = 0;
  virtual void buildFsPlayerType( const int type ) = 0;
  //@}

  /**
   * this event is generated, when stamina infos, included in the
   * fullstate-message, are parsed.
   * @param stamina the amount of stamina currently available
   * @param effort the present effort
   * @param recovery the present recovery
   */
  virtual void buildFsStaminaInfo( const float stamina, const float effort,
				   const float recovery ) = 0;


  //----------------------------- audio ---------------------------
  /**
   * this event is generated when a hear-message from the coach has been parsed
   * @param time the number of soccer-server-cycle, in which the message has
   * been generated
   * @param msg the message from the coach
   */
  virtual void buildCoachAudio( const int time, const char *msg ) = 0;

  /**
   * this event is generated when the parsing of a hear-message from the
   * referee has been completed 
   * @param time the number of soccer-server-cycle, in which the message has
   * been generated
   */
  virtual void buildRefAudio( const int time ) = 0;

  /**
   * this event is generated when a hear-message from the referee with an
   * unknown meaning has been parsed
   * @param time the number of soccer-server-cycle, in which the message has
   * been generated
   * @param msg the message from the referee   
   */
  virtual void buildUnknownRefAudio( const int time, const char *msg ) = 0;

  /**
   * \name player audio events
   * 
   * this events are generated when a hear-message from a player has been
   * parsed, thereby is:
   * "time" the number of soccer-server-cycle, in which the message has
   * been generated
   * "dir" the relative direction, from which the message has been "heard"
   * "unum" the uniform number of player who has said it
   * "msg" the actual message
   */
  //@{
  virtual void buildPlayerAudio( const int time, const float dir,
				 const int unum, const char *msg ) = 0;
  virtual void buildPlayerAudio( const int time, const float dir,
				 const char *msg ) = 0;
  virtual void buildPlayerAudio( const int time, const int unum ) = 0;
  virtual void buildPlayerAudio( const int time ) = 0;
  //@}

  /**
   * this event is generated when a hear-message from myself has been parsed
   * @param time the number of soccer-server-cycle, in which the message has
   * been generated
   * @param msg the message itself   
   */
  virtual void buildSelfAudio( const int time, const char *msg ) = 0;


  //----------------------------- change ---------------------------
  /**
   * \name substitution events
   *
   * the following events are generated, when a change-message (a notification
   * of a player substitution by the coach) has been parsed, thereby is:
   * "unum" the uniform number of the player who has been substituted
   * "type" the type of the new player
   */
  //@{
  virtual void buildSubstitution( const int unum, const int type ) = 0;
  virtual void buildSubstitution( const int unum ) = 0;
  //@}

  //----------------------------- error ---------------------------
  /**
   * \name error events
   *
   * the following events are generated, when an server error is parsed
   */
  //@{
  virtual void buildCantReconnect() = 0;
  virtual void buildInitError() = 0;
  virtual void buildNoMoreTeamOrPlayerOrGoalieError() = 0;
  virtual void buildNoSuchTeamOrAlreadyHaveCoachError() = 0;
  virtual void buildTooManyMovesError() = 0;
  virtual void buildUnknownCommandError() = 0;
  virtual void buildIllegalCommandError() = 0;
  virtual void buildSayMessageTooLongError() = 0;
  virtual void buildIllegalModeError() = 0;
  virtual void buildIllegalObjectFormError() = 0;
  virtual void buildSaidTooManyFreeformMessagesError() = 0;
  virtual void buildCannotSayFreeformWhilePlayonError() = 0;
  virtual void buildSaidTooManyMetaMessagesError() = 0;
  virtual void buildSaidTooManyAdviceMessagesError() = 0;
  virtual void buildSaidTooManyInfoMessagesError() = 0;
  virtual void buildSaidTooManyDefineMessagesError() = 0;
  virtual void buildCouldNotParseSayError() = 0;
  virtual void buildSaidTooManyMessagesError() = 0;
  virtual void buildUnknownError( const char *error ) = 0;
  //@}

  //----------------------------- warning ---------------------------
  /**
   * \name warning events
   *
   * the following events are generated, when an server warning is parsed
   */
  //@{
  virtual void buildNoTeamFoundWarning() = 0;
  virtual void buildNoSuchPlayerWarning() = 0;
  virtual void buildCannotSubWhilePlayOnWarning() = 0;
  virtual void buildNoSubsLeftWarning() = 0;
  virtual void buildMaxOfThatPlayerTypeOnFieldWarning() = 0;
  virtual void buildCannotChangeGoalieWarning() = 0;
  virtual void buildUnknownWarning( const char *warning ) = 0;
  //@}

  //----------------------------- ok ---------------------------
  /**
   * \name ok events
   *
   * the following events are generated, when an server ok is parsed
   */
  //@{
  virtual void buildClangVerOK( const int min, const int max ) = 0;
  virtual void buildEyeOK( const bool eyeOn ) = 0;
  virtual void buildUnknownOK( const char *ok ) = 0;
  //@}


  //------------------------- sync ------------------------
  /**
   * \name sync events
   *
   * the following events are generated, when the synchronized mode is used
   */
  //@{
  virtual void buildThink() = 0;
  //@}
  
  virtual void buildCoachFreeform(const char *msg) = 0;

};

#endif //PARSERINTFC_HH
