/* grammar file */
%{
#include <stdlib.h>
#include <string.h>
#include "ParserIntfc.hh"

#define YYERROR_VERBOSE           // verbose msgs in case of parse error
#define yyparse         ss_parse  // name of parsing function
#define yylex           ss_lex    // name of tokenizer function
#define YYPARSE_PARAM   param     // parameter for the parsing function

static ParserIntfc& getParser(void *p){
  return *((ParserIntfc*)p);
}

char* stripQuotes(char *str) {
  if ( !strcmp( str, "") )
    return str;

  if ( str[0] == '"' )
    str++;
  if ( str[strlen(str)-1] == '"' )
      str[strlen(str)-1] = '\0';

  return str;
}

#define PARSER getParser(param)

int yyerror(const char *);        // this funtion will be called in the case
                                  //  of a parse error
%}

//parser options
%pure_parser                      // generate a reentrant parser


//init
%token INIT


// server param token
%token SERVPARA
%token CATCH_BAN_CYCLE
%token CLANG_ADVICE_WIN
%token CLANG_DEFINE_WIN
%token CLANG_DEL_WIN
%token CLANG_INFO_WIN
%token CLANG_MESS_DELAY
%token CLANG_MESS_PER_CYCLE
%token CLANG_META_WIN
%token CLANG_RULE_WIN
%token CLANG_WIN_SIZE
%token COACH_PORT
%token DROP_BALL_TIME
%token FREEFORM_SEND_PERIOD
%token FREEFORM_WAIT_PERIOD
%token GAME_LOG_COMPRESSION
%token GAME_LOG_VERSION
%token GOALIE_MAX_MOVES
%token HALF_TIME
%token HEAR_DECAY
%token HEAR_INC
%token HEAR_MAX
%token MAX_GOAL_KICKS
%token OLCOACH_PORT
%token POINT_TO_BAN
%token POINT_TO_DURATION
%token PORT
%token RECV_STEP
%token SAY_COACH_CNT_MAX
%token SAY_COACH_MSG_SIZE
%token SAY_MSG_SIZE
%token SEND_STEP
%token SEND_VI_STEP
%token SENSE_BODY_STEP
%token SIMULATOR_STEP
%token SLOW_DOWN_FACTOR
%token START_GOAL_L
%token START_GOAL_R
%token SYNCH_MICRO_SLEEP
%token SYNCH_OFFSET
%token TACKLE_CYCLES
%token TEXT_LOG_COMPRESSION
%token GAME_LOG_DIR
%token GAME_LOG_FIXED_NAME
%token LANDMARK_FILE
%token LOG_DATE_FORMAT
%token TEXT_LOG_DIR
%token TEXT_LOG_FIXED_NAME
%token COACH
%token COACH_W_REFEREE
%token OLD_COACH_HEAR
%token WIND_NONE
%token WIND_RANDOM
%token BACK_PASSES
%token FORBID_KICK_OFF_OFFSIDE
%token FREE_KICK_FAULTS
%token FULLSTATE_L
%token FULLSTATE_R
%token GAME_LOG_DATED
%token GAME_LOG_FIXED
%token GAME_LOGGING
%token LOG_TIMES
%token PROFILE
%token PROPER_GOAL_KICKS
%token RECORD_MESSAGES
%token SEND_COMMS
%token SYNCH_MODE
%token TEAM_ACTUATOR_NOISE
%token TEXT_LOG_DATED
%token TEXT_LOG_FIXED
%token TEXT_LOGGING
%token USE_OFFSIDE
%token VERBOSE
%token AUDIO_CUT_DIST
%token BALL_ACCEL_MAX
%token BALL_DECAY
%token BALL_RAND
%token BALL_SIZE
%token BALL_SPEED_MAX
%token BALL_WEIGHT
%token CATCH_PROBABILITY
%token CATCHABLE_AREA_L
%token CATCHABLE_AREA_W
%token CKICK_MARGIN
%token CONTROL_RADIUS
%token DASH_POWER_RATE
%token EFFORT_DEC
%token EFFORT_DEC_THR
%token EFFORT_INC
%token EFFORT_INC_THR
%token EFFORT_INIT
%token EFFORT_MIN
%token GOAL_WIDTH
%token INERTIA_MOMENT
%token KICK_POWER_RATE
%token KICK_RAND
%token KICK_RAND_FACTOR_L
%token KICK_RAND_FACTOR_R
%token KICKABLE_MARGIN
%token MAXMOMENT
%token MAXNECKANG
%token MAXNECKMOMENT
%token MAXPOWER
%token MINMOMENT
%token MINNECKANG
%token MINNECKMOMENT
%token MINPOWER
%token OFFSIDE_ACTIVE_AREA_SIZE
%token OFFSIDE_KICK_MARGIN
%token PLAYER_ACCEL_MAX
%token PLAYER_DECAY
%token PLAYER_RAND
%token PLAYER_SIZE
%token PLAYER_SPEED_MAX
%token PLAYER_WEIGHT
%token PRAND_FACTOR_L
%token PRAND_FACTOR_R
%token QUANTIZE_STEP
%token QUANTIZE_STEP_L
%token RECOVER_DEC
%token RECOVER_DEC_THR
%token RECOVER_MIN
%token SLOWNESS_ON_TOP_FOR_LEFT_TEAM
%token SLOWNESS_ON_TOP_FOR_RIGHT_TEAM
%token STAMINA_INC_MAX
%token STAMINA_MAX
%token STOPPED_BALL_VEL
%token TACKLE_BACK_DIST
%token TACKLE_DIST
%token TACKLE_EXPONENT
%token TACKLE_POWER_RATE
%token TACKLE_WIDTH
%token VISIBLE_ANGLE
%token VISIBLE_DISTANCE
%token WIND_ANG
%token WIND_DIR
%token WIND_FORCE
%token WIND_RAND

%token CLANG_OK

// playpara
%token PLAYPARA
%token PLAYER_TYPES
%token PT_MAX
%token RANDOM_SEED
%token SUBS_MAX
%token DASH_POWER_RATE_DELTA_MAX
%token DASH_POWER_RATE_DELTA_MIN
%token EFFORT_MAX_DELTA_FACTOR
%token EFFORT_MIN_DELTA_FACTOR
%token EXTRA_STAMINA_DELTA_MAX
%token EXTRA_STAMINA_DELTA_MIN
%token INERTIA_MOMENT_DELTA_FACTOR
%token KICK_RAND_DELTA_FACTOR
%token KICKABLE_MARGIN_DELTA_MAX
%token KICKABLE_MARGIN_DELTA_MIN
%token NEW_DASH_POWER_RATE_DELTA_MAX
%token NEW_DASH_POWER_RATE_DELTA_MIN
%token NEW_STAMINA_INC_MAX_DELTA_FACTOR
%token PLAYER_DECAY_DELTA_MAX
%token PLAYER_DECAY_DELTA_MIN
%token PLAYER_SIZE_DELTA_FACTOR
%token PLAYER_SPEED_MAX_DELTA_MAX
%token PLAYER_SPEED_MAX_DELTA_MIN
%token STAMINA_INC_MAX_DELTA_FACTOR

// playtype
%token PLAYTYPE
%token ID
%token PLAYER_SPEED_MAX  // redundant
%token STAMINA_INC_MAX   // redundant
%token PLAYER_DECAY
%token INERTIA_MOMENT
%token DASH_POWER_RATE
%token PLAYER_SIZE
%token KICKABLE_MARGIN
%token KICK_RAND
%token EXTRA_STAMINA
%token EFFORT_MAX
%token EFFORT_MIN


//sense body
%token SENSE_BODY
%token STAMINA VIEW_MODE SPEED
%token KICK DASH TURN SAY TURN_NECK CATCH MOVE CHANGE_VIEW HEAD_ANGLE
%token ARM FOCUS TACKLE
%token MOVABLE EXPIRES TARGET COUNT NONE
%token HIGH LOW             // states of view quality
%token NARROW NORMAL WIDE   // states of view width


//visual
%token SEE
%token SEE_GLOBAL
%token P B G F L
%token C R T
%token BB BF BG BP

%type <ival> flagname
%type <ival> goalname
%type <ival> ballname
%type <ival> playername


//fullstate
%token FULLSTATE
%token PMODE
%token VMODE
%token GOAL_L
%token GOAL_R

//audio
%token HEAR
%token REFEREE
%token SELF
%token GOALIE
%token ONLINE_COACH_LEFT
%token ONLINE_COACH_RIGHT

%token <str>  CLANG


//change ("Coach" messages)
%token CHANGE
%token FREEFORM

//errors
%token CANT
%token ERROR
%token NO_MORE_TEAM_OR_PLAYER_OR_GOALIE
%token NO_SUCH_TEAM_OR_ALREADY_HAVE_COACH
%token TOO_MANY_MOVES
%token UNKNOWN_COMMAND
%token ILLEGAL_COMMAND_FORM
%token SAY_MESSAGE_TOO_LONG
%token ILLEGAL_MODE
%token ILLEGAL_OBJECT_FORM
%token SAID_TOO_MANY_FREEFORM_MESSAGES
%token CANNOT_SAY_FREEFORM_WHILE_PLAYON
%token SAID_TOO_MANY_META_MESSAGES
%token SAID_TOO_MANY_ADVICE_MESSAGES
%token SAID_TOO_MANY_INFO_MESSAGES
%token SAID_TOO_MANY_DEFINE_MESSAGES
%token COULD_NOT_PARSE_SAY
%token SAID_TOO_MANY_MESSAGES


//warnings
%token WARN
%token NO_TEAM_FOUND
%token NO_SUCH_PLAYER
%token CANNOT_SUB_WHILE_PLAYON
%token NO_SUBS_LEFT
%token MAX_OF_THAT_PLAYER_TYPE_ON_FIELD
%token CANNOT_CHANGE_GOALIE


//ok
%token OK
%token VER
%token EYE

%token <ival> ON_OFF


//shared
%token LP
%token RP
%token SCORE
%token OUR
%token OPP

%token BEFORE_KICK_OFF
%token TIME_OVER
%token PLAY_ON
%token KICK_OFF_L
%token KICK_OFF_R
%token KICK_IN_L
%token KICK_IN_R
%token FREE_KICK_L
%token FREE_KICK_R
%token CORNER_KICK_L
%token CORNER_KICK_R
%token GOAL_KICK_L
%token GOAL_KICK_R
%token DROP_BALL
%token OFFSIDE_L
%token OFFSIDE_R
%token PENALTY_KICK_L
%token PENALTY_KICK_R
%token PAUSE
%token HUMAN_JUDGE
%token FOUL_L
%token FOUL_R
%token FOUL_CHARGE_L
%token FOUL_CHARGE_R
%token FOUL_PUSH_L
%token FOUL_PUSH_R
%token FOUL_MULTIPLE_ATTACK_L
%token FOUL_MULTIPLE_ATTACK_R
%token FOUL_BALLOUT_L
%token FOUL_BALLOUT_R
%token BACK_PASS_L
%token BACK_PASS_R
%token FREE_KICK_FAULT_L
%token FREE_KICK_FAULT_R
%token GOALIE_CATCH_BALL_L
%token GOALIE_CATCH_BALL_R
%token HALF_TIME
%token TIME_UP
%token INDIRECT_FREE_KICK_L
%token INDIRECT_FREE_KICK_R
%token PENALTY_SETUP_L
%token PENALTY_SETUP_R
%token PENALTY_READY_L
%token PENALTY_READY_R
%token PENALTY_TAKEN_L
%token PENALTY_TAKEN_R
%token PENALTY_MISS_L
%token PENALTY_MISS_R
%token PENALTY_SCORE_L
%token PENALTY_SCORE_R

// Referee messages for penalty shootout
%token PENALTY_ONFIELD_L
%token PENALTY_ONFIELD_R

%token THINK
%token DONE

%token <ival> GOAL_L_INT
%token <ival> GOAL_R_INT
%token <ival> INT
%token <rval> FLOAT
%token <str>  STR
%token <str>  QSTR
%type  <rval> VAL


%union {
  int   ival;
  float rval;
  char  *str;
}


%{
extern int ss_lex(YYSTYPE*);
%}


//grammar goes here
%%

message_list        : /* empty */
                    | message_list message  {}
                    ;

message   : init_msg      { /*cout << "init ok" << endl; YYACCEPT;*/ }
          | servpara_msg  { /*cout << "servpara ok" << endl; YYACCEPT;*/ }
          | playpara_msg  { /*cout << "playpara ok" << endl; YYACCEPT;*/ }
          | playtype_msg  { /*cout << "playtype ok" << endl; YYACCEPT;*/ }
          | sensebody_msg { /*cout << "sensebody ok" << endl; YYACCEPT;*/ }
          | visual_msg    { /*cout << "visual ok" << endl; YYACCEPT;*/ }
          | score_msg     { /*cout << "score ok" << endl; YYACCEPT;*/ }
          | fullstate_msg { /*cout << "fullstate ok" << endl; YYACCEPT;*/ }
          | audio_msg     { /*cout << "audio ok" << endl; YYACCEPT;*/ }
          | change_msg    { /*cout << "change ok" << endl; YYACCEPT;*/ }
          | error_msg     { /*cout << "error ok" << endl; YYACCEPT;*/ }
          | warning_msg   { /*cout << "worning ok" << endl; YYACCEPT;*/ }
          | ok_msg        { /*cout << "ok-msg ok" << endl; YYACCEPT;*/ }
          | sync_msg { /*cout << "sync-msk ok" << endl; YYACCEPT;*/ }
          ;


/* ----------------------------- shared ----------------------------------- */

VAL       : INT   { $$=$1;        }
          | FLOAT { $$=(float)$1; }
          ;

side      : L   { PARSER.buildLeftSide(); }
          | R   { PARSER.buildRightSide(); }
          | OUR { PARSER.buildOurSide(); }
          | OPP { PARSER.buildOppSide(); }
          ;

playmode  :
  BEFORE_KICK_OFF        { PARSER.buildBeforeKickOffPMode();       }
| TIME_OVER              { PARSER.buildTimeOverPMode();            }
| PLAY_ON                { PARSER.buildPlayOnPMode();              }
| KICK_OFF_L             { PARSER.buildKickOffLPMode();            }
| KICK_OFF_R             { PARSER.buildKickOffRPMode();            }
| KICK_IN_L              { PARSER.buildKickInLPMode();             }
| KICK_IN_R              { PARSER.buildKickInRPMode();             }
| FREE_KICK_L            { PARSER.buildFreeKickLPMode();           }
| FREE_KICK_R            { PARSER.buildFreeKickRPMode();           }
| CORNER_KICK_L          { PARSER.buildCornerKickLPMode();         }
| CORNER_KICK_R          { PARSER.buildCornerKickRPMode();         }
| GOAL_KICK_L            { PARSER.buildGoalKickLPMode();           }
| GOAL_KICK_R            { PARSER.buildGoalKickRPMode();           }
| GOAL_L                 { PARSER.buildGoalLPMode();               }
| GOAL_R                 { PARSER.buildGoalRPMode();               }
| GOAL_L_INT             { PARSER.buildGoalLPMode($1);             }
| GOAL_R_INT             { PARSER.buildGoalRPMode($1);             }
| DROP_BALL              { PARSER.buildDropBallPMode();            }
| OFFSIDE_L              { PARSER.buildOffsideLPMode();            }
| OFFSIDE_R              { PARSER.buildOffsideRPMode();            }
| PENALTY_KICK_L         { PARSER.buildPenaltyKickLPMode();        }
| PENALTY_KICK_R         { PARSER.buildPenaltyKickRPMode();        }
| PAUSE                  { PARSER.buildPausePMode();               }
| HUMAN_JUDGE            { PARSER.buildHumanJudgePMode();          }
| FOUL_L                 { PARSER.buildFoulLPMode();               }
| FOUL_R                 { PARSER.buildFoulRPMode();               }
| FOUL_CHARGE_L          { PARSER.buildFoulChargeLPMode();         }
| FOUL_CHARGE_R          { PARSER.buildFoulChargeRPMode();         }
| FOUL_PUSH_L            { PARSER.buildFoulPushLPMode();           }
| FOUL_PUSH_R            { PARSER.buildFoulPushRPMode();           }
| FOUL_MULTIPLE_ATTACK_L { PARSER.buildFoulMultipleAttackLPMode(); }
| FOUL_MULTIPLE_ATTACK_R { PARSER.buildFoulMultipleAttackRPMode(); }
| FOUL_BALLOUT_L         { PARSER.buildFoulBallOutLPMode();        }
| FOUL_BALLOUT_R         { PARSER.buildFoulBallOutRPMode();        }
| BACK_PASS_L            { PARSER.buildBackPassLPMode();           }
| BACK_PASS_R            { PARSER.buildBackPassRPMode();           }
| FREE_KICK_FAULT_L      { PARSER.buildFreeKickFaultLPMode();      }
| FREE_KICK_FAULT_R      { PARSER.buildFreeKickFaultRPMode();      }
| GOALIE_CATCH_BALL_L    { PARSER.buildGoalieCatchBallLPMode();    }
| GOALIE_CATCH_BALL_R    { PARSER.buildGoalieCatchBallRPMode();    }
| HALF_TIME              { PARSER.buildHalfTimePMode();            }
| TIME_UP                { PARSER.buildTimeUpRPMode();             }
| INDIRECT_FREE_KICK_L   { PARSER.buildIndirectFreeKickLPMode();   }
| INDIRECT_FREE_KICK_R   { PARSER.buildIndirectFreeKickLPMode();   }
| PENALTY_SETUP_L        { PARSER.buildPenaltySetupLPMode();       }
| PENALTY_SETUP_R        { PARSER.buildPenaltySetupRPMode();       }
| PENALTY_READY_L        { PARSER.buildPenaltyReadyLPMode();       }
| PENALTY_READY_R        { PARSER.buildPenaltyReadyRPMode();       }
| PENALTY_TAKEN_L        { PARSER.buildPenaltyTakenLPMode();       }
| PENALTY_TAKEN_R        { PARSER.buildPenaltyTakenRPMode();       }
| PENALTY_MISS_L         { PARSER.buildPenaltyMissLPMode();        }
| PENALTY_MISS_R         { PARSER.buildPenaltyMissRPMode();        }
| PENALTY_SCORE_L        { PARSER.buildPenaltyScoreLPMode();       }
| PENALTY_SCORE_R        { PARSER.buildPenaltyScoreRPMode();       }
;

view_quality        : HIGH { PARSER.buildViewQualityHigh(); }
                    | LOW  { PARSER.buildViewQualityLow(); }
                    ;

view_width          : NARROW { PARSER.buildViewWidthNarrow(); }
                    | NORMAL { PARSER.buildViewWidthNormal(); }
                    | WIDE   { PARSER.buildViewWidthWide(); }
                    ;

/*(arm (movable {0|1}) (expires <COUNT>) (target <DIST> <DIR>) (count <COUNT>))*/
arm_info  : LP ARM
 /*3-6*/  LP MOVABLE INT RP
 /*7-10*/ LP EXPIRES INT RP
/*11-15*/ LP TARGET VAL VAL RP
/*16-19*/ LP COUNT INT RP
          RP
          { PARSER.buildArmInfo($5, $9, $13, $14, $18); }
          ;

ballname  : LP B RP  { $$ = false; }
          | LP BB RP { $$ = true; }
          ;


/* -------------------------- init ------------------------------- */

          /* ( init Side Unum PlayMode ) */
init_msg  : LP INIT side INT  playmode    RP
          { PARSER.buildInit($4); }
          ;



/* -------------------------- server param ------------------------------- */

servpara_paras : /* empty */
               | servpara_paras servpara_para
               ;

servpara_para : LP CATCH_BAN_CYCLE INT RP { PARSER.buildServerParam(SP_CatchBanCycle, $3); }
              | LP CLANG_ADVICE_WIN INT RP { PARSER.buildServerParam(SP_CLangAdviceWin, $3); }
              | LP CLANG_DEFINE_WIN INT RP { PARSER.buildServerParam(SP_CLangDefineWin, $3); }
              | LP CLANG_DEL_WIN INT RP { PARSER.buildServerParam(SP_CLangDelWin, $3); }
              | LP CLANG_INFO_WIN INT RP { PARSER.buildServerParam(SP_CLangInfoWin, $3); }
              | LP CLANG_MESS_DELAY INT RP { PARSER.buildServerParam(SP_CLangMessDelay, $3); }
              | LP CLANG_MESS_PER_CYCLE INT RP { PARSER.buildServerParam(SP_CLangMessPerCycle, $3); }
              | LP CLANG_META_WIN INT RP { PARSER.buildServerParam(SP_CLangMetaWin, $3); }
              | LP CLANG_RULE_WIN INT RP { PARSER.buildServerParam(SP_CLangRuleWin, $3); }      
              | LP CLANG_WIN_SIZE INT RP { PARSER.buildServerParam(SP_CLangWinSize, $3); }           
              | LP COACH_PORT INT RP { PARSER.buildServerParam(SP_CoachPort, $3); }
              | LP DROP_BALL_TIME INT RP { PARSER.buildServerParam(SP_DropBallTime, $3); }
              | LP FREEFORM_SEND_PERIOD INT RP { PARSER.buildServerParam(SP_FreeformSendPeriod, $3); }
              | LP FREEFORM_WAIT_PERIOD INT RP { PARSER.buildServerParam(SP_FreeformWaitPeriod, $3); }          
              | LP GAME_LOG_COMPRESSION INT RP { PARSER.buildServerParam(SP_GameLogCompression, $3); }          
              | LP GAME_LOG_VERSION INT RP{ PARSER.buildServerParam(SP_GameLogVersion, $3); }  
              | LP GOALIE_MAX_MOVES INT RP{ PARSER.buildServerParam(SP_GoalieMaxMoves, $3); } 
              | LP HALF_TIME INT RP { PARSER.buildServerParam(SP_HalfTime, $3); } 
              | LP HEAR_DECAY INT RP { PARSER.buildServerParam(SP_HearDecay, $3); }
              | LP HEAR_INC INT RP { PARSER.buildServerParam(SP_HearInc, $3); } 
              | LP HEAR_MAX INT RP { PARSER.buildServerParam(SP_HearMax, $3); } 
              | LP MAX_GOAL_KICKS INT RP { PARSER.buildServerParam(SP_MaxGoalKicks, $3); } 
              | LP OLCOACH_PORT INT RP { PARSER.buildServerParam(SP_OLCoachPort, $3); } 
              | LP POINT_TO_BAN INT RP { PARSER.buildServerParam(SP_PointToBan, $3); }  
              | LP POINT_TO_DURATION INT RP { PARSER.buildServerParam(SP_PointToDuration, $3); } 
              | LP PORT INT RP { PARSER.buildServerParam(SP_Port, $3); } 
              | LP RECV_STEP INT RP { PARSER.buildServerParam(SP_RecvStep, $3); } 
              | LP SAY_COACH_CNT_MAX INT RP { PARSER.buildServerParam(SP_SayCoachCntMax, $3); } 
              | LP SAY_COACH_MSG_SIZE INT RP{ PARSER.buildServerParam(SP_SayCoachMsgSize, $3); } 
              | LP SAY_MSG_SIZE INT RP { PARSER.buildServerParam(SP_SayMsgSize, $3); } 
              | LP SEND_STEP INT RP { PARSER.buildServerParam(SP_SendStep, $3); } 
              | LP SEND_VI_STEP INT RP { PARSER.buildServerParam(SP_SendVIStep, $3); } 
              | LP SENSE_BODY_STEP INT RP { PARSER.buildServerParam(SP_SenseBodyStep, $3); } 
              | LP SIMULATOR_STEP INT RP { PARSER.buildServerParam(SP_SimulatorStep, $3); }
              | LP SLOW_DOWN_FACTOR INT RP { PARSER.buildServerParam(SP_SlowDownFactor, $3); }
              | LP START_GOAL_L INT RP { PARSER.buildServerParam(SP_StartGoalL, $3); } 
              | LP START_GOAL_R INT RP { PARSER.buildServerParam(SP_StartGoalR, $3); } 
              | LP SYNCH_MICRO_SLEEP INT RP { PARSER.buildServerParam(SP_SynchMicroSleep, $3); } 
              | LP SYNCH_OFFSET INT RP { PARSER.buildServerParam(SP_SynchOffset, $3); } 
              | LP TACKLE_CYCLES INT RP { PARSER.buildServerParam(SP_TackleCycles, $3); } 
              | LP TEXT_LOG_COMPRESSION INT RP { PARSER.buildServerParam(SP_TextLogCompression, $3); } 
/* QuotedSTRs */
              | LP GAME_LOG_DIR QSTR RP { PARSER.buildServerParam(SP_GameLogDir, $3); } 
              | LP GAME_LOG_FIXED_NAME QSTR RP { PARSER.buildServerParam(SP_GameLogFixedName, $3); } 
              | LP LANDMARK_FILE QSTR RP { PARSER.buildServerParam(SP_LandmarkFile, $3); }  
              | LP LOG_DATE_FORMAT QSTR RP { PARSER.buildServerParam(SP_LogDateFormat, $3); }  
              | LP TEXT_LOG_DIR QSTR RP { PARSER.buildServerParam(SP_TextLogDir, $3); } 
              | LP TEXT_LOG_FIXED_NAME QSTR RP { PARSER.buildServerParam(SP_TextLogFixedName, $3); } 
/* BOOLs */
              | LP COACH INT RP { PARSER.buildServerParam(SP_Coach, $3); } 
              | LP COACH_W_REFEREE INT RP { PARSER.buildServerParam(SP_CoachWReferee, $3); } 
              | LP OLD_COACH_HEAR INT RP { PARSER.buildServerParam(SP_OldCoachHear, $3); }  
              | LP WIND_NONE INT RP { PARSER.buildServerParam(SP_WindNone, $3); }  
              | LP WIND_RANDOM INT RP { PARSER.buildServerParam(SP_WindRandom, $3); } 
/* ON/OFFs */
              | LP BACK_PASSES INT RP { PARSER.buildServerParam(SP_BackPasses, $3); } 
              | LP FORBID_KICK_OFF_OFFSIDE INT RP { PARSER.buildServerParam(SP_ForbidKickOffOffside, $3); }
              | LP FREE_KICK_FAULTS INT RP { PARSER.buildServerParam(SP_FreeKickFaults, $3); }
              | LP FULLSTATE_L INT RP { PARSER.buildServerParam(SP_FullstateL, $3); } 
              | LP FULLSTATE_R INT RP { PARSER.buildServerParam(SP_FullstateR, $3); } 
              | LP GAME_LOG_DATED INT RP{ PARSER.buildServerParam(SP_GameLogDated, $3); } 
              | LP GAME_LOG_FIXED INT RP { PARSER.buildServerParam(SP_GameLogFixed, $3); }
              | LP GAME_LOGGING INT RP  { PARSER.buildServerParam(SP_GameLogging, $3); } 
              | LP LOG_TIMES INT RP  { PARSER.buildServerParam(SP_LogTimes, $3); } 
              | LP PROFILE INT RP { PARSER.buildServerParam(SP_Profile, $3); }  
              | LP PROPER_GOAL_KICKS INT RP { PARSER.buildServerParam(SP_ProperGoalKicks, $3); }
              | LP RECORD_MESSAGES INT RP { PARSER.buildServerParam(SP_RecordMessages, $3); } 
              | LP SEND_COMMS INT RP { PARSER.buildServerParam(SP_SendComms, $3); } 
              | LP SYNCH_MODE INT RP { PARSER.buildServerParam(SP_SynchMode, $3); } 
              | LP TEAM_ACTUATOR_NOISE INT RP { PARSER.buildServerParam(SP_TeamActuatorNoise, $3); }  
              | LP TEXT_LOG_DATED INT RP { PARSER.buildServerParam(SP_TextLogDated, $3); } 
              | LP TEXT_LOG_FIXED INT RP { PARSER.buildServerParam(SP_TextLogFixed, $3); } 
              | LP TEXT_LOGGING INT RP { PARSER.buildServerParam(SP_TextLogging, $3); }  
              | LP USE_OFFSIDE INT RP { PARSER.buildServerParam(SP_UseOffside, $3); }  
              | LP VERBOSE INT RP { PARSER.buildServerParam(SP_Verbose, $3); }   
/* VALs (Doubles etc.) */
              | LP AUDIO_CUT_DIST VAL RP { PARSER.buildServerParam(SP_AudioCutDist, $3); } 
              | LP BALL_ACCEL_MAX VAL RP { PARSER.buildServerParam(SP_BallAccelMax, $3); } 
              | LP BALL_DECAY VAL RP { PARSER.buildServerParam(SP_BallDecay, $3); } 
              | LP BALL_RAND VAL RP { PARSER.buildServerParam(SP_BallRand, $3); } 
              | LP BALL_SIZE VAL RP{ PARSER.buildServerParam(SP_BallSize, $3); } 
              | LP BALL_SPEED_MAX VAL RP { PARSER.buildServerParam(SP_BallSpeedMax, $3); } 
              | LP BALL_WEIGHT VAL RP { PARSER.buildServerParam(SP_BallWeight, $3); } 
              | LP CATCH_PROBABILITY VAL RP { PARSER.buildServerParam(SP_CatchProbability, $3); }
              | LP CATCHABLE_AREA_L VAL RP { PARSER.buildServerParam(SP_CatchableAreaL, $3); }
              | LP CATCHABLE_AREA_W VAL RP { PARSER.buildServerParam(SP_CatchableAreaW, $3); }
              | LP CKICK_MARGIN VAL RP{ PARSER.buildServerParam(SP_CKickMargin, $3); }    
              | LP CONTROL_RADIUS VAL RP { PARSER.buildServerParam(SP_ControlRadius, $3); } 
              | LP DASH_POWER_RATE VAL RP { PARSER.buildServerParam(SP_DashPowerRate, $3); }
              | LP EFFORT_DEC VAL RP { PARSER.buildServerParam(SP_EffortDec, $3); }     
              | LP EFFORT_DEC_THR VAL RP { PARSER.buildServerParam(SP_EffortDecThr, $3); } 
              | LP EFFORT_INC VAL RP { PARSER.buildServerParam(SP_EffortInc, $3); }
              | LP EFFORT_INC_THR VAL RP { PARSER.buildServerParam(SP_EffortIncThr, $3); }
              | LP EFFORT_INIT VAL RP { PARSER.buildServerParam(SP_EffortInit, $3); }  
              | LP EFFORT_MIN VAL RP { PARSER.buildServerParam(SP_EffortMin, $3); }   
              | LP GOAL_WIDTH VAL RP { PARSER.buildServerParam(SP_GoalWidth, $3); }     
              | LP INERTIA_MOMENT VAL RP { PARSER.buildServerParam(SP_InertiaMoment, $3); }     
              | LP KICK_POWER_RATE VAL RP { PARSER.buildServerParam(SP_KickPowerRate, $3); }     
              | LP KICK_RAND VAL RP { PARSER.buildServerParam(SP_KickRand, $3); }           
              | LP KICK_RAND_FACTOR_L VAL RP { PARSER.buildServerParam(SP_KickRandFactorL, $3); }  
              | LP KICK_RAND_FACTOR_R VAL RP { PARSER.buildServerParam(SP_KickRandFactorR, $3); }  
              | LP KICKABLE_MARGIN VAL RP { PARSER.buildServerParam(SP_KickableMargin, $3); }  
              | LP MAXMOMENT VAL RP { PARSER.buildServerParam(SP_MaxMoment, $3); }    
              | LP MAXNECKANG VAL RP { PARSER.buildServerParam(SP_MaxNeckAng, $3); }   
              | LP MAXNECKMOMENT VAL RP { PARSER.buildServerParam(SP_MaxNeckMoment, $3); }
              | LP MAXPOWER VAL RP { PARSER.buildServerParam(SP_MaxPower, $3); }   
              | LP MINMOMENT VAL RP{ PARSER.buildServerParam(SP_MinMoment, $3); }    
              | LP MINNECKANG VAL RP{ PARSER.buildServerParam(SP_MinNeckAng, $3); }    
              | LP MINNECKMOMENT VAL RP { PARSER.buildServerParam(SP_MinNeckMoment, $3); } 
              | LP MINPOWER VAL RP { PARSER.buildServerParam(SP_MinPower, $3); } 
              | LP OFFSIDE_ACTIVE_AREA_SIZE VAL RP { PARSER.buildServerParam(SP_OffsideActiveAreaSize, $3); }
              | LP OFFSIDE_KICK_MARGIN VAL RP { PARSER.buildServerParam(SP_OffsideKickMargin, $3); }
              | LP PLAYER_ACCEL_MAX VAL RP { PARSER.buildServerParam(SP_PlayerAccelMax, $3); }
              | LP PLAYER_DECAY VAL RP { PARSER.buildServerParam(SP_PlayerDecay, $3); }
              | LP PLAYER_RAND VAL RP { PARSER.buildServerParam(SP_PlayerRand, $3); }
              | LP PLAYER_SIZE VAL RP { PARSER.buildServerParam(SP_PlayerSize, $3); }
              | LP PLAYER_SPEED_MAX VAL RP { PARSER.buildServerParam(SP_PlayerSpeedMax, $3); }
              | LP PLAYER_WEIGHT VAL RP { PARSER.buildServerParam(SP_PlayerWeight, $3); }
              | LP PRAND_FACTOR_L VAL RP { PARSER.buildServerParam(SP_PRandFactorL, $3); }
              | LP PRAND_FACTOR_R VAL RP { PARSER.buildServerParam(SP_PRandFactorR, $3); }
              | LP QUANTIZE_STEP VAL RP { PARSER.buildServerParam(SP_QuantizeStep, $3); }
              | LP QUANTIZE_STEP_L VAL RP { PARSER.buildServerParam(SP_QuantizeStepL, $3); }
              | LP RECOVER_DEC VAL RP { PARSER.buildServerParam(SP_RecoverDec, $3); }
              | LP RECOVER_DEC_THR VAL RP { PARSER.buildServerParam(SP_RecoverDecThr, $3); }
              | LP RECOVER_MIN VAL RP { PARSER.buildServerParam(SP_RecoverMin, $3); }
              | LP SLOWNESS_ON_TOP_FOR_LEFT_TEAM VAL RP { PARSER.buildServerParam(SP_SlownessOnTopForLeftTeam, $3); }
              | LP SLOWNESS_ON_TOP_FOR_RIGHT_TEAM VAL RP { PARSER.buildServerParam(SP_SlownessOnTopForRightTeam, $3); }
              | LP STAMINA_INC_MAX VAL RP { PARSER.buildServerParam(SP_StaminaIncMax, $3); }
              | LP STAMINA_MAX VAL RP { PARSER.buildServerParam(SP_StaminaMax, $3); }
              | LP STOPPED_BALL_VEL VAL RP { PARSER.buildServerParam(SP_StoppedBallVel, $3); }
              | LP TACKLE_BACK_DIST VAL RP { PARSER.buildServerParam(SP_TackleBackDist, $3); }
              | LP TACKLE_DIST VAL RP { PARSER.buildServerParam(SP_TackleDist, $3); }
              | LP TACKLE_EXPONENT VAL RP { PARSER.buildServerParam(SP_TackleExponent, $3); }
              | LP TACKLE_POWER_RATE VAL RP{ PARSER.buildServerParam(SP_TacklePowerRate, $3); }
              | LP TACKLE_WIDTH VAL RP { PARSER.buildServerParam(SP_TackleWidth, $3); }
              | LP VISIBLE_ANGLE VAL RP { PARSER.buildServerParam(SP_VisibleAngle, $3); }
              | LP VISIBLE_DISTANCE VAL RP { PARSER.buildServerParam(SP_VisibleDistance, $3); }
              | LP WIND_ANG VAL RP { PARSER.buildServerParam(SP_WindAng, $3); }
              | LP WIND_DIR VAL RP  { PARSER.buildServerParam(SP_WindDir, $3); }
              | LP WIND_FORCE VAL RP { PARSER.buildServerParam(SP_WindForce, $3); }
              | LP WIND_RAND VAL RP { PARSER.buildServerParam(SP_WindRand, $3); }
              ;

          /* ( server_param Para1 Para2 ... ParaN ) */
servpara_msg : LP SERVPARA servpara_paras RP { PARSER.buildServerParam(); }
             ;

/* -------------------------- player param ------------------------------- */

          /* ( player_param Para1 Para2 ... ParaN ) */

playpara_msg : LP PLAYPARA playpara_paras RP { PARSER.buildPlayerParam(); }
             ;
playpara_paras : 
               | playpara_paras playpara_para
               ;

playpara_para : LP PLAYER_TYPES INT RP                     { PARSER.buildPlayerParam(PP_PlayerTypes, $3);}
              | LP PT_MAX INT RP                           { PARSER.buildPlayerParam(PP_PTMax, $3);}
              | LP RANDOM_SEED VAL RP                      { PARSER.buildPlayerParam(PP_RandomSeed, $3);}
              | LP SUBS_MAX INT RP                         { PARSER.buildPlayerParam(PP_SubsMax, $3);}
              | LP DASH_POWER_RATE_DELTA_MAX VAL RP        { PARSER.buildPlayerParam(PP_DashPowerRateDeltaMax, $3);}
              | LP DASH_POWER_RATE_DELTA_MIN VAL RP        { PARSER.buildPlayerParam(PP_DashPowerRateDeltaMin, $3);}
              | LP EFFORT_MAX_DELTA_FACTOR VAL RP          { PARSER.buildPlayerParam(PP_EffortMaxDeltaFactor, $3);}
              | LP EFFORT_MIN_DELTA_FACTOR VAL RP          { PARSER.buildPlayerParam(PP_EffortMinDeltaFactor, $3);}
              | LP EXTRA_STAMINA_DELTA_MAX VAL RP          { PARSER.buildPlayerParam(PP_ExtraStaminaDeltaMax, $3);}
              | LP EXTRA_STAMINA_DELTA_MIN VAL RP          { PARSER.buildPlayerParam(PP_ExtraStaminaDeltaMin, $3);}
              | LP INERTIA_MOMENT_DELTA_FACTOR VAL RP      { PARSER.buildPlayerParam(PP_InertiaMomentDeltaFactor, $3);}
              | LP KICK_RAND_DELTA_FACTOR VAL RP           { PARSER.buildPlayerParam(PP_KickRandDeltaFactor, $3);}
              | LP KICKABLE_MARGIN_DELTA_MAX VAL RP        { PARSER.buildPlayerParam(PP_KickableMarginDeltaMax, $3);}
              | LP KICKABLE_MARGIN_DELTA_MIN VAL RP        { PARSER.buildPlayerParam(PP_KickableMarginDeltaMin, $3);}
              | LP NEW_DASH_POWER_RATE_DELTA_MAX VAL RP    { PARSER.buildPlayerParam(PP_NewDashPowerRateDeltaMax, $3);}
              | LP NEW_DASH_POWER_RATE_DELTA_MIN VAL RP    { PARSER.buildPlayerParam(PP_NewDashPowerRateDeltaMin, $3);}
              | LP NEW_STAMINA_INC_MAX_DELTA_FACTOR VAL RP { PARSER.buildPlayerParam(PP_NewStaminaIncMaxDeltaFactor, $3);}
              | LP PLAYER_DECAY_DELTA_MAX VAL RP           { PARSER.buildPlayerParam(PP_PlayerDecayDeltaMax, $3);}
              | LP PLAYER_DECAY_DELTA_MIN VAL RP           { PARSER.buildPlayerParam(PP_PlayerDecayDeltaMin, $3);}
              | LP PLAYER_SIZE_DELTA_FACTOR VAL RP         { PARSER.buildPlayerParam(PP_PlayerSizeDeltaFactor, $3);}
              | LP PLAYER_SPEED_MAX_DELTA_MAX VAL RP       { PARSER.buildPlayerParam(PP_PlayerSpeedMaxDeltaMax, $3);}
              | LP PLAYER_SPEED_MAX_DELTA_MIN VAL RP       { PARSER.buildPlayerParam(PP_PlayerSpeedMaxDeltaMin, $3);}
              | LP STAMINA_INC_MAX_DELTA_FACTOR VAL RP     { PARSER.buildPlayerParam(PP_StaminaIncMaxDeltaFactor, $3);}
              ;

/* -------------------------- player type ------------------------------- */

          /* ( player_param Para1 Para2 ... ParaN ) */
playtype_msg : LP PLAYTYPE
  LP ID INT RP                 /*3-6*/
  LP PLAYER_SPEED_MAX VAL RP   /*7-10*/
  LP STAMINA_INC_MAX VAL RP    /*11-14*/
  LP PLAYER_DECAY VAL RP       /*15-18*/
  LP INERTIA_MOMENT VAL RP     /*19-22*/
  LP DASH_POWER_RATE VAL RP    /*23-26*/
  LP PLAYER_SIZE VAL RP        /*27-30*/
  LP KICKABLE_MARGIN VAL RP    /*31-34*/
  LP KICK_RAND VAL RP          /*35-38*/
  LP EXTRA_STAMINA VAL RP      /*39-42*/
  LP EFFORT_MAX VAL RP         /*43-46*/
  LP EFFORT_MIN VAL RP         /*47-50*/
  RP
  {
    PARSER.buildPlayerType( $5, $9, $13, $17, $21, $25, $29, $33, $37, $41,
                            $45, $49 );
  }
  ;


/* -------------------------- sense body ------------------------------- */

/*(sense_body 0 (view_mode high normal) (stamina 4000 1) (speed 0 174)(head_angle 0)
 (kick 0) (dash 0) (turn 1) (say 0) (turn_neck 0) (catch 0) (move 1) (change_view 0)
 (arm (movable 0) (expires 0) (target 0 0) (count 0))
 (focus (target none) (count 0))
 (tackle (expires 0) (count 0)))*/

sensebody_msg     : LP SENSE_BODY INT     /*(  sense_body time */
         /* 4- 8*/  LP VIEW_MODE view_quality view_width RP /* unused */
         /* 9-13*/  LP STAMINA VAL VAL RP   /*(stamina cur_stamina effort)*/
         /*14-18*/  LP SPEED VAL INT RP     /*(speed amount direction)*/
         /*19-22*/  LP HEAD_ANGLE INT RP    /*(head_angle angle)*/
         /*23-26*/  LP KICK INT RP          /*(kick kick_count)*/
         /*27-30*/  LP DASH INT RP          /*(dash dash_count)*/
         /*31-34*/  LP TURN INT RP          /*(turn turn_count)*/
         /*35-38*/  LP SAY  INT RP          /*(say say_count)*/
         /*39-42*/  LP TURN_NECK INT RP     /*(turn_neck t_neck_count)*/
         /*43-46*/  LP CATCH INT RP         /*(catch catch_count)*/
         /*47-50*/  LP MOVE INT RP          /*(move move_count)*/
         /*51-54*/  LP CHANGE_VIEW INT RP   /*(change_view c_view_count)*/
                    arm_info
                    focus_info
                    tackle_info
                    RP                      /*)*/
                    {
                      PARSER.buildSenseBody($3,  $11,
                                             $12, $16, $17, $21,
                                             $25, $29, $33, $37,
                                             $41, $45, $49, $53);
                    }
                    ;

                    /*(focus (target none) (count <COUNT>))*/
                    /*(focus (target {l|r} <UNUM>) (count <COUNT>)*/
focus_info          : LP FOCUS
                    LP TARGET target_info RP
                    LP COUNT INT RP
                    RP
                    { PARSER.buildFocusInfo( $9 ); }
                    ;

target_info         : NONE     { PARSER.buildFocusTargetNone(); }
                    | side INT { PARSER.buildFocusTarget($2); }
                    ;

                    /*(tackle (expires <COUNT>) (count <COUNT>))*/
tackle_info         : LP TACKLE
                    LP EXPIRES INT RP
                    LP COUNT INT RP
                    RP
                    { PARSER.buildTackleInfo( $5, $9 ); }
                    ;


/* ----------------------------- visual ----------------------------------- */

/*(see 0 ((g l) 62.8 -5) ((f c) 10 -5 0 0) ((f l t) 70.8 23) ((f l b) 70.8 -34)
 ((f g l b) 62.8 -12) ((f g l t) 62.8 1) ((f p l b) 50.4 -29) ((f p l c) 46.1 -5)
 ((f p l t) 50.4 18) ((f t l 30) 55.7 39) ((f t l 40) 63.4 33) ((f t l 50) 71.5 28)
 ((f b l 40) 63.4 -43) ((f b l 50) 71.5 -38) ((f l 0) 67.4 -5) ((f l t 10) 68 3)
 ((f l t 20) 70.1 11) ((f l t 30) 73.7 19) ((f l b 10) 68 -14) ((f l b 20) 70.1 -22)
 ((f l b 30) 73.7 -29) ((b) 10 -5 0 0) ((p "team1" 1) 20.1 -19 0 0 175 175)
 ((p "team1" 2) 30 -5 0 0 175 175) ((l l) 62.8 85))*/

visual_msg          : LP SEE INT object_list RP
                    { PARSER.buildVisual( $3 ); }
                    | LP SEE_GLOBAL INT glob_object_list RP
                    { PARSER.buildGlobalVisual( $3 ); }
                    ;

object_list         : /* empty */
                    | object_list object
                    ;

glob_object_list    : /* empty */
                    | glob_object_list glob_object
                    ;

object    : line | flag | goal | ball | player
          ;

glob_object         : glob_goal | glob_ball | glob_player
                    ;

line      : LP linename VAL RP     { PARSER.buildLine( $3 ); }
          | LP linename VAL VAL RP { PARSER.buildLine( $3, $4 ); }
          ;

linename  : LP L location RP { /* nothing */ }
          ;

flag      : LP flagname VAL VAL VAL VAL RP
          { PARSER.buildFlag( $2, $3, $4, $5, $6 ); }
          | LP flagname VAL VAL RP
          { PARSER.buildFlag( $2, $3, $4 ); }
          | LP flagname VAL RP
          { PARSER.buildFlag( $2, $3 ); }
          ;

flagname  : LP F flag_location RP { $$ = false; }
          | LP BF RP              { $$ = true;  }
          ;

flag_location       : location
                    | location location
                    | location INT
                    { PARSER.buildFlagOffset( $2 ); }
                    | location location INT
                    { PARSER.buildFlagOffset( $3 ); }
                    | location location location
                    ;

goal      : LP goalname VAL VAL VAL VAL RP
          { PARSER.buildGoal( $2, $3, $4, $5, $6 ); }
          | LP goalname VAL VAL RP
          { PARSER.buildGoal( $2, $3, $4 ); }
          | LP goalname VAL RP
          { PARSER.buildGoal( $2, $3 ); }
          ;

goalname  : LP G location RP { $$ = false; }
          | LP BG RP         { $$ = true; }
	  ;

glob_goal : LP goalname VAL VAL RP
          { PARSER.buildGlobalGoal( $3, $4 ); }
          ;

ball      : LP ballname VAL VAL VAL VAL RP
          { PARSER.buildBall( $2, $3, $4, $5, $6 ); }
          | LP ballname VAL VAL RP
          { PARSER.buildBall( $2, $3, $4 ); }
          | LP ballname VAL RP
          { PARSER.buildBall( $2, $3 ); }
          ;

glob_ball : LP ballname VAL VAL VAL VAL RP
          { PARSER.buildGlobalBall( $3, $4, $5, $6 ); }
          ;

player    : LP playername VAL VAL VAL VAL VAL VAL extra_info RP
          { PARSER.buildPlayer( $2, $3, $4, $5, $6, $7, $8 ); }
          | LP playername VAL VAL VAL VAL extra_info RP
          { PARSER.buildPlayer( $2, $3, $4, $5, $6 ); }
          | LP playername VAL VAL extra_info RP
          { PARSER.buildPlayer( $2, $3, $4 ); }
          | LP playername VAL RP
          { PARSER.buildPlayer( $2, $3); }
          ;

extra_info          : // nothing
                    | T     { PARSER.buildPlayerExtraInfo( true ); }
                    | VAL   { PARSER.buildPlayerExtraInfo( $1 ); }
                    | VAL T { PARSER.buildPlayerExtraInfo( $1, true ); }
                    ;

glob_player         : LP playername VAL VAL VAL VAL VAL VAL RP
                    { PARSER.buildGlobalPlayer( $3, $4, $5, $6, $7, $8); }
                    ;

playername          : LP BP RP
                    { PARSER.buildPlayerName(); $$ = true; }
                    | LP P RP
                    { PARSER.buildPlayerName(); $$ = false; }
                    | LP P QSTR RP
                    { PARSER.buildPlayerName(stripQuotes( $3 )); $$ = false; }
                    | LP P QSTR INT RP
                    {
                      PARSER.buildPlayerName(stripQuotes( $3 ), $4 );
                      $$ = false;
                    }
                    | LP P QSTR INT GOALIE RP
                    {
                      PARSER.buildPlayerName(stripQuotes( $3 ),
                                             $4, false );
                      $$ = false;
                    }
                    ;

location  : T  { PARSER.buildTopLocation(); }
          | B  { PARSER.buildBottomLocation(); }
          | L  { PARSER.buildLeftLocation(); }
          | R  { PARSER.buildRightLocation(); }
          | C  { PARSER.buildCenterLocation(); }
          | P  { PARSER.buildPenaltyLocation(); }
          | G  { PARSER.buildGoalLocation(); }
          ;


/* ---------------------------- score --------------------------------- */

score_msg : LP SCORE INT INT INT RP
          { PARSER.buildScore( $3, $4, $5); }
          ;


/* --------------------------- fullstate-------------------------------- */

/*(fullstate 0 (pmode before_kick_off) (vmode high normal) (count 0 0 0 0 0 0 0 0)
 (arm (movable 0) (expires 0) (target 0 0) (count 0)) (score 0 0) ((b) 0 0 0 0)
 ((p l 1 g) -47.6 0 0 0 -20.5 0 (stamina 4e+03 1 1))
 ((p l 2 5) -13.4 17 0 0 141 0 (stamina 4e+03 0.846 1))
 ((p r 1 g) 3 -37 0 0 0 0 (stamina 4e+03 1 1))
 )*/
fullstate_msg       : LP FULLSTATE INT fs_pmode fs_viewmode fs_count arm_info fs_score
                    fs_ball fs_player_list RP
                    { PARSER.buildFullState( $3 ); }
                    ;

fs_pmode  : LP PMODE playmode RP
          ;

fs_viewmode         : LP VMODE view_quality view_width RP
		    ;

fs_count  : LP COUNT INT INT INT INT INT INT INT INT RP
          { PARSER.buildFsCount( $3, $4, $5, $6, $7, $8, $9, $10); }
          ;

fs_score  : LP SCORE INT INT RP
          { PARSER.buildFsScore( $3, $4 ); }
          ;

fs_ball   : LP ballname VAL VAL VAL VAL RP
          { PARSER.buildFsBall( $3, $4, $5, $6 ); }
          ;

fs_player_list      : fs_player_list fs_player
                    | fs_player
                    ;

fs_player : LP fs_playername VAL VAL VAL VAL VAL VAL fs_stamina_info RP
          { PARSER.buildFsPlayer( $3, $4, $5, $6, $7, $8 ); }
          ;

fs_playername       : LP P side INT fs_playertype RP
                    { PARSER.buildFsPlayername( $4 ); }
                    ;

fs_playertype       : G   { PARSER.buildFsPlayerType(true); }
                    | INT { PARSER.buildFsPlayerType( $1 ); }
                    ;

fs_stamina_info     : LP STAMINA VAL VAL VAL RP
                    { PARSER.buildFsStaminaInfo( $3, $4, $5 ); }
                    ;

/* ----------------------------- audio ----------------------------------- */

audio_msg : coach_audio
          | ref_audio
          | player_audio
          | self_audio
          ;

coach_audio         : LP HEAR INT coach QSTR RP
                    { PARSER.buildCoachAudio( $3, stripQuotes( $5 ) ); }
                    | LP HEAR INT coach LP FREEFORM QSTR RP RP
                    { PARSER.buildCoachFreeform( stripQuotes( $7 ) ); }
                    ;

coach     : COACH              { PARSER.buildNeutralSide(); }
          | ONLINE_COACH_LEFT  { PARSER.buildLeftSide();    }
          | ONLINE_COACH_RIGHT { PARSER.buildRightSide();   }
          ;

ref_audio : LP HEAR INT REFEREE playmode RP
          { PARSER.buildRefAudio( $3 ); }
          | LP HEAR INT REFEREE penalty RP
          { PARSER.buildRefAudio( $3 ); }
          | LP HEAR INT REFEREE STR RP
          { PARSER.buildUnknownRefAudio( $3, $5 ); }
          ;

penalty : PENALTY_ONFIELD_L { PARSER.buildPenaltySideL(); }
        | PENALTY_ONFIELD_R { PARSER.buildPenaltySideR(); }
        ;

// (hear time direction TEAM [rcssserver-UNUM] "message")
// (hear time TEAM [rcssserver-UNUM])
player_audio        : LP HEAR INT VAL side INT QSTR RP
                    { PARSER.buildPlayerAudio( $3, $4, $6, stripQuotes( $7 ) ); }
                    | LP HEAR INT VAL side QSTR RP
                    { PARSER.buildPlayerAudio( $3, $4, stripQuotes( $6 ) ); }
                    | LP HEAR INT side INT RP
                    { PARSER.buildPlayerAudio( $3, $5 ); }
                    | LP HEAR INT side RP
                    { PARSER.buildPlayerAudio( $3 ); }
                    ;

self_audio          : LP HEAR INT SELF QSTR RP
                    { PARSER.buildSelfAudio( $3, stripQuotes( $5 ) ); }
                    ;


/* ---------------------------- change --------------------------------- */

change_msg          : LP CHANGE INT INT RP
                    { PARSER.buildSubstitution( $3, $4); }
                    | LP CHANGE INT RP
                    { PARSER.buildSubstitution( $3 ); }
                    ;


/* ---------------------------- error --------------------------------- */

error_msg : LP CANT INIT RP
          { PARSER.buildCantReconnect(); }
          | LP ERROR INIT RP
          { PARSER.buildInitError(); }
          | LP ERROR NO_MORE_TEAM_OR_PLAYER_OR_GOALIE RP
          { PARSER.buildNoMoreTeamOrPlayerOrGoalieError(); }
          | LP ERROR NO_SUCH_TEAM_OR_ALREADY_HAVE_COACH	RP
          { PARSER.buildNoSuchTeamOrAlreadyHaveCoachError(); }
          | LP ERROR TOO_MANY_MOVES RP
          { PARSER.buildTooManyMovesError(); }
          | LP ERROR UNKNOWN_COMMAND RP
          { PARSER.buildUnknownCommandError(); }
          | LP ERROR ILLEGAL_COMMAND_FORM RP
          { PARSER.buildIllegalCommandError(); }
          | LP ERROR SAY_MESSAGE_TOO_LONG RP
          { PARSER.buildSayMessageTooLongError(); }
          | LP ERROR ILLEGAL_MODE RP
          { PARSER.buildIllegalModeError(); }
          | LP ERROR ILLEGAL_OBJECT_FORM RP
          { PARSER.buildIllegalObjectFormError(); }
          | LP ERROR SAID_TOO_MANY_FREEFORM_MESSAGES RP
          { PARSER.buildSaidTooManyFreeformMessagesError(); }
          | LP ERROR CANNOT_SAY_FREEFORM_WHILE_PLAYON RP
          { PARSER.buildCannotSayFreeformWhilePlayonError(); }
          | LP ERROR SAID_TOO_MANY_META_MESSAGES RP
          { PARSER.buildSaidTooManyMetaMessagesError(); }
          | LP ERROR SAID_TOO_MANY_ADVICE_MESSAGES RP
          { PARSER.buildSaidTooManyAdviceMessagesError(); }
          | LP ERROR SAID_TOO_MANY_INFO_MESSAGES RP
          { PARSER.buildSaidTooManyInfoMessagesError(); }
          | LP ERROR SAID_TOO_MANY_DEFINE_MESSAGES RP
          { PARSER.buildSaidTooManyDefineMessagesError(); }
          | LP ERROR COULD_NOT_PARSE_SAY RP
          { PARSER.buildCouldNotParseSayError(); }
          | LP ERROR SAID_TOO_MANY_MESSAGES RP
          { PARSER.buildSaidTooManyMessagesError(); }
          | LP ERROR STR RP
          { PARSER.buildUnknownError( $3 ); }
          ;


/* ----------------------------- warning ----------------------------------- */

warning_msg         : LP WARN NO_TEAM_FOUND RP
                    { PARSER.buildNoTeamFoundWarning(); }
                    | LP WARN NO_SUCH_PLAYER RP
                    { PARSER.buildNoSuchPlayerWarning(); }
                    | LP WARN CANNOT_SUB_WHILE_PLAYON RP
                    { PARSER.buildCannotSubWhilePlayOnWarning(); }
                    | LP WARN NO_SUBS_LEFT RP
                    { PARSER.buildNoSubsLeftWarning(); }
                    | LP WARN MAX_OF_THAT_PLAYER_TYPE_ON_FIELD RP
                    { PARSER.buildMaxOfThatPlayerTypeOnFieldWarning(); }
                    | LP WARN CANNOT_CHANGE_GOALIE RP
                    { PARSER.buildCannotChangeGoalieWarning(); }
                    | LP WARN STR RP
                    { PARSER.buildUnknownWarning( $3 ); }
                    ;


/* ----------------------------- ok ----------------------------------- */

ok_msg    : LP OK CLANG_OK LP VER INT INT RP RP
          { PARSER.buildClangVerOK( $6, $7 ); }
          | LP OK EYE ON_OFF RP
          { PARSER.buildEyeOK( $4 ); }
          | LP OK CLANG_OK RP 
          { ; }
          | LP OK STR RP
          { PARSER.buildUnknownOK( $3 ); }
         ;

/* ---------------------------- sync -------------------------------------*/

sync_msg  : LP THINK RP { PARSER.buildThink(); }
          ;

%%

int yyerror(const char *s)
{
  printf("yyerror: \"%s\"\n",s);
  return(0);
}

