#include "CenterBackBehaviour.hh"


/** Creates a Center Back Behaviour and initializes it
    @param leftWing is this the left Center Back or the right one
*/
CenterBackBehaviour::CenterBackBehaviour(bool leftWing) : Behaviour()
{
  left = leftWing;
}

/** Destroys the Center Back Behaviour */
CenterBackBehaviour::~CenterBackBehaviour()
{
}


/** Handles the behaviour if the Agent does not have the Ball (the Ball is not kickable)*/
void CenterBackBehaviour::playOnWithoutBall(){
    // if the player looses the ball while dribbling he would
    // otherwise still think to be dribbling if he gets into the
    // reach of another ball... later... uhm... and would not stop it...
    currentlyDribbling = false;
    
    const Player *fastest = worldModel->fastestInSetToBall(PS_Teammates, true);
    const Player *fastestFromAll = worldModel->fastestInSetToBall(PS_Players, true);
   
  
 
    // iam fastsest, so hurry and get the ball
    if(fastestFromAll != NULL && *fastestFromAll==*myself 
       && worldModel->mayKickBall() ){
      if(skillPool->getBall()){	
	skillPool->stopBall();
	lookAround(true);
      }else{
	lookAround(true);
      }
      return;
      
    } 
  
    /****  we need interceptionPoint for later Calculations  *****/
    static Vector2D intPoint; 
    int cycles;
    if(!worldModel->interceptionPoint(*myself, intPoint, cycles, false))
      intPoint=ball->getPosition();
    
   
    //only fastest of own team   
    if( fastest != NULL && *fastest==*myself
       	&& (fabs(intPoint.getY()) <15
	    || intPoint.distanceTo(Vector2D(-52,0)) < 23 ) ){ //und werde nicht rausgelockt 
      
      if(worldModel->mayKickBall()){
	if(skillPool->getBall()){	
	  skillPool->stopBall();
	  lookAround(true);
	}else{
	  lookAround(true);
	}
	  return;
      }
    }


    //try to stop enemy if he is near goal and iam near Ball(help fastest)
    if(myself->getPosition().distanceTo(intPoint)<4 
       && ball->getPosition().distanceTo(Vector2D(-52,0))<23){

      if(worldModel->mayKickBall()){
	if(skillPool->getBall()){	
	  skillPool->stopBall();
	  lookAround(true);
	}else{
	  lookAround(true);
	}
	return;
      }
    }

    if(myself->getPosition().getX()<-25  && ball->getPosition().getX()<-32
       && (worldModel->hasBallPossessionOpp()
	  )){ 
      coverEnemy();//contains a kind of lookAround
      
    }
    else // doPositioning contains a kind of lookAround
      doPositioning();
    
}



void CenterBackBehaviour::playOnWithBall(){
  //Formation *formation = Formations::getCurrentFormation();
  //int unum = myself->getUniformNumber();
  //int myBehaviour = formation->getBehaviour(unum);

  if (ball->getVelocity().isInvalid()){
    skillPool->holdBall();
    skillPool->faceOnlyNeckToBall();
    return;
  }
  

  // Shoot a goal if possible
  if (tryToScoreGoal()){
    lookAround(true);
    return;
  }

  //get ball out of dangerfull area
  if (worldModel->isAttacked(worldModel->myself) && myself->getPosition().distanceTo(Vector2D(-52,0)) < 18){
    skillPool->kickAway(false, true);
    lookAround(true);
    return;
  }

  if (!skillPool->turnWithBallTo(0)){
    lookAround(true);
    return;
  }


  // * no dibbling at this point *

  // * simple passing routine *

  const Player* passCand = getPassCandidate();
  if (passCand != NULL){
    skillPool->passBallToPlayer(passCand);
    lookAround(true);
    return;
  }


  if (worldModel->isAttacked(worldModel->myself)){
    skillPool->kickAway(false, true);
    lookAround(true);
    return;

  }

  // Try to leave the penalty area as fast as possible with the ball
  if (Field::OWN_PENALTY_AREA.isInside(myself->getPosition())){
    Vector2D dribblePoint = Vector2D(15, 45 * sign(myself->getPosition().getY()), POLAR);
    skillPool->simpleDribbleToPoint(dribblePoint, 4.0f);
    lookAround(false);
    return;
  }


  // This should be done if the ball wouldn't be kickable next turn
  // eg. leave the kickable area
  if (isStopBallNeeded()){
    skillPool->stopBall();
    lookAround(true);
    return;
  }


  skillPool->kickAway(false, true);
  lookAround(true);
  return;
}


void CenterBackBehaviour::coverEnemy(){
  //Test their Players
  static Vector2D coverPoint;
  static Vector2D enemyPos;
  static Player *enemy;

  Vector2D goal=Vector2D(-52,0);
  Vector2D toGoal;
  
  bool needToCover=false;
  vector<Player>::iterator i; 
  vector<Player>::iterator t; 
  bool alreadyMarked=false;


  const Player *fastest = worldModel->fastestInSetToBall(PS_Teammates, true);

  for (i = worldModel->theirTeam.getPlayers().begin(); 
       i < worldModel->theirTeam.getPlayers().end(); 
       ++i) {
    
    if(i->getPosition().distanceTo(Vector2D(-52,0)) < 22 && i->getPosition().getX() <-28 && fabs(i->getPosition().getY()) <15 ){
      if(!i->getConfidence( worldModel->getTime() ) >=0.88)
	continue;

      Vector2D strategicPos=getStrategicPosition();
      if(i->getPosition().distanceTo(strategicPos)>15)
	continue;//nicht quer ueber feld rennen
      if(i->getPosition().getX() < worldModel->getOppOffsideX(-2))
	continue;//keine Spieler decken die Abseits stehen
       if( (const Player *)&(*i)==fastest )
	 continue;// ballfuehrenden Spieler nicht decken 
      if(fabs(i->getPosition().getY())>15)
	continue;// Spieler auf dem Fluegel nicht decken

      int passDistance=(int)( ball->getPosition()-i->getPosition() ).getLength();		                           
      toGoal=(goal - i->getPosition()).setLength(1);//mark also goal	
      alreadyMarked=false;                //no one still needed to be marked

      Vector2D tempCoverPoint= i->getPosition() +(ball->getPosition()  - i->getPosition()).setLength((passDistance%5)*0.5+1)+toGoal;
      
      //Test our Players (do we already cover the markable enenmy)
      for (t = worldModel->myTeam.getPlayers().begin(); 
	   t < worldModel->myTeam.getPlayers().end(); 
	   ++t) {
	if(t->getUniformNumber()==1)
	    continue;
	if(t->isGoodConfident(worldModel->getTime())){
	  
	  if(t->getPosition().distanceTo(tempCoverPoint)<myself->getPosition().distanceTo(tempCoverPoint) 
	     &&  t->getPosition().distanceTo(tempCoverPoint)<3){
	      if (t->getUniformNumber() != myself->getUniformNumber()){ 
		
		alreadyMarked=true;

	      }
	    }
	  }
      }
      
      //this enemy isn`t marked by a Teammate
      if(!alreadyMarked)
	if(!needToCover){// --- new markable enemy found ---
	  coverPoint= tempCoverPoint;
	  needToCover=true;
	  enemyPos=i->getPosition();
	  enemy=&(*i);
	}
	else //is this a more interesting enemy to mark
	  if(tempCoverPoint.distanceTo( getStrategicPosition() ) < coverPoint.distanceTo(getStrategicPosition() ) ){//Strat Pos
	    	 
	    coverPoint= tempCoverPoint;
	    enemyPos=i->getPosition();	  
	    enemy=&(*i);
	  }
    } 
  }
  
  if(needToCover){

    APL_ELLIPSE3_MOSH(coverPoint,0.5,0.5, (APLLevel) 4  , 255, 255, 0);
    if(skillPool->goToPoint(coverPoint,0.4,100)){
   
      if(enemy->getConfidence(worldModel->getTime())>0.90){
	lookAround();
	APL_LINE_MOSH(myself->getPosition(),ball->getPosition());
      }
      
      if(enemy->getConfidence(worldModel->getTime())<=0.90){
	skillPool->faceNeckAndBodyToPoint(enemyPos);    
	APL_LINE_MOSH(myself->getPosition(),enemyPos);
      }
    }

    if(enemy->getConfidence(worldModel->getTime())>0.90){
      APL_LINE_MOSH(myself->getPosition(),ball->getPosition());
      lookAround();
    }
    if(enemy->getConfidence(worldModel->getTime())<=0.90){
        skillPool->faceNeckToPoint(enemyPos);    
	APL_LINE_MOSH(myself->getPosition(),enemyPos);
    }
 
  }   
  else
    doPositioning();
  
}  











