#include "WingAttackerBehaviour.hh"


/** Constructor initializing the Behaviour and calling the super Constructor
    @param left is this the left Wing Attacker or the right Wing Attacker, defaults to left
*/
WingAttackerBehaviour::WingAttackerBehaviour(bool left) : Behaviour()
{
  leftWing = left;
}

/** Destructor doing nothing at the Moment, but this will change*/
WingAttackerBehaviour::~WingAttackerBehaviour()
{
}


/** Handles the Behaviour if the Agent has the Ball (the Ball is kickable)*/
void WingAttackerBehaviour::playOnWithBall(){
  //Formation *formation = Formations::getCurrentFormation();
  //int unum = myself->getUniformNumber();
  //int myBehaviour = formation->getBehaviour(unum);

  // This should be done if the ball wouldnt be kickable next turn
  // eg. leave the kickable area
  if (ball->getVelocity().isInvalid()){
    skillPool->holdBall();
    skillPool->faceOnlyNeckToBall();
    return;
  }

  //DEBUG1_HARLEKIN("not holding ball");
  
  // Shoot a goal if possible
  if (tryToScoreGoal()){
    lookAround(true);
    return;
  }

  //DEBUG1_HARLEKIN("not scoring goal");

  if (considerShouldPass()){
    lookAround(true);
    return;
  }

  //DEBUG1_HARLEKIN("not passing (should)");

  // No passing to be done so try to dribble
  if (considerDribbling()){
    currentlyDribbling = true;
    lookAround(false);
    return;
  }

  if (considerCouldPass()){
    lookAround(true);
    return;
  }

  if (isStopBallNeeded()){
    skillPool->stopBall();
    lookAround(true);
    return;
  }

  skillPool->holdBall();
  lookAround(false);
  return; 
}

/** Returns the Position to which the Wing Attacker should go if nearing thze Midfielder.
    @param the currently calculated Positio, if nearing is needed the position will be changed otherwise
    it will be left untouched
    @todo this is in an early testing Stage DO NOT USE YET !!
*/
void WingAttackerBehaviour::shouldNearMidfielderPosition(Vector2D* position){
  const Player* midfieldAttacker = worldModel->getPlayerByUnum(10);
  if(midfieldAttacker != NULL){
      int minusY, plusY;
      if(myself->getPosition().getX() > 15){
	minusY = -15;
	plusY = 15;
      }else if(myself->getPosition().getX() > 20){
	minusY = -20;
	plusY = 20;
      }else if(myself->getPosition().getX() > 25){
	minusY = -25;
	plusY = 25;
      }else{
	return;
      }
      position->setX(min(midfieldAttacker->getPosition().getX() + 4, worldModel->getOffsideX()));
      if(leftWing){
	position->setY(midfieldAttacker->getPosition().getY() + minusY);
	position->setY(max(position->getY(), -30.0f));
      }else{
	position->setY(midfieldAttacker->getPosition().getY() + plusY);
	position->setY(min(position->getY(), 30.0f));
      }
  }
}


