#include "WingDefenderBehaviour.hh"


/** Creates and initializes the WingDefenderBehaviour
    @param leftWing determines whether this is the Behaviour for the left or right Back
*/
WingDefenderBehaviour::WingDefenderBehaviour(bool leftWing) : Behaviour()
{
  left = leftWing;
}

/** Destroys the Behaviour*/
WingDefenderBehaviour::~WingDefenderBehaviour()
{
}

void WingDefenderBehaviour::playOnWithBall(){
  // The actual behaviour looks like this:
  //    (always top down for every cycle)
  //  1. try to shoot a goal (just to be sure)
  //  3. kickAway if in dangerous situation
  //  3. pass (direct) if possible
  //  4. kickAway IF attacked or close to the midfield
  //  5. turn towards enemy goal
  //  6. stop the ball if needed (disabling fast play)
  //  7. dribble towards midfield

  if (ball->getVelocity().isInvalid()){
    skillPool->holdBall();
    skillPool->faceOnlyNeckToBall();
    return;
  }

  //DEBUG1_HARLEKIN("Not holding ball");
  
  // Shoot a goal if possible, just to be sure
  // This can perhaps happen shortly after a corner kick etc.
  if (tryToScoreGoal()){
    lookAround(true);
    return;
  }
  
  if (worldModel->isAttacked(worldModel->myself) && myself->getPosition().distanceTo(Vector2D(-52,0)) < 18){
    skillPool->kickAway(false, true);
    lookAround(true);
    return;
  }
  
  
  if (!skillPool->turnWithBallTo(0)){
    lookAround(true);
    return;
  }

  //DEBUG1_HARLEKIN("Already facing east");

  //DEBUG1_HARLEKIN("Not scoring goal");

  // * no dibbling *

  const Player* passCand = getPassCandidate();
  if (passCand != NULL){
    skillPool->passBallToPlayer(passCand);
    lookAround(true);
    return;
  }

  //DEBUG1_HARLEKIN("Not passing");

  if (worldModel->isAttacked(*myself) ||
      myself->getPosition().getX() > -5.0){ // do not move too far away
    skillPool->kickAway(false, true);
    lookAround(true);
    return;
  }
  
  if (worldModel->isAttacked(*myself) ||
      myself->getPosition().getX() <= -5.0){ // do not move too far away
    skillPool->kickAway(false, true);
    lookAround(true);
    return;
  }


  //DEBUG1_HARLEKIN("Not attacked and not behind -5.0, y");
    
  // This should be done if the ball wouldnt be kickable next turn
  // eg. leave the kickable area
  if (isStopBallNeeded()){
    skillPool->stopBall();
    lookAround(true);
    return;
  }

  //DEBUG1_HARLEKIN("Stop ball not needed");
    
  skillPool->simpleDribbleToPoint(Vector2D(0,myself->getPosition().getY()), 4.0f);
  lookAround(false);
  return;
}


void WingDefenderBehaviour::playOnWithoutBall(){
    // if the player looses the ball while dribbling he would
    // otherwise still think to be dribbling if he gets into the
    // reach of another ball... later... uhm... and would not stop it...
    currentlyDribbling = false;
    
    const Player *fastest = worldModel->fastestInSetToBall(PS_Teammates, true);
    const Player *fastestFromAll = worldModel->fastestInSetToBall(PS_Players, true);
    // ich bin schnellster also schnapp ich ihn mir
    if(fastestFromAll != NULL && *fastestFromAll==*myself 
       && worldModel->mayKickBall() ){
      APL_ELLIPSE3_MOSH(myself->getPosition(),1.5,1.5, (APLLevel) 4  , 255, 0, 0);
      if(skillPool->getBall()){	
	skillPool->stopBall();
	lookAround(true);
      }else{
	lookAround(true);
      }
      return;
    } 
    
    //ich bin Schnellster aus meinem Team  
    if(fastest != NULL && *fastest==*myself ){ 
      //float dist =myself->getPosition().getX()-ball->getPosition().getX();

      //ich wurde vom Gegner ueberlaufen
      if(ball->getPosition().getX() < myself->getPosition().getX()-2){ 
	if(myself->getPosition().getX()>-38 ){
	  
	  if(myself->getPosition().getY()>0)
	    skillPool->goToPoint(Vector2D(-52,ball->getPosition().getY()/2),1,100);
	  else
	    skillPool->goToPoint(Vector2D(-52,ball->getPosition().getY()/2),1,100);
	  lookAround(true); 
	  return;
	}
	else{
	  if(fabs(ball->getPosition().getY()) 
	     > fabs(myself->getPosition().getY()) )
	    skillPool->goToPoint(Vector2D(-52,myself->getPosition().getY()),1,100);
	  lookAround(true);
	  return;
	}	
      }
      
      //ich wurde noch nicht ueberlaufen, also attackiere ich den Gegner
      if(worldModel->mayKickBall()){
	if(skillPool->getBall()){	
	  skillPool->stopBall();
	  lookAround(true);
	}else{
	  lookAround(true);
	}
	return;
      }
    }

    if(myself->getPosition().getX()<-25  && ball->getPosition().getX()<-32
       && ( worldModel->hasBallPossessionOpp() )  ){
      coverEnemy();// contains a kind of lookAround
    }
    else // doPositioning contains a kind of lookAround
      doPositioning();
}


void WingDefenderBehaviour::coverEnemy(){
  //Test their Players
  static Vector2D coverPoint;
  static Vector2D enemyPos;
  Vector2D goal=Vector2D(-52,0);
  Vector2D toGoal;
  static Player *enemy;
  bool needToCover=false;
  vector<Player>::iterator i; 
  vector<Player>::iterator t; 
  bool alreadyMarked=false;
  
  const Player *fastest = worldModel->fastestInSetToBall(PS_Teammates, true);
  
    for (i = worldModel->theirTeam.getPlayers().begin(); 
       i < worldModel->theirTeam.getPlayers().end(); 
       ++i) {
      if(i->getPosition().getX() < -25 ){
      
      if(i->getPosition().distanceTo(getStrategicPosition())>15)
	continue;//nicht quer ueber feld rennen
      if( (const Player *)&(*i)==fastest )
	continue;// ballfuehrenden Spieler nicht decken
      if(!i->getConfidence(worldModel->getTime()) >=0.88)
	continue;
      if(i->getPosition().getX() < worldModel->getOppOffsideX(-2))
	continue;//Spieler steht Abseits, also nicht decken      
      
      
      int passDistance=(int)( ball->getPosition()-i->getPosition() ).getLength();		
      alreadyMarked=false;
      toGoal=(goal-i->getPosition()).setLength(0.5);		
      
       Vector2D tempCoverPoint= i->getPosition() +(ball->getPosition()  - i->getPosition()).setLength((passDistance%5)*1+1)+toGoal;

       //Test our Players (do we already cover an enenmy)
       for (t = worldModel->myTeam.getPlayers().begin(); 
	    t < worldModel->myTeam.getPlayers().end(); 
	    ++t) {
	 if(t->isGoodConfident(worldModel->getTime())){

	   if(t->getPosition().distanceTo(tempCoverPoint)<3){
	     
	     if (t->getUniformNumber() != myself->getUniformNumber()
		  &&t->getUniformNumber() != 1){ 

	       alreadyMarked=true;
	       
	    }
	  }
	}
      }
       
      
      if(!alreadyMarked) //this enemy is not marked by a Teammate
	if(!needToCover){// --- First markable enemy found ---
	  coverPoint= tempCoverPoint;
	  needToCover=true;
	  enemyPos=i->getPosition();
	  enemy=&(*i);
	}
	else //is this a more interesting enemy to mark
	  if( tempCoverPoint.distanceTo( getStrategicPosition() ) < coverPoint.distanceTo(getStrategicPosition() ) ){//Strat Pos
	    
	    coverPoint= tempCoverPoint;
	    enemyPos=i->getPosition();	  
	    enemy=&(*i);
	  }
      } 
    }
    
    //look only to enemy if he is not confident any more, else look at ball    
    if(needToCover){
      if(skillPool->goToPoint(coverPoint,0.7,100)){
	
	if(enemy->getConfidence(worldModel->getTime())>0.90){
	  lookAround();
	}
	
	if(enemy->getConfidence(worldModel->getTime())<=0.90){
	  skillPool->faceNeckAndBodyToPoint(enemyPos);    
	}
	
      }
      
      if(enemy->getConfidence(worldModel->getTime())>0.90){
	lookAround();
      }
      
      if(enemy->getConfidence(worldModel->getTime())<=0.90){
        skillPool->faceNeckToPoint(enemyPos);    
      }
      
    }   
    else
      doPositioning();
    
}  





