/******************************************************************************
 * MovingObject.cc
 * Copyright (c) 2001 Virtual Werder
 *
 *
 *
 * implements:
 *   class MovingObject
 *
 * requires:
 *
 *
 * Created on: 16.10.2001 by wenns
 * Maintainer: wenns
 * Group:
 * Co-authors:
 *
 * Changelog:
 *
 ******************************************************************************/

#include <algorithm>  //max()

#include "MovingObject.hh"
#include "debug.hh"

MovingObject::MovingObject()
  : position(Vector2D(UNDEFINED_FLOAT, UNDEFINED_FLOAT)),
    velocity(Vector2D(UNDEFINED_FLOAT, UNDEFINED_FLOAT)),
    timeSeen(), timeOfPosition(),
    timeOfVelocity()
{}


MovingObject::MovingObject(const Vector2D &pos, const Vector2D &vel)
  : position(pos), velocity(vel),
    timeSeen(),
    timeOfPosition(), timeOfVelocity()
{}


MovingObject::~MovingObject(){}


float MovingObject::getConfidence(const Time *time) const{
  if( timeSeen.isUndefined() )
    return 0.0;
  
  return max( 0.0, 1.0-(float)(*time - timeSeen).getCycles()/100.0);
}


float MovingObject::getVelConfidence(const Time *time) const{
  if( timeOfVelocity.isUndefined() )
    return 0.0;
  
  return max( 0.0, 1.0-(float)(*time - timeOfVelocity).getCycles()/100.0);
}



Vector2D MovingObject::predictedPosition(int cycles) const {
  if (position.isInvalid() || velocity.isInvalid()){
    DEBUG1_WENNS("[MovingObject::predictedPosition()] \
                 Can't estimate future if don't know present");
    return position;
  }

  Vector2D predPos = position;
  for (int i=0; i < cycles; i++)
    predPos += velocity;

  return predPos;
}

