/******************************************************************************
 * Myself.hh
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   class Myself
 *
 * requires:
 *
 *
 * Created on: 22.08.2001 by wenns
 * Maintainer: harlekin
 * Group:
 * Co-authors:
 *
 * Changelog:
 *
 *****************************************************************************/

#ifndef MYSELF_HH
#define MYSELF_HH

#include "BodyInfo.hh"
#include "Player.hh"
#include "debug.hh"

/**
 * the mental model of his own is a part of the mental model of the world.
 * This class contains all the knowledge of the agent about himself and offers
 * some methods calculating on this data.
 */
class Myself: public Player{
public:
  Myself();
  virtual ~Myself();
  
  Vector2D getVelRelToBody() const { return velRelToBody; }
  const Vector2D *getVelRelToBodyPt() const { return &velRelToBody; }
  void setVelRelToBody(const Vector2D &relVel) { velRelToBody = relVel; }
  float getViewAngle() const;
  float getPowerForDash( const Vector2D &posRelTo, float angBody, 
			 const Vector2D &vel, double dEffort, 
			 float maxPower = 0 );
  float correctDashPowerForStamina(const float dashPower, const float stamina) const;
  
  /**
   * calculates the expected position of the agent on the supposition that
   * the agent will be trying to dash the specified number of cycles with
   * the specified dash power. If a turn angle is specified, turning is
   * assumed to be executed first. Getting tired of the agent (running low
   * of stamina ) is taken into account.
   * @param cycles the position of the agent in so many cycles should be
   * predicted
   * @param dashPower the dash power, that is assumed the agent will be
   * trying to dash while the considered time period
   * @param turnAngle the turn angle, that is assumed the agent will be
   * turning before dashing.
   * @return predicted position of the agent
   */
  Vector2D predictedPosition(int cycles = 1, float dashPower = 0, float turnAngle = 0) const;
  int predictedCyclesToPoint(const Vector2D &point, bool goalie = false) const;
  
  /** stores all available information about the virtual body of the agent
      and the command counter */
  BodyInfo bodyInfo;
  float getEffort() const{ return bodyInfo.getEffort(); }
  float getStamina() const{ return bodyInfo.getStamina(); }
  float getRecovery() const{ return 1.0; } // REVISIT!!! Calculation of Recovery needed
private:
  /**
   * the velocity of the object, relative to the neck angle,
   * in cartesian representation
   * ( but can easily be converted in polar koordinates )
   * scale unit: meter per cycle
   */
  Vector2D velRelToBody;

  /**
   * calculates and updates the values for stamina, effort and recovery of
   * the agent in the next simulation cycle on the supposition of dashing
   * with the specified dashPower in this simulation cycle
   */
  void updatePredictedStamina(float &stamina, float &effort,
			      float &recovery, float dashPower) const;
};

#endif //MYSELF_HH

