/******************************************************************************
 * Time.cc
 * Copyright (c) 2001 Virtual Werder
 *
 * implements:
 *   class Time
 *
 * requires:
 *
 *
 * Created on: 03.02.2003 by wenns
 * Maintainer: wenns
 * Group:
 * Co-authors:
 *
 * Changelog:
 *
 ******************************************************************************/

#include "Time.hh"
#include <algorithm>

Time::Time()
{
  this->setUndefined();
}

Time::Time(int t, int sCount)
  : time(max(t, -1)), stoppedCount(max(sCount, 0))
{}


Time::~Time(){}


void Time::update(int timeStamp){
  if(timeStamp < 0)
    return;                //dont allow "regular" negative times
  if( time == timeStamp )
    stoppedCount++;
  else{
    time = timeStamp;
    stoppedCount = 0;
  }
}


void Time::setUndefined() {
  time = TIME_UNDEFINED;
  stoppedCount = 0;

}

Time Time::operator + (const int i) const {
  if(isUndefined()  ||  (getTime()+i) < 0)
    return getUndefinedTime();
  if (i > 0){
    if (getStoppedCount() == 0)
      return Time(getTime() + i, 0);
    else
      return Time(getTime(), getStoppedCount() + i);
  }else{
    if (getStoppedCount() == 0)
      return Time(getTime() + i, 0);
    else{
      int x = getStoppedCount() + i;
      if (x < 0)
	return Time(getTime() + x, 0);
      else
	return Time(getTime(), x);
    }
  }
}
Time Time::operator + ( const Time &t ) const {
  if(isUndefined() || t.isUndefined() || (getTime() + t.getTime() < 0))
    return getUndefinedTime();
  return Time( getTime() + t.getTime(),
	       max(getStoppedCount(), t.getStoppedCount()) );
}


Time Time::operator - ( const Time &t ) const {
  if(isUndefined() || t.isUndefined() || (time - t.getTime() < 0))
    return getUndefinedTime();
  return Time( time - t.getTime(),
	       max(0, stoppedCount - t.getStoppedCount()) );
}


void Time::operator = ( const Time &t ){
  setTime( t.getTime() );
  setStoppedCount( t.getStoppedCount() );
}


bool Time::operator == ( const int i ) const{
  return (getTime()==i && getStoppedCount()==0);
}
bool Time::operator == ( const Time &t ) const{
  return (getTime()==t.getTime() && getStoppedCount()==t.getStoppedCount());
}


bool Time::operator < (const Time &t) const{
  return (getTime() < t.getTime()) ||
    (getTime()==t.getTime() &&  getStoppedCount() < t.getStoppedCount());
}
bool Time::operator <  ( const int i ) const{
  return (getTime() < i) || (getTime()==i && getStoppedCount() > 0);
}


bool Time::operator <= ( const Time &t ) const{
  return ( *this < t ) || (*this == t );
}
bool Time::operator <= ( const int i ) const{
  return ( *this < i ) || (*this == i );
}
